/* Seamless3d

Copyright (C) 2001-2013 Graham John Perrett (AKA thyme) thyme@seamless3d.com www.seamless3d.com

Permission is hereby granted, free of charge, to any person obtaining a copy of this
software and associated documentation files (the "Software"), to deal in the Software
without restriction, including without limitation the rights to use, copy, modify,
merge, publish, distribute, sublicense, and/or sell copies of the Software, and to
permit persons to whom the Software is furnished to do so, subject to the following
conditions:

The above copyright notice and this permission notice shall be included in all copies or
substantial portions of the Software.

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR IMPLIED,
INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR
PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE
LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT
OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR
OTHER DEALINGS IN THE SOFTWARE.
*/

#include "seamless3d.h"

extern const char* tip_cCLathe_curve;
extern const char* tip_cCLathe_distance;
extern const char* tip_cCLathe_baseRad;
extern const char* tip_cCLathe_autoClose;
const char* tip_stem_add = "add generated triangles to existing triangles";
extern const char* tip_stem_slide;
extern const char* tip_stem_orientation;
extern const char* tip_stem_staves;
extern const char* tip_stem_xDistance;
extern const char* tip_stem_bands;
extern const char* tip_stem_yDistance;
extern const char* tip_stem_height;
extern const char* tip_stem_sync;
extern const char* tip_stem_mould;
extern const char* tip_stem_insideOut;
extern const char* tip_stem_joinEnds;
extern const char* tip_stem_endSP;
extern const char* tip_stem_beginSP;
extern const char* tip_rectangleBuild_staves;
extern const char* tip_rectangleBuild_bands;
extern const char* tip_rectangleBuild_endSP;
extern const char* tip_rectangleBuild_beginSP;

struct InitPromptNode{
	const int* list;
	const int numOf;
	InitPromptNode(const int* list, const int numOf):
		list(list),
		numOf(numOf)
	{}
};
#define INIT_PROMPT_NODE(x) InitPromptNode(x, sizeof(x)/sizeof(int))

class BIField{//built in field
public:
	const int type;
	const char* str;
	const unsigned int flags;
	const InitPromptNode initPromptNode;
	const char* tip;
	BIField(const int type, const char* str, unsigned int flags, const InitPromptNode initPromptNode, const char* tip=0):
		type(type),
		str(str),
		flags(flags),
		initPromptNode(initPromptNode),
		tip(tip)
	{}
	BIField(const int type, const char* str, unsigned int flags, const char* tip=0):
		type(type),
		str(str),
		flags(flags),
		initPromptNode(0,0),
		tip(tip)
	{}
};
class BIProto{//built in proto
public:
	const char* ntype;
	const char* containerField;
	const BIField* bifieldRef;
	const int numOf;
	BIProto(const char* ntype, const char* containerField, const int numOf, const BIField* bifieldRef):
		ntype(ntype),
		containerField(containerField),
		numOf(numOf),
		bifieldRef(bifieldRef)
	{}
};
class BIProto0 : public BIProto{//built in proto
public:
	BIProto0(const char* ntype, const char* containerField):
		BIProto(ntype, containerField, 0, 0)
	{}
};
class BIProto1 : public BIProto{//built in proto
public:
	const BIField bifield0;
	BIProto1(const char* ntype, const char* containerField, BIField bifield0):
		BIProto(ntype, containerField, 1, &this->bifield0),
		bifield0(bifield0)
	{}
};
class BIProto2 : public BIProto{//built in proto
public:
	const BIField bifield0;
	const BIField bifield1;
	BIProto2(const char* ntype, const char* containerField, BIField bifield0, BIField bifield1):
		BIProto(ntype, containerField, 2, &this->bifield0),
		bifield0(bifield0),
		bifield1(bifield1)
	{}
};
class BIProto3 : public BIProto{//built in proto
public:
	const BIField bifield0;
	const BIField bifield1;
	const BIField bifield2;
	BIProto3(const char* ntype, const char* containerField, BIField bifield0, BIField bifield1, BIField bifield2):
		BIProto(ntype, containerField, 3, &this->bifield0),
		bifield0(bifield0),
		bifield1(bifield1),
		bifield2(bifield2)
	{}
};
class BIProto4 : public BIProto{//built in proto
public:
	const BIField bifield0;
	const BIField bifield1;
	const BIField bifield2;
	const BIField bifield3;
	BIProto4(const char* ntype, const char* containerField, BIField bifield0, BIField bifield1, BIField bifield2, BIField bifield3):
		BIProto(ntype, containerField, 4, &this->bifield0),
		bifield0(bifield0),
		bifield1(bifield1),
		bifield2(bifield2),
		bifield3(bifield3)
	{}
};
class BIProto5 : public BIProto{//built in proto
public:
	const BIField bifield0;
	const BIField bifield1;
	const BIField bifield2;
	const BIField bifield3;
	const BIField bifield4;
	const BIField* my_bifieldRef;
	BIProto5(const char* ntype, const char* containerField, BIField bifield0, BIField bifield1, BIField bifield2, BIField bifield3, BIField bifield4):
		BIProto(ntype, containerField, 5, &this->bifield0),
		bifield0(bifield0),
		bifield1(bifield1),
		bifield2(bifield2),
		bifield3(bifield3),
		bifield4(bifield4),
		my_bifieldRef(&this->bifield0)
	{}
};
class BIProto6 : public BIProto{//built in proto
public:
	const BIField bifield0;
	const BIField bifield1;
	const BIField bifield2;
	const BIField bifield3;
	const BIField bifield4;
	const BIField bifield5;
	BIProto6(const char* ntype, const char* containerField, BIField bifield0, BIField bifield1, BIField bifield2, BIField bifield3, BIField bifield4, BIField bifield5):
		BIProto(ntype, containerField, 6, &this->bifield0),
		bifield0(bifield0),
		bifield1(bifield1),
		bifield2(bifield2),
		bifield3(bifield3),
		bifield4(bifield4),
		bifield5(bifield5)
	{}
};
class BIProto7 : public BIProto{//built in proto
public:
	const BIField bifield0;
	const BIField bifield1;
	const BIField bifield2;
	const BIField bifield3;
	const BIField bifield4;
	const BIField bifield5;
	const BIField bifield6;
	BIProto7(const char* ntype, const char* containerField, BIField bifield0, BIField bifield1, BIField bifield2, BIField bifield3, BIField bifield4, BIField bifield5, BIField bifield6):
		BIProto(ntype, containerField, 7, &this->bifield0),
		bifield0(bifield0),
		bifield1(bifield1),
		bifield2(bifield2),
		bifield3(bifield3),
		bifield4(bifield4),
		bifield5(bifield5),
		bifield6(bifield6)
	{}
};
class BIProto8 : public BIProto{//built in proto
public:
	const BIField bifield0;
	const BIField bifield1;
	const BIField bifield2;
	const BIField bifield3;
	const BIField bifield4;
	const BIField bifield5;
	const BIField bifield6;
	const BIField bifield7;
	BIProto8(const char* ntype, const char* containerField, BIField bifield0, BIField bifield1, BIField bifield2, BIField bifield3, BIField bifield4, BIField bifield5, BIField bifield6, BIField bifield7):
		BIProto(ntype, containerField, 8, &this->bifield0),
		bifield0(bifield0),
		bifield1(bifield1),
		bifield2(bifield2),
		bifield3(bifield3),
		bifield4(bifield4),
		bifield5(bifield5),
		bifield6(bifield6),
		bifield7(bifield7)
	{}
};
class BIProto9 : public BIProto{//built in proto
public:
	const BIField bifield0;
	const BIField bifield1;
	const BIField bifield2;
	const BIField bifield3;
	const BIField bifield4;
	const BIField bifield5;
	const BIField bifield6;
	const BIField bifield7;
	const BIField bifield8;
	BIProto9(const char* ntype, const char* containerField, BIField bifield0, BIField bifield1, BIField bifield2, BIField bifield3, BIField bifield4, BIField bifield5, BIField bifield6, BIField bifield7, BIField bifield8):
		BIProto(ntype, containerField, 9, &this->bifield0),
		bifield0(bifield0),
		bifield1(bifield1),
		bifield2(bifield2),
		bifield3(bifield3),
		bifield4(bifield4),
		bifield5(bifield5),
		bifield6(bifield6),
		bifield7(bifield7),
		bifield8(bifield8)
	{}
};
class BIProto10 : public BIProto{//built in proto
public:
	const BIField bifield0;
	const BIField bifield1;
	const BIField bifield2;
	const BIField bifield3;
	const BIField bifield4;
	const BIField bifield5;
	const BIField bifield6;
	const BIField bifield7;
	const BIField bifield8;
	const BIField bifield9;
	BIProto10(const char* ntype, const char* containerField, BIField bifield0, BIField bifield1, BIField bifield2, BIField bifield3, BIField bifield4, BIField bifield5, BIField bifield6, BIField bifield7, BIField bifield8, BIField bifield9):
		BIProto(ntype, containerField, 10, &this->bifield0),
		bifield0(bifield0),
		bifield1(bifield1),
		bifield2(bifield2),
		bifield3(bifield3),
		bifield4(bifield4),
		bifield5(bifield5),
		bifield6(bifield6),
		bifield7(bifield7),
		bifield8(bifield8),
		bifield9(bifield9)
	{}
};
class BIProto11 : public BIProto{//built in proto
public:
	const BIField bifield0;
	const BIField bifield1;
	const BIField bifield2;
	const BIField bifield3;
	const BIField bifield4;
	const BIField bifield5;
	const BIField bifield6;
	const BIField bifield7;
	const BIField bifield8;
	const BIField bifield9;
	const BIField bifield10;
	BIProto11(const char* ntype, const char* containerField, BIField bifield0, BIField bifield1, BIField bifield2, BIField bifield3, BIField bifield4, BIField bifield5, BIField bifield6, BIField bifield7, BIField bifield8, BIField bifield9, BIField bifield10):
		BIProto(ntype, containerField, 11, &this->bifield0),
		bifield0(bifield0),
		bifield1(bifield1),
		bifield2(bifield2),
		bifield3(bifield3),
		bifield4(bifield4),
		bifield5(bifield5),
		bifield6(bifield6),
		bifield7(bifield7),
		bifield8(bifield8),
		bifield9(bifield9),
		bifield10(bifield10)
	{}
};
class BIProto12 : public BIProto{//built in proto
public:
	const BIField bifield0;
	const BIField bifield1;
	const BIField bifield2;
	const BIField bifield3;
	const BIField bifield4;
	const BIField bifield5;
	const BIField bifield6;
	const BIField bifield7;
	const BIField bifield8;
	const BIField bifield9;
	const BIField bifield10;
	const BIField bifield11;
	BIProto12(const char* ntype, const char* containerField, BIField bifield0, BIField bifield1, BIField bifield2, BIField bifield3, BIField bifield4, BIField bifield5, BIField bifield6, BIField bifield7, BIField bifield8, BIField bifield9, BIField bifield10, BIField bifield11):
		BIProto(ntype, containerField, 12, &this->bifield0),
		bifield0(bifield0),
		bifield1(bifield1),
		bifield2(bifield2),
		bifield3(bifield3),
		bifield4(bifield4),
		bifield5(bifield5),
		bifield6(bifield6),
		bifield7(bifield7),
		bifield8(bifield8),
		bifield9(bifield9),
		bifield10(bifield10),
		bifield11(bifield11)
	{}
};
class BIProto13 : public BIProto{//built in proto
public:
	const BIField bifield0;
	const BIField bifield1;
	const BIField bifield2;
	const BIField bifield3;
	const BIField bifield4;
	const BIField bifield5;
	const BIField bifield6;
	const BIField bifield7;
	const BIField bifield8;
	const BIField bifield9;
	const BIField bifield10;
	const BIField bifield11;
	const BIField bifield12;
	BIProto13(const char* ntype, const char* containerField, BIField bifield0, BIField bifield1, BIField bifield2, BIField bifield3, BIField bifield4, BIField bifield5, BIField bifield6, BIField bifield7, BIField bifield8, BIField bifield9, BIField bifield10, BIField bifield11, BIField bifield12):
		BIProto(ntype, containerField, 13, &this->bifield0),
		bifield0(bifield0),
		bifield1(bifield1),
		bifield2(bifield2),
		bifield3(bifield3),
		bifield4(bifield4),
		bifield5(bifield5),
		bifield6(bifield6),
		bifield7(bifield7),
		bifield8(bifield8),
		bifield9(bifield9),
		bifield10(bifield10),
		bifield11(bifield11),
		bifield12(bifield12)
	{}
};
class BIProto14 : public BIProto{//built in proto
public:
	const BIField bifield0;
	const BIField bifield1;
	const BIField bifield2;
	const BIField bifield3;
	const BIField bifield4;
	const BIField bifield5;
	const BIField bifield6;
	const BIField bifield7;
	const BIField bifield8;
	const BIField bifield9;
	const BIField bifield10;
	const BIField bifield11;
	const BIField bifield12;
	const BIField bifield13;
	BIProto14(const char* ntype, const char* containerField, BIField bifield0, BIField bifield1, BIField bifield2, BIField bifield3, BIField bifield4, BIField bifield5, BIField bifield6, BIField bifield7, BIField bifield8, BIField bifield9, BIField bifield10, BIField bifield11, BIField bifield12, BIField bifield13):
		BIProto(ntype, containerField, 14, &this->bifield0),
		bifield0(bifield0),
		bifield1(bifield1),
		bifield2(bifield2),
		bifield3(bifield3),
		bifield4(bifield4),
		bifield5(bifield5),
		bifield6(bifield6),
		bifield7(bifield7),
		bifield8(bifield8),
		bifield9(bifield9),
		bifield10(bifield10),
		bifield11(bifield11),
		bifield12(bifield12),
		bifield13(bifield13)
	{}
};
class BIProto15 : public BIProto{//built in proto
public:
	const BIField bifield0;
	const BIField bifield1;
	const BIField bifield2;
	const BIField bifield3;
	const BIField bifield4;
	const BIField bifield5;
	const BIField bifield6;
	const BIField bifield7;
	const BIField bifield8;
	const BIField bifield9;
	const BIField bifield10;
	const BIField bifield11;
	const BIField bifield12;
	const BIField bifield13;
	const BIField bifield14;
	BIProto15(const char* ntype, const char* containerField, BIField bifield0, BIField bifield1, BIField bifield2, BIField bifield3, BIField bifield4, BIField bifield5, BIField bifield6, BIField bifield7, BIField bifield8, BIField bifield9, BIField bifield10, BIField bifield11, BIField bifield12, BIField bifield13, BIField bifield14):
		BIProto(ntype, containerField, 15, &this->bifield0),
		bifield0(bifield0),
		bifield1(bifield1),
		bifield2(bifield2),
		bifield3(bifield3),
		bifield4(bifield4),
		bifield5(bifield5),
		bifield6(bifield6),
		bifield7(bifield7),
		bifield8(bifield8),
		bifield9(bifield9),
		bifield10(bifield10),
		bifield11(bifield11),
		bifield12(bifield12),
		bifield13(bifield13),
		bifield14(bifield14)
	{}
};
class BIProto16 : public BIProto{//built in proto
public:
	const BIField bifield0;
	const BIField bifield1;
	const BIField bifield2;
	const BIField bifield3;
	const BIField bifield4;
	const BIField bifield5;
	const BIField bifield6;
	const BIField bifield7;
	const BIField bifield8;
	const BIField bifield9;
	const BIField bifield10;
	const BIField bifield11;
	const BIField bifield12;
	const BIField bifield13;
	const BIField bifield14;
	const BIField bifield15;
	BIProto16(const char* ntype, const char* containerField, BIField bifield0, BIField bifield1, BIField bifield2, BIField bifield3, BIField bifield4, BIField bifield5, BIField bifield6, BIField bifield7, BIField bifield8, BIField bifield9, BIField bifield10, BIField bifield11, BIField bifield12, BIField bifield13, BIField bifield14, BIField bifield15):
		BIProto(ntype, containerField, 16, &this->bifield0),
		bifield0(bifield0),
		bifield1(bifield1),
		bifield2(bifield2),
		bifield3(bifield3),
		bifield4(bifield4),
		bifield5(bifield5),
		bifield6(bifield6),
		bifield7(bifield7),
		bifield8(bifield8),
		bifield9(bifield9),
		bifield10(bifield10),
		bifield11(bifield11),
		bifield12(bifield12),
		bifield13(bifield13),
		bifield14(bifield14),
		bifield15(bifield15)
	{}
};
class BIProto17 : public BIProto{//built in proto
public:
	const BIField bifield0;
	const BIField bifield1;
	const BIField bifield2;
	const BIField bifield3;
	const BIField bifield4;
	const BIField bifield5;
	const BIField bifield6;
	const BIField bifield7;
	const BIField bifield8;
	const BIField bifield9;
	const BIField bifield10;
	const BIField bifield11;
	const BIField bifield12;
	const BIField bifield13;
	const BIField bifield14;
	const BIField bifield15;
	const BIField bifield16;
	BIProto17(const char* ntype, const char* containerField, BIField bifield0, BIField bifield1, BIField bifield2, BIField bifield3, BIField bifield4, BIField bifield5, BIField bifield6, BIField bifield7, BIField bifield8, BIField bifield9, BIField bifield10, BIField bifield11, BIField bifield12, BIField bifield13, BIField bifield14, BIField bifield15, BIField bifield16):
		BIProto(ntype, containerField, 17, &this->bifield0),
		bifield0(bifield0),
		bifield1(bifield1),
		bifield2(bifield2),
		bifield3(bifield3),
		bifield4(bifield4),
		bifield5(bifield5),
		bifield6(bifield6),
		bifield7(bifield7),
		bifield8(bifield8),
		bifield9(bifield9),
		bifield10(bifield10),
		bifield11(bifield11),
		bifield12(bifield12),
		bifield13(bifield13),
		bifield14(bifield14),
		bifield15(bifield15),
		bifield16(bifield16)
	{}
};
class BIProto18 : public BIProto{//built in proto
public:
	const BIField bifield0;
	const BIField bifield1;
	const BIField bifield2;
	const BIField bifield3;
	const BIField bifield4;
	const BIField bifield5;
	const BIField bifield6;
	const BIField bifield7;
	const BIField bifield8;
	const BIField bifield9;
	const BIField bifield10;
	const BIField bifield11;
	const BIField bifield12;
	const BIField bifield13;
	const BIField bifield14;
	const BIField bifield15;
	const BIField bifield16;
	const BIField bifield17;
	BIProto18(const char* ntype, const char* containerField, BIField bifield0, BIField bifield1, BIField bifield2, BIField bifield3, BIField bifield4, BIField bifield5, BIField bifield6, BIField bifield7, BIField bifield8, BIField bifield9, BIField bifield10, BIField bifield11, BIField bifield12, BIField bifield13, BIField bifield14, BIField bifield15, BIField bifield16, BIField bifield17):
		BIProto(ntype, containerField, 18, &this->bifield0),
		bifield0(bifield0),
		bifield1(bifield1),
		bifield2(bifield2),
		bifield3(bifield3),
		bifield4(bifield4),
		bifield5(bifield5),
		bifield6(bifield6),
		bifield7(bifield7),
		bifield8(bifield8),
		bifield9(bifield9),
		bifield10(bifield10),
		bifield11(bifield11),
		bifield12(bifield12),
		bifield13(bifield13),
		bifield14(bifield14),
		bifield15(bifield15),
		bifield16(bifield16),
		bifield17(bifield17)
	{}
};
class BIProto19 : public BIProto{//built in proto
public:
	const BIField bifield0;
	const BIField bifield1;
	const BIField bifield2;
	const BIField bifield3;
	const BIField bifield4;
	const BIField bifield5;
	const BIField bifield6;
	const BIField bifield7;
	const BIField bifield8;
	const BIField bifield9;
	const BIField bifield10;
	const BIField bifield11;
	const BIField bifield12;
	const BIField bifield13;
	const BIField bifield14;
	const BIField bifield15;
	const BIField bifield16;
	const BIField bifield17;
	const BIField bifield18;
	BIProto19(const char* ntype, const char* containerField, BIField bifield0, BIField bifield1, BIField bifield2, BIField bifield3, BIField bifield4, BIField bifield5, BIField bifield6, BIField bifield7, BIField bifield8, BIField bifield9, BIField bifield10, BIField bifield11, BIField bifield12, BIField bifield13, BIField bifield14, BIField bifield15, BIField bifield16, BIField bifield17, BIField bifield18):
		BIProto(ntype, containerField, 19, &this->bifield0),
		bifield0(bifield0),
		bifield1(bifield1),
		bifield2(bifield2),
		bifield3(bifield3),
		bifield4(bifield4),
		bifield5(bifield5),
		bifield6(bifield6),
		bifield7(bifield7),
		bifield8(bifield8),
		bifield9(bifield9),
		bifield10(bifield10),
		bifield11(bifield11),
		bifield12(bifield12),
		bifield13(bifield13),
		bifield14(bifield14),
		bifield15(bifield15),
		bifield16(bifield16),
		bifield17(bifield17),
		bifield18(bifield18)
	{}
};
class BIProto20 : public BIProto{//built in proto
public:
	const BIField bifield0;
	const BIField bifield1;
	const BIField bifield2;
	const BIField bifield3;
	const BIField bifield4;
	const BIField bifield5;
	const BIField bifield6;
	const BIField bifield7;
	const BIField bifield8;
	const BIField bifield9;
	const BIField bifield10;
	const BIField bifield11;
	const BIField bifield12;
	const BIField bifield13;
	const BIField bifield14;
	const BIField bifield15;
	const BIField bifield16;
	const BIField bifield17;
	const BIField bifield18;
	const BIField bifield19;
	BIProto20(const char* ntype, const char* containerField, BIField bifield0, BIField bifield1, BIField bifield2, BIField bifield3, BIField bifield4, BIField bifield5, BIField bifield6, BIField bifield7, BIField bifield8, BIField bifield9, BIField bifield10, BIField bifield11, BIField bifield12, BIField bifield13, BIField bifield14, BIField bifield15, BIField bifield16, BIField bifield17, BIField bifield18, BIField bifield19):
		BIProto(ntype, containerField, 20, &this->bifield0),
		bifield0(bifield0),
		bifield1(bifield1),
		bifield2(bifield2),
		bifield3(bifield3),
		bifield4(bifield4),
		bifield5(bifield5),
		bifield6(bifield6),
		bifield7(bifield7),
		bifield8(bifield8),
		bifield9(bifield9),
		bifield10(bifield10),
		bifield11(bifield11),
		bifield12(bifield12),
		bifield13(bifield13),
		bifield14(bifield14),
		bifield15(bifield15),
		bifield16(bifield16),
		bifield17(bifield17),
		bifield18(bifield18),
		bifield19(bifield19)
	{}
};
class BIProto21 : public BIProto{//built in proto
public:
	const BIField bifield0;
	const BIField bifield1;
	const BIField bifield2;
	const BIField bifield3;
	const BIField bifield4;
	const BIField bifield5;
	const BIField bifield6;
	const BIField bifield7;
	const BIField bifield8;
	const BIField bifield9;
	const BIField bifield10;
	const BIField bifield11;
	const BIField bifield12;
	const BIField bifield13;
	const BIField bifield14;
	const BIField bifield15;
	const BIField bifield16;
	const BIField bifield17;
	const BIField bifield18;
	const BIField bifield19;
	const BIField bifield20;
	BIProto21(const char* ntype, const char* containerField, BIField bifield0, BIField bifield1, BIField bifield2, BIField bifield3, BIField bifield4, BIField bifield5, BIField bifield6, BIField bifield7, BIField bifield8, BIField bifield9, BIField bifield10, BIField bifield11, BIField bifield12, BIField bifield13, BIField bifield14, BIField bifield15, BIField bifield16, BIField bifield17, BIField bifield18, BIField bifield19, BIField bifield20):
		BIProto(ntype, containerField, 21, &this->bifield0),
		bifield0(bifield0),
		bifield1(bifield1),
		bifield2(bifield2),
		bifield3(bifield3),
		bifield4(bifield4),
		bifield5(bifield5),
		bifield6(bifield6),
		bifield7(bifield7),
		bifield8(bifield8),
		bifield9(bifield9),
		bifield10(bifield10),
		bifield11(bifield11),
		bifield12(bifield12),
		bifield13(bifield13),
		bifield14(bifield14),
		bifield15(bifield15),
		bifield16(bifield16),
		bifield17(bifield17),
		bifield18(bifield18),
		bifield19(bifield19),
		bifield20(bifield20)
	{}
};
class BIProto22 : public BIProto{//built in proto
public:
	const BIField bifield0;
	const BIField bifield1;
	const BIField bifield2;
	const BIField bifield3;
	const BIField bifield4;
	const BIField bifield5;
	const BIField bifield6;
	const BIField bifield7;
	const BIField bifield8;
	const BIField bifield9;
	const BIField bifield10;
	const BIField bifield11;
	const BIField bifield12;
	const BIField bifield13;
	const BIField bifield14;
	const BIField bifield15;
	const BIField bifield16;
	const BIField bifield17;
	const BIField bifield18;
	const BIField bifield19;
	const BIField bifield20;
	const BIField bifield21;
	BIProto22(const char* ntype, const char* containerField, BIField bifield0, BIField bifield1, BIField bifield2, BIField bifield3, BIField bifield4, BIField bifield5, BIField bifield6, BIField bifield7, BIField bifield8, BIField bifield9, BIField bifield10, BIField bifield11, BIField bifield12, BIField bifield13, BIField bifield14, BIField bifield15, BIField bifield16, BIField bifield17, BIField bifield18, BIField bifield19, BIField bifield20, BIField bifield21):
		BIProto(ntype, containerField, 22, &this->bifield0),
		bifield0(bifield0),
		bifield1(bifield1),
		bifield2(bifield2),
		bifield3(bifield3),
		bifield4(bifield4),
		bifield5(bifield5),
		bifield6(bifield6),
		bifield7(bifield7),
		bifield8(bifield8),
		bifield9(bifield9),
		bifield10(bifield10),
		bifield11(bifield11),
		bifield12(bifield12),
		bifield13(bifield13),
		bifield14(bifield14),
		bifield15(bifield15),
		bifield16(bifield16),
		bifield17(bifield17),
		bifield18(bifield18),
		bifield19(bifield19),
		bifield20(bifield20),
		bifield21(bifield21)
	{}
};
class BIProto23 : public BIProto{//built in proto
public:
	const BIField bifield0;
	const BIField bifield1;
	const BIField bifield2;
	const BIField bifield3;
	const BIField bifield4;
	const BIField bifield5;
	const BIField bifield6;
	const BIField bifield7;
	const BIField bifield8;
	const BIField bifield9;
	const BIField bifield10;
	const BIField bifield11;
	const BIField bifield12;
	const BIField bifield13;
	const BIField bifield14;
	const BIField bifield15;
	const BIField bifield16;
	const BIField bifield17;
	const BIField bifield18;
	const BIField bifield19;
	const BIField bifield20;
	const BIField bifield21;
	const BIField bifield22;
	BIProto23(const char* ntype, const char* containerField, BIField bifield0, BIField bifield1, BIField bifield2, BIField bifield3, BIField bifield4, BIField bifield5, BIField bifield6, BIField bifield7, BIField bifield8, BIField bifield9, BIField bifield10, BIField bifield11, BIField bifield12, BIField bifield13, BIField bifield14, BIField bifield15, BIField bifield16, BIField bifield17, BIField bifield18, BIField bifield19, BIField bifield20, BIField bifield21, BIField bifield22):
		BIProto(ntype, containerField, 23, &this->bifield0),
		bifield0(bifield0),
		bifield1(bifield1),
		bifield2(bifield2),
		bifield3(bifield3),
		bifield4(bifield4),
		bifield5(bifield5),
		bifield6(bifield6),
		bifield7(bifield7),
		bifield8(bifield8),
		bifield9(bifield9),
		bifield10(bifield10),
		bifield11(bifield11),
		bifield12(bifield12),
		bifield13(bifield13),
		bifield14(bifield14),
		bifield15(bifield15),
		bifield16(bifield16),
		bifield17(bifield17),
		bifield18(bifield18),
		bifield19(bifield19),
		bifield20(bifield20),
		bifield21(bifield21),
		bifield22(bifield22)
	{}
};
class BIProto24 : public BIProto{//built in proto
public:
	const BIField bifield0;
	const BIField bifield1;
	const BIField bifield2;
	const BIField bifield3;
	const BIField bifield4;
	const BIField bifield5;
	const BIField bifield6;
	const BIField bifield7;
	const BIField bifield8;
	const BIField bifield9;
	const BIField bifield10;
	const BIField bifield11;
	const BIField bifield12;
	const BIField bifield13;
	const BIField bifield14;
	const BIField bifield15;
	const BIField bifield16;
	const BIField bifield17;
	const BIField bifield18;
	const BIField bifield19;
	const BIField bifield20;
	const BIField bifield21;
	const BIField bifield22;
	const BIField bifield23;
	BIProto24(const char* ntype, const char* containerField, BIField bifield0, BIField bifield1, BIField bifield2, BIField bifield3, BIField bifield4, BIField bifield5, BIField bifield6, BIField bifield7, BIField bifield8, BIField bifield9, BIField bifield10, BIField bifield11, BIField bifield12, BIField bifield13, BIField bifield14, BIField bifield15, BIField bifield16, BIField bifield17, BIField bifield18, BIField bifield19, BIField bifield20, BIField bifield21, BIField bifield22, BIField bifield23):
		BIProto(ntype, containerField, 24, &this->bifield0),
		bifield0(bifield0),
		bifield1(bifield1),
		bifield2(bifield2),
		bifield3(bifield3),
		bifield4(bifield4),
		bifield5(bifield5),
		bifield6(bifield6),
		bifield7(bifield7),
		bifield8(bifield8),
		bifield9(bifield9),
		bifield10(bifield10),
		bifield11(bifield11),
		bifield12(bifield12),
		bifield13(bifield13),
		bifield14(bifield14),
		bifield15(bifield15),
		bifield16(bifield16),
		bifield17(bifield17),
		bifield18(bifield18),
		bifield19(bifield19),
		bifield20(bifield20),
		bifield21(bifield21),
		bifield22(bifield22),
		bifield23(bifield23)
	{}
};
class BIProto25 : public BIProto{//built in proto
public:
	const BIField bifield0;
	const BIField bifield1;
	const BIField bifield2;
	const BIField bifield3;
	const BIField bifield4;
	const BIField bifield5;
	const BIField bifield6;
	const BIField bifield7;
	const BIField bifield8;
	const BIField bifield9;
	const BIField bifield10;
	const BIField bifield11;
	const BIField bifield12;
	const BIField bifield13;
	const BIField bifield14;
	const BIField bifield15;
	const BIField bifield16;
	const BIField bifield17;
	const BIField bifield18;
	const BIField bifield19;
	const BIField bifield20;
	const BIField bifield21;
	const BIField bifield22;
	const BIField bifield23;
	const BIField bifield24;
	BIProto25(const char* ntype, const char* containerField, BIField bifield0, BIField bifield1, BIField bifield2, BIField bifield3, BIField bifield4, BIField bifield5, BIField bifield6, BIField bifield7, BIField bifield8, BIField bifield9, BIField bifield10, BIField bifield11, BIField bifield12, BIField bifield13, BIField bifield14, BIField bifield15, BIField bifield16, BIField bifield17, BIField bifield18, BIField bifield19, BIField bifield20, BIField bifield21, BIField bifield22, BIField bifield23, BIField bifield24):
		BIProto(ntype, containerField, 25, &this->bifield0),
		bifield0(bifield0),
		bifield1(bifield1),
		bifield2(bifield2),
		bifield3(bifield3),
		bifield4(bifield4),
		bifield5(bifield5),
		bifield6(bifield6),
		bifield7(bifield7),
		bifield8(bifield8),
		bifield9(bifield9),
		bifield10(bifield10),
		bifield11(bifield11),
		bifield12(bifield12),
		bifield13(bifield13),
		bifield14(bifield14),
		bifield15(bifield15),
		bifield16(bifield16),
		bifield17(bifield17),
		bifield18(bifield18),
		bifield19(bifield19),
		bifield20(bifield20),
		bifield21(bifield21),
		bifield22(bifield22),
		bifield23(bifield23),
		bifield24(bifield24)
	{}
};

class BIProto26 : public BIProto{//built in proto
public:
	const BIField bifield0;
	const BIField bifield1;
	const BIField bifield2;
	const BIField bifield3;
	const BIField bifield4;
	const BIField bifield5;
	const BIField bifield6;
	const BIField bifield7;
	const BIField bifield8;
	const BIField bifield9;
	const BIField bifield10;
	const BIField bifield11;
	const BIField bifield12;
	const BIField bifield13;
	const BIField bifield14;
	const BIField bifield15;
	const BIField bifield16;
	const BIField bifield17;
	const BIField bifield18;
	const BIField bifield19;
	const BIField bifield20;
	const BIField bifield21;
	const BIField bifield22;
	const BIField bifield23;
	const BIField bifield24;
	const BIField bifield25;
	BIProto26(const char* ntype, const char* containerField, BIField bifield0, BIField bifield1, BIField bifield2, BIField bifield3, BIField bifield4, BIField bifield5, BIField bifield6, BIField bifield7, BIField bifield8, BIField bifield9, BIField bifield10, BIField bifield11, BIField bifield12, BIField bifield13, BIField bifield14, BIField bifield15, BIField bifield16, BIField bifield17, BIField bifield18, BIField bifield19, BIField bifield20, BIField bifield21, BIField bifield22, BIField bifield23, BIField bifield24, BIField bifield25):
		BIProto(ntype, containerField, 26, &this->bifield0),
		bifield0(bifield0),
		bifield1(bifield1),
		bifield2(bifield2),
		bifield3(bifield3),
		bifield4(bifield4),
		bifield5(bifield5),
		bifield6(bifield6),
		bifield7(bifield7),
		bifield8(bifield8),
		bifield9(bifield9),
		bifield10(bifield10),
		bifield11(bifield11),
		bifield12(bifield12),
		bifield13(bifield13),
		bifield14(bifield14),
		bifield15(bifield15),
		bifield16(bifield16),
		bifield17(bifield17),
		bifield18(bifield18),
		bifield19(bifield19),
		bifield20(bifield20),
		bifield21(bifield21),
		bifield22(bifield22),
		bifield23(bifield23),
		bifield24(bifield24),
		bifield25(bifield25)
	{}
};
class BIProto27 : public BIProto{//built in proto
public:
	const BIField bifield0;
	const BIField bifield1;
	const BIField bifield2;
	const BIField bifield3;
	const BIField bifield4;
	const BIField bifield5;
	const BIField bifield6;
	const BIField bifield7;
	const BIField bifield8;
	const BIField bifield9;
	const BIField bifield10;
	const BIField bifield11;
	const BIField bifield12;
	const BIField bifield13;
	const BIField bifield14;
	const BIField bifield15;
	const BIField bifield16;
	const BIField bifield17;
	const BIField bifield18;
	const BIField bifield19;
	const BIField bifield20;
	const BIField bifield21;
	const BIField bifield22;
	const BIField bifield23;
	const BIField bifield24;
	const BIField bifield25;
	const BIField bifield26;
	BIProto27(const char* ntype, const char* containerField, BIField bifield0, BIField bifield1, BIField bifield2, BIField bifield3, BIField bifield4, BIField bifield5, BIField bifield6, BIField bifield7, BIField bifield8, BIField bifield9, BIField bifield10, BIField bifield11, BIField bifield12, BIField bifield13, BIField bifield14, BIField bifield15, BIField bifield16, BIField bifield17, BIField bifield18, BIField bifield19, BIField bifield20, BIField bifield21, BIField bifield22, BIField bifield23, BIField bifield24, BIField bifield25, BIField bifield26):
		BIProto(ntype, containerField, 27, &this->bifield0),
		bifield0(bifield0),
		bifield1(bifield1),
		bifield2(bifield2),
		bifield3(bifield3),
		bifield4(bifield4),
		bifield5(bifield5),
		bifield6(bifield6),
		bifield7(bifield7),
		bifield8(bifield8),
		bifield9(bifield9),
		bifield10(bifield10),
		bifield11(bifield11),
		bifield12(bifield12),
		bifield13(bifield13),
		bifield14(bifield14),
		bifield15(bifield15),
		bifield16(bifield16),
		bifield17(bifield17),
		bifield18(bifield18),
		bifield19(bifield19),
		bifield20(bifield20),
		bifield21(bifield21),
		bifield22(bifield22),
		bifield23(bifield23),
		bifield24(bifield24),
		bifield25(bifield25),
		bifield26(bifield26)
	{}
};
class BIProto28 : public BIProto{//built in proto
public:
	const BIField bifield0;
	const BIField bifield1;
	const BIField bifield2;
	const BIField bifield3;
	const BIField bifield4;
	const BIField bifield5;
	const BIField bifield6;
	const BIField bifield7;
	const BIField bifield8;
	const BIField bifield9;
	const BIField bifield10;
	const BIField bifield11;
	const BIField bifield12;
	const BIField bifield13;
	const BIField bifield14;
	const BIField bifield15;
	const BIField bifield16;
	const BIField bifield17;
	const BIField bifield18;
	const BIField bifield19;
	const BIField bifield20;
	const BIField bifield21;
	const BIField bifield22;
	const BIField bifield23;
	const BIField bifield24;
	const BIField bifield25;
	const BIField bifield26;
	const BIField bifield27;
	BIProto28(const char* ntype, const char* containerField, BIField bifield0, BIField bifield1, BIField bifield2, BIField bifield3, BIField bifield4, BIField bifield5, BIField bifield6, BIField bifield7, BIField bifield8, BIField bifield9, BIField bifield10, BIField bifield11, BIField bifield12, BIField bifield13, BIField bifield14, BIField bifield15, BIField bifield16, BIField bifield17, BIField bifield18, BIField bifield19, BIField bifield20, BIField bifield21, BIField bifield22, BIField bifield23, BIField bifield24, BIField bifield25, BIField bifield26, BIField bifield27):
		BIProto(ntype, containerField, 28, &this->bifield0),
		bifield0(bifield0),
		bifield1(bifield1),
		bifield2(bifield2),
		bifield3(bifield3),
		bifield4(bifield4),
		bifield5(bifield5),
		bifield6(bifield6),
		bifield7(bifield7),
		bifield8(bifield8),
		bifield9(bifield9),
		bifield10(bifield10),
		bifield11(bifield11),
		bifield12(bifield12),
		bifield13(bifield13),
		bifield14(bifield14),
		bifield15(bifield15),
		bifield16(bifield16),
		bifield17(bifield17),
		bifield18(bifield18),
		bifield19(bifield19),
		bifield20(bifield20),
		bifield21(bifield21),
		bifield22(bifield22),
		bifield23(bifield23),
		bifield24(bifield24),
		bifield25(bifield25),
		bifield26(bifield26),
		bifield27(bifield27)
	{}
};
class BIProto29 : public BIProto{//built in proto
public:
	const BIField bifield0;
	const BIField bifield1;
	const BIField bifield2;
	const BIField bifield3;
	const BIField bifield4;
	const BIField bifield5;
	const BIField bifield6;
	const BIField bifield7;
	const BIField bifield8;
	const BIField bifield9;
	const BIField bifield10;
	const BIField bifield11;
	const BIField bifield12;
	const BIField bifield13;
	const BIField bifield14;
	const BIField bifield15;
	const BIField bifield16;
	const BIField bifield17;
	const BIField bifield18;
	const BIField bifield19;
	const BIField bifield20;
	const BIField bifield21;
	const BIField bifield22;
	const BIField bifield23;
	const BIField bifield24;
	const BIField bifield25;
	const BIField bifield26;
	const BIField bifield27;
	const BIField bifield28;
	BIProto29(const char* ntype, const char* containerField, BIField bifield0, BIField bifield1, BIField bifield2, BIField bifield3, BIField bifield4, BIField bifield5, BIField bifield6, BIField bifield7, BIField bifield8, BIField bifield9, BIField bifield10, BIField bifield11, BIField bifield12, BIField bifield13, BIField bifield14, BIField bifield15, BIField bifield16, BIField bifield17, BIField bifield18, BIField bifield19, BIField bifield20, BIField bifield21, BIField bifield22, BIField bifield23, BIField bifield24, BIField bifield25, BIField bifield26, BIField bifield27, BIField bifield28):
		BIProto(ntype, containerField, 29, &this->bifield0),
		bifield0(bifield0),
		bifield1(bifield1),
		bifield2(bifield2),
		bifield3(bifield3),
		bifield4(bifield4),
		bifield5(bifield5),
		bifield6(bifield6),
		bifield7(bifield7),
		bifield8(bifield8),
		bifield9(bifield9),
		bifield10(bifield10),
		bifield11(bifield11),
		bifield12(bifield12),
		bifield13(bifield13),
		bifield14(bifield14),
		bifield15(bifield15),
		bifield16(bifield16),
		bifield17(bifield17),
		bifield18(bifield18),
		bifield19(bifield19),
		bifield20(bifield20),
		bifield21(bifield21),
		bifield22(bifield22),
		bifield23(bifield23),
		bifield24(bifield24),
		bifield25(bifield25),
		bifield26(bifield26),
		bifield27(bifield27),
		bifield28(bifield28)
	{}
};
class BIProto30 : public BIProto{//built in proto
public:
	const BIField bifield0;
	const BIField bifield1;
	const BIField bifield2;
	const BIField bifield3;
	const BIField bifield4;
	const BIField bifield5;
	const BIField bifield6;
	const BIField bifield7;
	const BIField bifield8;
	const BIField bifield9;
	const BIField bifield10;
	const BIField bifield11;
	const BIField bifield12;
	const BIField bifield13;
	const BIField bifield14;
	const BIField bifield15;
	const BIField bifield16;
	const BIField bifield17;
	const BIField bifield18;
	const BIField bifield19;
	const BIField bifield20;
	const BIField bifield21;
	const BIField bifield22;
	const BIField bifield23;
	const BIField bifield24;
	const BIField bifield25;
	const BIField bifield26;
	const BIField bifield27;
	const BIField bifield28;
	const BIField bifield29;
	BIProto30(const char* ntype, const char* containerField, BIField bifield0, BIField bifield1, BIField bifield2, BIField bifield3, BIField bifield4, BIField bifield5, BIField bifield6, BIField bifield7, BIField bifield8, BIField bifield9, BIField bifield10, BIField bifield11, BIField bifield12, BIField bifield13, BIField bifield14, BIField bifield15, BIField bifield16, BIField bifield17, BIField bifield18, BIField bifield19, BIField bifield20, BIField bifield21, BIField bifield22, BIField bifield23, BIField bifield24, BIField bifield25, BIField bifield26, BIField bifield27, BIField bifield28, BIField bifield29):
		BIProto(ntype, containerField, 30, &this->bifield0),
		bifield0(bifield0),
		bifield1(bifield1),
		bifield2(bifield2),
		bifield3(bifield3),
		bifield4(bifield4),
		bifield5(bifield5),
		bifield6(bifield6),
		bifield7(bifield7),
		bifield8(bifield8),
		bifield9(bifield9),
		bifield10(bifield10),
		bifield11(bifield11),
		bifield12(bifield12),
		bifield13(bifield13),
		bifield14(bifield14),
		bifield15(bifield15),
		bifield16(bifield16),
		bifield17(bifield17),
		bifield18(bifield18),
		bifield19(bifield19),
		bifield20(bifield20),
		bifield21(bifield21),
		bifield22(bifield22),
		bifield23(bifield23),
		bifield24(bifield24),
		bifield25(bifield25),
		bifield26(bifield26),
		bifield27(bifield27),
		bifield28(bifield28),
		bifield29(bifield29)
	{}
};
class BIProto31 : public BIProto{//built in proto
public:
	const BIField bifield0;
	const BIField bifield1;
	const BIField bifield2;
	const BIField bifield3;
	const BIField bifield4;
	const BIField bifield5;
	const BIField bifield6;
	const BIField bifield7;
	const BIField bifield8;
	const BIField bifield9;
	const BIField bifield10;
	const BIField bifield11;
	const BIField bifield12;
	const BIField bifield13;
	const BIField bifield14;
	const BIField bifield15;
	const BIField bifield16;
	const BIField bifield17;
	const BIField bifield18;
	const BIField bifield19;
	const BIField bifield20;
	const BIField bifield21;
	const BIField bifield22;
	const BIField bifield23;
	const BIField bifield24;
	const BIField bifield25;
	const BIField bifield26;
	const BIField bifield27;
	const BIField bifield28;
	const BIField bifield29;
	const BIField bifield30;
	BIProto31(const char* ntype, const char* containerField, BIField bifield0, BIField bifield1, BIField bifield2, BIField bifield3, BIField bifield4, BIField bifield5, BIField bifield6, BIField bifield7, BIField bifield8, BIField bifield9, BIField bifield10, BIField bifield11, BIField bifield12, BIField bifield13, BIField bifield14, BIField bifield15, BIField bifield16, BIField bifield17, BIField bifield18, BIField bifield19, BIField bifield20, BIField bifield21, BIField bifield22, BIField bifield23, BIField bifield24, BIField bifield25, BIField bifield26, BIField bifield27, BIField bifield28, BIField bifield29, BIField bifield30):
		BIProto(ntype, containerField, 31, &this->bifield0),
		bifield0(bifield0),
		bifield1(bifield1),
		bifield2(bifield2),
		bifield3(bifield3),
		bifield4(bifield4),
		bifield5(bifield5),
		bifield6(bifield6),
		bifield7(bifield7),
		bifield8(bifield8),
		bifield9(bifield9),
		bifield10(bifield10),
		bifield11(bifield11),
		bifield12(bifield12),
		bifield13(bifield13),
		bifield14(bifield14),
		bifield15(bifield15),
		bifield16(bifield16),
		bifield17(bifield17),
		bifield18(bifield18),
		bifield19(bifield19),
		bifield20(bifield20),
		bifield21(bifield21),
		bifield22(bifield22),
		bifield23(bifield23),
		bifield24(bifield24),
		bifield25(bifield25),
		bifield26(bifield26),
		bifield27(bifield27),
		bifield28(bifield28),
		bifield29(bifield29),
		bifield30(bifield30)
	{}
};
class BIProto32 : public BIProto{//built in proto
public:
	const BIField bifield0;
	const BIField bifield1;
	const BIField bifield2;
	const BIField bifield3;
	const BIField bifield4;
	const BIField bifield5;
	const BIField bifield6;
	const BIField bifield7;
	const BIField bifield8;
	const BIField bifield9;
	const BIField bifield10;
	const BIField bifield11;
	const BIField bifield12;
	const BIField bifield13;
	const BIField bifield14;
	const BIField bifield15;
	const BIField bifield16;
	const BIField bifield17;
	const BIField bifield18;
	const BIField bifield19;
	const BIField bifield20;
	const BIField bifield21;
	const BIField bifield22;
	const BIField bifield23;
	const BIField bifield24;
	const BIField bifield25;
	const BIField bifield26;
	const BIField bifield27;
	const BIField bifield28;
	const BIField bifield29;
	const BIField bifield30;
	const BIField bifield31;
	BIProto32(const char* ntype, const char* containerField, BIField bifield0, BIField bifield1, BIField bifield2, BIField bifield3, BIField bifield4, BIField bifield5, BIField bifield6, BIField bifield7, BIField bifield8, BIField bifield9, BIField bifield10, BIField bifield11, BIField bifield12, BIField bifield13, BIField bifield14, BIField bifield15, BIField bifield16, BIField bifield17, BIField bifield18, BIField bifield19, BIField bifield20, BIField bifield21, BIField bifield22, BIField bifield23, BIField bifield24, BIField bifield25, BIField bifield26, BIField bifield27, BIField bifield28, BIField bifield29, BIField bifield30, BIField bifield31):
		BIProto(ntype, containerField, 32, &this->bifield0),
		bifield0(bifield0),
		bifield1(bifield1),
		bifield2(bifield2),
		bifield3(bifield3),
		bifield4(bifield4),
		bifield5(bifield5),
		bifield6(bifield6),
		bifield7(bifield7),
		bifield8(bifield8),
		bifield9(bifield9),
		bifield10(bifield10),
		bifield11(bifield11),
		bifield12(bifield12),
		bifield13(bifield13),
		bifield14(bifield14),
		bifield15(bifield15),
		bifield16(bifield16),
		bifield17(bifield17),
		bifield18(bifield18),
		bifield19(bifield19),
		bifield20(bifield20),
		bifield21(bifield21),
		bifield22(bifield22),
		bifield23(bifield23),
		bifield24(bifield24),
		bifield25(bifield25),
		bifield26(bifield26),
		bifield27(bifield27),
		bifield28(bifield28),
		bifield29(bifield29),
		bifield30(bifield30),
		bifield31(bifield31)
	{}
};
class BIProto33 : public BIProto{//built in proto
public:
	const BIField bifield0;
	const BIField bifield1;
	const BIField bifield2;
	const BIField bifield3;
	const BIField bifield4;
	const BIField bifield5;
	const BIField bifield6;
	const BIField bifield7;
	const BIField bifield8;
	const BIField bifield9;
	const BIField bifield10;
	const BIField bifield11;
	const BIField bifield12;
	const BIField bifield13;
	const BIField bifield14;
	const BIField bifield15;
	const BIField bifield16;
	const BIField bifield17;
	const BIField bifield18;
	const BIField bifield19;
	const BIField bifield20;
	const BIField bifield21;
	const BIField bifield22;
	const BIField bifield23;
	const BIField bifield24;
	const BIField bifield25;
	const BIField bifield26;
	const BIField bifield27;
	const BIField bifield28;
	const BIField bifield29;
	const BIField bifield30;
	const BIField bifield31;
	const BIField bifield32;
	BIProto33(const char* ntype, const char* containerField, BIField bifield0, BIField bifield1, BIField bifield2, BIField bifield3, BIField bifield4, BIField bifield5, BIField bifield6, BIField bifield7, BIField bifield8, BIField bifield9, BIField bifield10, BIField bifield11, BIField bifield12, BIField bifield13, BIField bifield14, BIField bifield15, BIField bifield16, BIField bifield17, BIField bifield18, BIField bifield19, BIField bifield20, BIField bifield21, BIField bifield22, BIField bifield23, BIField bifield24, BIField bifield25, BIField bifield26, BIField bifield27, BIField bifield28, BIField bifield29, BIField bifield30, BIField bifield31, BIField bifield32):
		BIProto(ntype, containerField, 33, &this->bifield0),
		bifield0(bifield0),
		bifield1(bifield1),
		bifield2(bifield2),
		bifield3(bifield3),
		bifield4(bifield4),
		bifield5(bifield5),
		bifield6(bifield6),
		bifield7(bifield7),
		bifield8(bifield8),
		bifield9(bifield9),
		bifield10(bifield10),
		bifield11(bifield11),
		bifield12(bifield12),
		bifield13(bifield13),
		bifield14(bifield14),
		bifield15(bifield15),
		bifield16(bifield16),
		bifield17(bifield17),
		bifield18(bifield18),
		bifield19(bifield19),
		bifield20(bifield20),
		bifield21(bifield21),
		bifield22(bifield22),
		bifield23(bifield23),
		bifield24(bifield24),
		bifield25(bifield25),
		bifield26(bifield26),
		bifield27(bifield27),
		bifield28(bifield28),
		bifield29(bifield29),
		bifield30(bifield30),
		bifield31(bifield31),
		bifield32(bifield32)
	{}
};

class BIProto34 : public BIProto{//built in proto
public:
	const BIField bifield0;
	const BIField bifield1;
	const BIField bifield2;
	const BIField bifield3;
	const BIField bifield4;
	const BIField bifield5;
	const BIField bifield6;
	const BIField bifield7;
	const BIField bifield8;
	const BIField bifield9;
	const BIField bifield10;
	const BIField bifield11;
	const BIField bifield12;
	const BIField bifield13;
	const BIField bifield14;
	const BIField bifield15;
	const BIField bifield16;
	const BIField bifield17;
	const BIField bifield18;
	const BIField bifield19;
	const BIField bifield20;
	const BIField bifield21;
	const BIField bifield22;
	const BIField bifield23;
	const BIField bifield24;
	const BIField bifield25;
	const BIField bifield26;
	const BIField bifield27;
	const BIField bifield28;
	const BIField bifield29;
	const BIField bifield30;
	const BIField bifield31;
	const BIField bifield32;
	const BIField bifield33;
	BIProto34(const char* ntype, const char* containerField, BIField bifield0, BIField bifield1, BIField bifield2, BIField bifield3, BIField bifield4, BIField bifield5, BIField bifield6, BIField bifield7, BIField bifield8, BIField bifield9, BIField bifield10, BIField bifield11, BIField bifield12, BIField bifield13, BIField bifield14, BIField bifield15, BIField bifield16, BIField bifield17, BIField bifield18, BIField bifield19, BIField bifield20, BIField bifield21, BIField bifield22, BIField bifield23, BIField bifield24, BIField bifield25, BIField bifield26, BIField bifield27, BIField bifield28, BIField bifield29, BIField bifield30, BIField bifield31, BIField bifield32, BIField bifield33):
		BIProto(ntype, containerField, 34, &this->bifield0),
		bifield0(bifield0),
		bifield1(bifield1),
		bifield2(bifield2),
		bifield3(bifield3),
		bifield4(bifield4),
		bifield5(bifield5),
		bifield6(bifield6),
		bifield7(bifield7),
		bifield8(bifield8),
		bifield9(bifield9),
		bifield10(bifield10),
		bifield11(bifield11),
		bifield12(bifield12),
		bifield13(bifield13),
		bifield14(bifield14),
		bifield15(bifield15),
		bifield16(bifield16),
		bifield17(bifield17),
		bifield18(bifield18),
		bifield19(bifield19),
		bifield20(bifield20),
		bifield21(bifield21),
		bifield22(bifield22),
		bifield23(bifield23),
		bifield24(bifield24),
		bifield25(bifield25),
		bifield26(bifield26),
		bifield27(bifield27),
		bifield28(bifield28),
		bifield29(bifield29),
		bifield30(bifield30),
		bifield31(bifield31),
		bifield32(bifield32),
		bifield33(bifield33)
	{}
};


class BIProto35 : public BIProto{//built in proto
public:
	const BIField bifield0;
	const BIField bifield1;
	const BIField bifield2;
	const BIField bifield3;
	const BIField bifield4;
	const BIField bifield5;
	const BIField bifield6;
	const BIField bifield7;
	const BIField bifield8;
	const BIField bifield9;
	const BIField bifield10;
	const BIField bifield11;
	const BIField bifield12;
	const BIField bifield13;
	const BIField bifield14;
	const BIField bifield15;
	const BIField bifield16;
	const BIField bifield17;
	const BIField bifield18;
	const BIField bifield19;
	const BIField bifield20;
	const BIField bifield21;
	const BIField bifield22;
	const BIField bifield23;
	const BIField bifield24;
	const BIField bifield25;
	const BIField bifield26;
	const BIField bifield27;
	const BIField bifield28;
	const BIField bifield29;
	const BIField bifield30;
	const BIField bifield31;
	const BIField bifield32;
	const BIField bifield33;
	const BIField bifield34;
	BIProto35(const char* ntype, const char* containerField, BIField bifield0, BIField bifield1, BIField bifield2, BIField bifield3, BIField bifield4, BIField bifield5, BIField bifield6, BIField bifield7, BIField bifield8, BIField bifield9, BIField bifield10, BIField bifield11, BIField bifield12, BIField bifield13, BIField bifield14, BIField bifield15, BIField bifield16, BIField bifield17, BIField bifield18, BIField bifield19, BIField bifield20, BIField bifield21, BIField bifield22, BIField bifield23, BIField bifield24, BIField bifield25, BIField bifield26, BIField bifield27, BIField bifield28, BIField bifield29, BIField bifield30, BIField bifield31, BIField bifield32, BIField bifield33, BIField bifield34):
		BIProto(ntype, containerField, 35, &this->bifield0),
		bifield0(bifield0),
		bifield1(bifield1),
		bifield2(bifield2),
		bifield3(bifield3),
		bifield4(bifield4),
		bifield5(bifield5),
		bifield6(bifield6),
		bifield7(bifield7),
		bifield8(bifield8),
		bifield9(bifield9),
		bifield10(bifield10),
		bifield11(bifield11),
		bifield12(bifield12),
		bifield13(bifield13),
		bifield14(bifield14),
		bifield15(bifield15),
		bifield16(bifield16),
		bifield17(bifield17),
		bifield18(bifield18),
		bifield19(bifield19),
		bifield20(bifield20),
		bifield21(bifield21),
		bifield22(bifield22),
		bifield23(bifield23),
		bifield24(bifield24),
		bifield25(bifield25),
		bifield26(bifield26),
		bifield27(bifield27),
		bifield28(bifield28),
		bifield29(bifield29),
		bifield30(bifield30),
		bifield31(bifield31),
		bifield32(bifield32),
		bifield33(bifield33),
		bifield34(bifield34)
	{}
};

class BIProto36 : public BIProto{//built in proto
public:
	const BIField bifield0;
	const BIField bifield1;
	const BIField bifield2;
	const BIField bifield3;
	const BIField bifield4;
	const BIField bifield5;
	const BIField bifield6;
	const BIField bifield7;
	const BIField bifield8;
	const BIField bifield9;
	const BIField bifield10;
	const BIField bifield11;
	const BIField bifield12;
	const BIField bifield13;
	const BIField bifield14;
	const BIField bifield15;
	const BIField bifield16;
	const BIField bifield17;
	const BIField bifield18;
	const BIField bifield19;
	const BIField bifield20;
	const BIField bifield21;
	const BIField bifield22;
	const BIField bifield23;
	const BIField bifield24;
	const BIField bifield25;
	const BIField bifield26;
	const BIField bifield27;
	const BIField bifield28;
	const BIField bifield29;
	const BIField bifield30;
	const BIField bifield31;
	const BIField bifield32;
	const BIField bifield33;
	const BIField bifield34;
	const BIField bifield35;
	BIProto36(const char* ntype, const char* containerField, BIField bifield0, BIField bifield1, BIField bifield2, BIField bifield3, BIField bifield4, BIField bifield5, BIField bifield6, BIField bifield7, BIField bifield8, BIField bifield9, BIField bifield10, BIField bifield11, BIField bifield12, BIField bifield13, BIField bifield14, BIField bifield15, BIField bifield16, BIField bifield17, BIField bifield18, BIField bifield19, BIField bifield20, BIField bifield21, BIField bifield22, BIField bifield23, BIField bifield24, BIField bifield25, BIField bifield26, BIField bifield27, BIField bifield28, BIField bifield29, BIField bifield30, BIField bifield31, BIField bifield32, BIField bifield33, BIField bifield34, BIField bifield35):
		BIProto(ntype, containerField, 36, &this->bifield0),
		bifield0(bifield0),
		bifield1(bifield1),
		bifield2(bifield2),
		bifield3(bifield3),
		bifield4(bifield4),
		bifield5(bifield5),
		bifield6(bifield6),
		bifield7(bifield7),
		bifield8(bifield8),
		bifield9(bifield9),
		bifield10(bifield10),
		bifield11(bifield11),
		bifield12(bifield12),
		bifield13(bifield13),
		bifield14(bifield14),
		bifield15(bifield15),
		bifield16(bifield16),
		bifield17(bifield17),
		bifield18(bifield18),
		bifield19(bifield19),
		bifield20(bifield20),
		bifield21(bifield21),
		bifield22(bifield22),
		bifield23(bifield23),
		bifield24(bifield24),
		bifield25(bifield25),
		bifield26(bifield26),
		bifield27(bifield27),
		bifield28(bifield28),
		bifield29(bifield29),
		bifield30(bifield30),
		bifield31(bifield31),
		bifield32(bifield32),
		bifield33(bifield33),
		bifield34(bifield34),
		bifield35(bifield35)
	{}
};

extern int g_userProtoBegin;




static const BIProto4 mesh_ = BIProto4(
	"Mesh", "mesh",
	BIField(MFVEC3F,	"coord []", 0),
	BIField(MFVEC3F,	"normal []", 0),
	BIField(MFCOLOR,	"color []", 0),
	BIField(MFINT32,	"coordIndex []", 0)
);


static const BIProto* builtInProtoList[]={
	&mesh_,
};

const char* constBuiltInNodeNameList[]={
	"Mesh",
};

void DynamicMField::read2(Field *&field, Properties& sprop, bool& wasMF, const BIField& bi){
	if(bi.type==MFBOOL){
		wasMF=true;
		field=new MFBool(M_Bool());
	}
	else if(bi.type==MFCOLOR){
		wasMF=true;
		field=new MFColor(M_Color3f());
	}
	else if(bi.type==MFFLOAT){
		wasMF=true;
		field=new MFFloat(M_Float());
	}
	else if(bi.type==MFINT32){
		wasMF=true;
		field=new MFInt32(M_Int());
	}
	else if(bi.type==MFNODE){
		wasMF=true;
		field=new MFNode(M_Node());
	}
	else if(bi.type==MFROTATION){
		wasMF=true;
		field=new MFRotation(M_Rotation());
	}
	else if(bi.type==MFSTRING){
		wasMF=true;
		field=new MFString(M_String());
	}
	else if(bi.type==MFDOUBLE){
		wasMF=true;
		field=new MFTime(M_Double());
	}
	else if(bi.type==MFVEC2F){
		wasMF=true;
		field=new MFVec2f(M_Vec2f());
	}
	else if(bi.type==MFVEC3F){
		wasMF=true;
		field=new MFVec3f(M_Vec3f());
	}
	else if(bi.type==SFBOOL){
		field=new SFBool(false);
	}
	else if(bi.type==SFCOLOR){
		field=new SFColor(Color3f());
	}
	else if(bi.type==SFFLOAT){
		field=new SFFloat(0);
	}
	else if(bi.type==SFINT32){
		field=new SFInt32(0);
	}
	else if(bi.type==SFNODE){
		field=new SFNode(NodePtr());
	}
	else if(bi.type==SFROTATION){
		field=new SFRotation(Rotation());
	}
	else if(bi.type==SFSTRING){
		field=new SFString(String());
	}
	else if(bi.type==SFTIME){
		field=new SFTime(0);
	}
	else if(bi.type==SFVEC2F){
		field=new SFVec2f(Vec2f());
	}
	else if(bi.type==SFVEC3F){
		field=new SFVec3f(Vec3f());
	}
	else throw;

}
void DynamicMField::read(M_Properties& properties, const BIProto& binode, String& str){
	bool groupHasBegan=false;
	int groupCntr=-1;
	for(int k=0; k<binode.numOf; k++){
		Properties sprop;
		bool wasMF=false;
		const BIField* biPtr=binode.bifieldRef;
		const BIField& bi=biPtr[k];
		sprop.flags=bi.flags;
		sprop.promptNode.length=bi.initPromptNode.numOf;
		for(int j=0; j<bi.initPromptNode.numOf; j++){
			sprop.promptNode[j]=bi.initPromptNode.list[j];
		}
		str=bi.str;
		int i=0;
		sprop.name=str.getWord(i);

		Field* field=0;
		sprop.tip=bi.tip;

		read2(field, sprop, wasMF, bi);

		if(sprop.initializeOnly() || sprop.inputOutput()){
			field->read(str,i);
		}
		*this+=field;

		bool beginGroup=false;
		bool endGroup=false;
		str.stb(i);
		if(i<str.length){
			sprop.read(*field, str, i, beginGroup, endGroup);
		}
		if( wasMF&&(sprop.isReadable()^sprop.isWritable())==false&&field->getType()!=MFNODE){
			if(groupHasBegan==false&&beginGroup==false){
				sprop.groupIndex=++groupCntr;
			}
			else if(beginGroup){
				sprop.groupIndex=++groupCntr;
				groupHasBegan=true;
			}
			else if(groupHasBegan){
				sprop.groupIndex=groupCntr;
			}
			if(endGroup){
				groupHasBegan=false;
			}
		}
		properties+=sprop;
	}
}



M_ProtoInterface g_lookupProtoInterface;
extern String g_protoInterfaceListStr;
extern String g_protoInterfaceTips;
extern String g_protoInterfaceNodePromptStr;
void initLookup(){
///////////////

	static String str;
	nodeLookUpStr.length=0;
	const int numOfBuiltInNodes = sizeof(constBuiltInNodeNameList)/sizeof(const char*);
	for(int i=0; i<numOfBuiltInNodes; i++){
		nodeLookUpStr+=constBuiltInNodeNameList[i];
	}
	g_lookupProtoInterface.length=numOfBuiltInNodes;
	containerFieldLookUpStr.length=numOfBuiltInNodes;

	for(int k=0; k<numOfBuiltInNodes; k++){
		ProtoInterface* protoInterface=new ProtoInterface;
		g_lookupProtoInterface[k]=protoInterface;
		if(builtInProtoList[k]->containerField){
			containerFieldLookUpStr[k]=builtInProtoList[k]->containerField;
		}
		protoInterface->read(protoInterface->properties, *(builtInProtoList[k]), str);
	}
	g_userProtoBegin=g_lookupProtoInterface.length;
}


