/* Seamless3d

Copyright (C) 2001-2014 Graham John Perrett (AKA thyme) thyme@seamless3d.com www.seamless3d.com

Permission is hereby granted, free of charge, to any person obtaining a copy of this
software and associated documentation files (the "Software"), to deal in the Software
without restriction, including without limitation the rights to use, copy, modify,
merge, publish, distribute, sublicense, and/or sell copies of the Software, and to
permit persons to whom the Software is furnished to do so, subject to the following
conditions:

The above copyright notice and this permission notice shall be included in all copies or
substantial portions of the Software.

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR IMPLIED,
INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR
PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE
LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT
OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR
OTHER DEALINGS IN THE SOFTWARE.
*/

#include <QOpenGLFunctions>
#include <QOpenGLShaderProgram>
#include <QOpenGLShaderProgram>
#include <QOpenGLBuffer>



#include "seamless3d.h"
#include "render-ogl.h"

QOpenGLShaderProgram CNCVertex::program;

GLuint CNCVertex::m_coord = 0;
GLuint CNCVertex::m_normal = 0;
GLuint CNCVertex::m_color = 0;
GLuint CNCVertex::m_matrixUniform = 0;

bool Render::initShader(QOpenGLShaderProgram& program, const char* vshaderFile, const char* fshaderFile)
{
    // Compile vertex shader
    if (!program.addShaderFromSourceFile(QOpenGLShader::Vertex, vshaderFile)){
        return false;
    }

    // Compile fragment shader
    if (!program.addShaderFromSourceFile(QOpenGLShader::Fragment, fshaderFile)){
        return false;
    }

    // Link shader pipeline
    if (!program.link()){
        return false;
    }

    // Bind shader pipeline for use
    if (!program.bind()){
        return false;
    }

    return true;
}



void Render::convertToRenderFormatCNC(const M_Vec3f& coord, const M_Vec3f& normal, M_Color3f& color, M_Int& coordIndex){
    if (pVB != NULL){
        delete[] pVB;
        pVB = 0;
    }

    if(pIB != NULL){
        delete[] pIB;
        pIB = 0;
    }


    arrayBuf.create();
    indexBuf.create();

    vertexCount = coord.length;
    indexCount = coordIndex.length;

    CNCVertex* vertices = new CNCVertex[vertexCount];
    for(int k = 0; k < vertexCount; k++){
        vertices[k] = CNCVertex(coord[k], normal[k], color[k]);
    }
    pVB = vertices;

    pIB = new GLushort[indexCount];


    for(int j = 0; j < coordIndex.length; j++){
        pIB[j] = coordIndex[j];
    }

    // Transfer vertex data to VBO 0
    arrayBuf.bind();
    arrayBuf.allocate(pVB, vertexCount * sizeof(CNCVertex));

    // Transfer index data to VBO 1
    indexBuf.bind();
    indexBuf.allocate(pIB, indexCount * sizeof(GLushort));
    indexBuf.release();
    arrayBuf.release();

}
