
/* Seamless3d

Copyright (C) 2001-2014 Graham John Perrett (AKA thyme) thyme@seamless3d.com www.seamless3d.com

Permission is hereby granted, free of charge, to any person obtaining a copy of this
software and associated documentation files (the "Software"), to deal in the Software
without restriction, including without limitation the rights to use, copy, modify,
merge, publish, distribute, sublicense, and/or sell copies of the Software, and to
permit persons to whom the Software is furnished to do so, subject to the following
conditions:

The above copyright notice and this permission notice shall be included in all copies or
substantial portions of the Software.

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR IMPLIED,
INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR
PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE
LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT
OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR
OTHER DEALINGS IN THE SOFTWARE.
*/

#include "seamless3d.h"
float g_version = 3.004;

const char* protoInitilaiseStr2=0;

void searchTillText(char*& p);
Node* getNode2(const String& str, int& i);

int g_userProtoBegin = 0;
M_Node g_defLookUp;
ProtoDeclare* g_protoDeclare;

char floatFormat[]={"%.6f"};
char floatFormat2[]={" %.6f"};
char g_vrmlSquareOpenBracket[]={"["};
char g_vrmlSquareCloseBracket[]={"]"};
char g_vrmlQuote[]={"\""};

const char g_exposedField[]="exposedField";
const char g_eventIn[]="eventIn";
const char g_eventOut[]="eventOut";
const char g_field[]="field";

const char g_inputOutput[]="inputOutput";
const char g_inputOnly[]="inputOnly";
const char g_outputOnly[]="outputOnly";
const char g_initializeOnly[]="initializeOnly";

const char* g_exposedFieldPtr=g_exposedField;
const char* g_eventInPtr=g_eventIn;
const char* g_eventOutPtr=g_eventOut;
const char* g_fieldPtr=g_field;

void changeFloatFormat(int numOfDigits){
    char chr='0'+numOfDigits;
    floatFormat[2]=floatFormat2[3]=chr;
}

M_String nodeLookUpStr;
M_String containerFieldLookUpStr;
Color3f g_bColor;
bool g_collapse=false;


bool isLegalNameChar(const char& ch){
    if( ch>='a'&&ch<='z')return true;
    if( ch>='/'&&ch<='Z')return true;
    if(	ch<=' '||
        ch==0x7f||
        ch=='\''||
        ch=='\"'||
        ch=='#'||
        ch==','||
        ch=='.'||
        ch=='['||
        ch==']'||
        ch=='\\'||
        ch=='{'||
        ch=='}'
        ){
        return false;
    }
    return true;
}
template<>
void M_Bool::getField(const String& str,int& i){
    str.stb(i);
    if(str[i]=='['){
        i++;
        str.stb(i);
        int k,ctr;

        for(k=i,ctr=0;str[k]!=']';ctr++){
            getBool(str, k);
            str.stb(k);
        }
        length=ctr;

        for(ctr=0;str[i]!=']';ctr++){
            length.pointer[ctr]=getBool(str, i);
            str.stb(i);
        }
        i++;
    }
    else{
        *this+=getBool(str,i);
    }
}

template<>
void M_Float::getField(const String& str,int& i){
    str.stb(i);
    if(str[i]=='['){
        i++;
        str.stb(i);
        int k,ctr;


        for(k=i,ctr=0;str[k]!=']';ctr++){
            getFloat(str, k);
            str.stb(k);
        }
        M_Float tmpThis; tmpThis.length=ctr;

        for(ctr=0;str[i]!=']';ctr++){
            tmpThis[ctr]=getFloat(str, i);
            str.stb(i);
        }
        *this=tmpThis;
        i++;
    }
    else{
        *this+=getFloat(str, i);
    }
}

void Color3f::getField(const String& str, int& i){
    r=getFloat(str, i);
    str.stb(i);
    g=getFloat(str, i);
    str.stb(i);
    b=getFloat(str, i);
}
template<>
void M_Int::getField(const String& str,int& i){
    str.stb(i);
    if(str[i]=='['){
        i++;
        str.stb(i);
        int k,ctr;

        for(k=i,ctr=0;str[k]!=']';ctr++){
            getInt(str, k);
            str.stb(k);
        }
        length=ctr;

        for(ctr=0;str[i]!=']';ctr++){
            length.pointer[ctr]=getInt(str, i);
            str.stb(i);
        }
        i++;
    }
    else{
        *this+=getInt(str,i);
    }
}

template<>
void M_Color3f::getField(const String& str,int& i){
    str.stb(i);
    Color3f dummy;
    if(str[i]=='['){
        i++;
        str.stb(i);
        int k,ctr;


        for(k=i,ctr=0;str[k]!=']';ctr++){
            dummy.getField(str, k);
            str.stb(k);
        }
        M_Color3f tmpThis; tmpThis.length=ctr;

        for(ctr=0;str[i]!=']';ctr++){
            tmpThis[ctr].getField(str, i);
            str.stb(i);
        }
        *this=tmpThis;
        i++;
    }
    else{
        dummy.getField(str, i);
        *this=dummy;
    }
}
template<>
void M_Vec3f::getField(const String& str,int& i){
    str.stb(i);
    Vec3f dummy;
    if(str[i]=='['){
        i++;
        str.stb(i);
        int k,ctr;


        for(k=i,ctr=0;str[k]!=']';ctr++){
            dummy.getField(str, k);
            str.stb(k);
        }
        M_Vec3f tmpThis; tmpThis.length=ctr;

        for(ctr=0;str[i]!=']';ctr++){
            tmpThis[ctr].getField(str, i);
            str.stb(i);
        }
        *this=tmpThis;
        i++;
    }
    else{
        dummy.getField(str, i);
        *this=dummy;
    }
}
template<>
void M_Rotation::getField(const String& str,int& i){
    str.stb(i);
    Rotation dummy;
    if(str[i]=='['){
        i++;
        str.stb(i);
        int k,ctr;


        for(k=i,ctr=0;str[k]!=']';ctr++){
            dummy.getField(str, k);
            str.stb(k);
        }
        M_Rotation tmpThis; tmpThis.length=ctr;

        for(ctr=0;str[i]!=']';ctr++){
            tmpThis[ctr].getField(str, i);
            str.stb(i);
        }
        *this=tmpThis;
        i++;
    }
    else{
        dummy.getField(str, i);
        *this=dummy;
    }
}
template<>
void M_Vec2f::getField(const String& str,int& i){
    str.stb(i);
    Vec2f dummy;
    if(str[i]=='['){
        i++;
        str.stb(i);
        int k,ctr;


        for(k=i,ctr=0;str[k]!=']';ctr++){
            dummy.getField(str, k);
            str.stb(k);
        }
        M_Vec2f tmpThis; tmpThis.length=ctr;

        for(ctr=0;str[i]!=']';ctr++){
            tmpThis[ctr].getField(str, i);
            str.stb(i);
        }
        *this=tmpThis;
        i++;
    }
    else{
        dummy.getField(str, i);
        *this=dummy;
    }
}
template <class T> void Vec3<T>::getField(const String& str, int& i){
    x=getFloat(str, i);
    str.stb(i);
    y=getFloat(str, i);
    str.stb(i);
    z=getFloat(str, i);
}

template <class T> void Vec2<T>::getField(const String& str, int& i){
    x=getFloat(str, i);
    str.stb(i);
    y=getFloat(str, i);
}

void Rotation::getField(const String& str, int& i){
    Vec3f axis;
    axis.x=getFloat(str, i);
    str.stb(i);
    axis.y=getFloat(str, i);
    str.stb(i);
    axis.z=getFloat(str, i);
    str.stb(i);
    *this=Rotation(axis.normalize(),getFloat(str, i));
}

void checkBackgroundColor(const String& str, int& i){
    str.stbc(i);
    while(i < str.length && str[i]=='#'){
        i++;
        if(str.cmpKWord("bColor", i)){
            str.stb(i);
            g_bColor.getField(str,i);
        }
//		str.stb(i);
        if(str[i]==' ')i++;
        if(str.cmpKWord("collapse", i)){
            g_collapse=true;
        }

        str.stnl(i);
        str.stbc(i);
    }
}

Node* getNode2(const String& str, int& i){
    checkBackgroundColor(str, i);

    Node* node=0;

//	str.stb(i);
    String strDefName;
    if(str.substring(i, i + 2)=="DEF"){
        str.stw_stb(i);
        strDefName=str.extractName(i);
        str.stb(i);
    }
    else switch(str[i++]){
        case 'A':{
            break;
        }
        case 'B':{
            break;
        }
        case 'C':{
            break;
        }
        case 'D':{
            break;
        }
        case 'E':{
            break;
        }
        case 'F':{
            break;
        }
        case'G':{
            break;
        }
        case 'H':{
            break;
        }
        case 'I':{
            break;
        }
        case 'J':{
            break;
        }
        case 'L':{
            break;
        }
        case 'M':{
            if(str.cmpKWord("esh",i)){
                node = new Mesh;
            }
            break;
        }
        case 'N':{
            break;
        }
        case 'O':{
            break;
        }
        case 'P':{
            break;
        }
        case 'R':{
            break;
        }
        case 'S':{
            break;
        }
        case 'T':{
            break;
        }
        case 'V':{
            break;
        }
        case 'W':{
            break;
        }
        case 'b':{
            break;
        }
        case 'd':{
            break;
        }
        case 'f':{
            break;
        }
        case 'i':{
            break;
        }
        case 'v':{
            break;
        }
        default: {
                 }
    }
    if(node==0){
        throw;
    }
    node->name=strDefName;
    node->getNode(str, i);

    return node;
}

void getNodesFromStr(M_Node& m_node, const String &str, int &i, bool multipleNodes=true){
    str.stb(i);
    while(i < str.length-1 && str[i]!=']'){
        m_node+=getNode2(str, i);

        checkBackgroundColor(str, i);
        if(!multipleNodes) break;
    }
    if(multipleNodes) i++;
}
template<>
void M_Node::getField(const String& str, int &i){
    bool brackFlag=false;
    str.stb(i);
    if(str[i]=='['){
        brackFlag=true;
        i++;
    }
    str.stb(i);
    getNodesFromStr(*this, str, i, brackFlag);
}

template<>
void M_String::getField(const String& str,int& i){// assumed length=0
    str.stb(i);
    if(str[i]=='['){
        String dummy;
        i++;
        str.stb(i);

        while(str[i]!=']'){
            dummy.getField(str, i);
            (*this)+=String(dummy.length+1,0); //economize on buf size
            (*this)[length-1]+=dummy;
            str.stb(i);
        }
        i++;
    }
    else{
        length=1;
        (*this)[0].getField(str, i);
    }
}


ProtoInstance::ProtoInstance(const int k){
    userNodeType=k;
    const ProtoInterface* protoInterface=g_lookupProtoInterface[k];
#ifdef SEAMLESS3DGUI
        index.length=50;
#endif
    for(int i=0; i<protoInterface->size(); i++){
        Field* field=protoInterface->getField(i);
        switch(field->getType()){
        case MFBOOL:{
            MFBool* f=dynamic_cast<MFBool*>(field);
            *this+=new MFBool(M_Bool(f->getValue()));
        }
        break;
        case MFCOLOR:{
            MFColor* f=dynamic_cast<MFColor*>(field);
            *this+=new MFColor(M_Color3f(f->getValue()));
        }
        break;
        case MFFLOAT:{
            MFFloat* f=dynamic_cast<MFFloat*>(field);
            *this+=new MFFloat(M_Float(f->getValue()));
        }
        break;
        case MFINT32:{
            MFInt32* f=dynamic_cast<MFInt32*>(field);
            *this+=new MFInt32(M_Int(f->getValue()));
        }
        break;
        case MFNODE:{
            MFNode* f=dynamic_cast<MFNode*>(field);
            *this+=new MFNode(M_Node(f->getValue()));
        }
        break;
        case MFROTATION:{
            MFRotation* f=dynamic_cast<MFRotation*>(field);
            *this+=new MFRotation(M_Rotation(f->getValue()));
        }
        break;
        case MFSTRING:{
            MFString* f=dynamic_cast<MFString*>(field);
            *this+=new MFString(M_String(f->getValue()));
        }
        break;
        case MFDOUBLE:{
            MFTime* f=dynamic_cast<MFTime*>(field);
            *this+=new MFTime(M_Double(f->getValue()));
        }
        break;
        case MFVEC2F:{
            MFVec2f* f=dynamic_cast<MFVec2f*>(field);
            *this+=new MFVec2f(M_Vec2f(f->getValue()));
        }
        break;
        case MFVEC3F:{
            MFVec3f* f=dynamic_cast<MFVec3f*>(field);
            *this+=new MFVec3f(M_Vec3f(f->getValue()));
        }
        break;
        case SFBOOL:{
            SFBool* f=dynamic_cast<SFBool*>(field);
            *this+=new SFBool(bool(f->getValue()));
        }
        break;
        case SFCOLOR:{
            SFColor* f=dynamic_cast<SFColor*>(field);
            *this+=new SFColor(Color3f(f->getValue()));
        }
        break;
        case SFFLOAT:{
            SFFloat* f=dynamic_cast<SFFloat*>(field);
            *this+=new SFFloat(float(f->getValue()));
        }
        break;
        case SFINT32:{
            SFInt32* f=dynamic_cast<SFInt32*>(field);
            *this+=new SFInt32(int(f->getValue()));
        }
        break;
        case SFNODE:{
            SFNode* f=dynamic_cast<SFNode*>(field);
            *this+=new SFNode(NodePtr(f->getValue()));
        }
        break;
        case SFROTATION:{
            SFRotation* f=dynamic_cast<SFRotation*>(field);
            *this+=new SFRotation(Rotation(f->getValue()));
        }
        break;
        case SFSTRING:{
            SFString* f=dynamic_cast<SFString*>(field);
            *this+=new SFString(String(f->getValue()));
        }
        break;
        case SFTIME:{
            SFTime* f=dynamic_cast<SFTime*>(field);
            *this+=new SFTime(double(f->getValue()));
        }
        break;
        case SFVEC2F:{
            SFVec2f* f=dynamic_cast<SFVec2f*>(field);
            *this+=new SFVec2f(Vec2f(f->getValue()));
        }
        break;
        case SFVEC3F:{
            SFVec3f* f=dynamic_cast<SFVec3f*>(field);
            *this+=new SFVec3f(Vec3f(f->getValue()));
        }
        break;
        default: throw;
        }
    }
}


void DynamicMField::read2(Field *&field, Properties& sprop, bool& wasMF, const String& str,int& i){
    str.stb(i);
    if(str.cmpKWord("MFBool",i)){
        wasMF=true;
        sprop.name=str.getWord(i);
        field=new MFBool(*new M_Bool());
    }
    else if(str.cmpKWord("MFColor",i)){
        wasMF=true;
        sprop.name=str.getWord(i);
        field=new MFColor(*new M_Color3f());
    }
    else if(str.cmpKWord("MFFloat",i)){
        wasMF=true;
        sprop.name=str.getWord(i);
        field=new MFFloat(*new M_Float());
    }
    else if(str.cmpKWord("MFInt32",i)){
        wasMF=true;
        sprop.name=str.getWord(i);
        field=new MFInt32(*new M_Int());
    }
    else if(str.cmpKWord("MFNode",i)){
        wasMF=true;
        sprop.name=str.getWord(i);
        field=new MFNode(*new M_Node());
    }
    else if(str.cmpKWord("MFRotation",i)){
        wasMF=true;
        sprop.name=str.getWord(i);
        field=new MFRotation(*new M_Rotation());
    }
    else if(str.cmpKWord("MFString",i)){
        wasMF=true;
        sprop.name=str.getWord(i);
        field=new MFString(*new M_String());
    }
    else if(str.cmpKWord("MFTime",i)){
        wasMF=true;
        sprop.name=str.getWord(i);
        field=new MFTime(*new M_Double());
    }
    else if(str.cmpKWord("MFVec2f",i)){
        wasMF=true;
        sprop.name=str.getWord(i);
        field=new MFVec2f(*new M_Vec2f());
    }
    else if(str.cmpKWord("MFVec3f",i)){
        wasMF=true;
        sprop.name=str.getWord(i);
        field=new MFVec3f(*new M_Vec3f());
    }
    else if(str.cmpKWord("SFBool",i)){
        sprop.name=str.getWord(i);
        field=new SFBool(*new bool(false));
    }
    else if(str.cmpKWord("SFColor",i)){
        sprop.name=str.getWord(i);
        field=new SFColor(*new Color3f());
    }
    else if(str.cmpKWord("SFFloat",i)){
        sprop.name=str.getWord(i);
        field=new SFFloat(*new float(0));
    }
    else if(str.cmpKWord("SFInt32",i)){
        sprop.name=str.getWord(i);
        field=new SFInt32(*new int(0));
    }
    else if(str.cmpKWord("SFNode",i)){
        sprop.name=str.getWord(i);
        field=new SFNode(*new NodePtr(0));
    }
    else if(str.cmpKWord("SFRotation",i)){
        sprop.name=str.getWord(i);
        field=new SFRotation(*new Rotation());
    }
    else if(str.cmpKWord("SFString",i)){
        sprop.name=str.getWord(i);
        field=new SFString(*new String());
    }
    else if(str.cmpKWord("SFTime",i)){
        sprop.name=str.getWord(i);
        field=new SFTime(*new double(0));
    }
    else if(str.cmpKWord("SFVec2f",i)){
        sprop.name=str.getWord(i);
        field=new SFVec2f(*new Vec2f());
    }
    else if(str.cmpKWord("SFVec3f",i)){
        sprop.name=str.getWord(i);
        field=new SFVec3f(*new Vec3f());
    }
    else throw;

}
void Properties::read(Field& field, const String& str,int& i, bool& beginGroup, bool& endGroup){
    str.stb(i);
    if(str[i]!='{'){
        throw this;
    }
    str.stb(++i);
    while(str[i]!='}'){
        if(str.cmpKWord("promptNode",i)){
            str.stb(i);
            i++;
            int j=0;
            while(str[i]!='\"'){
                str.stb(i);
                for(j=0; j<LOOKUPLEN; j++){
                    if(str.cmpKWord(nodeLookUpStr[j],i)){
                        promptNode+=j;
                        break;
                    }
                }
                if(j==LOOKUPLEN) throw this;
            }
            i++;
        }
        else if (str.cmpKWord("tip",i)){
            str.stb(i);
            i++;
            tip=protoInitilaiseStr2+i;
            while(str[i]!='\"')i++;
            *const_cast<char*>(protoInitilaiseStr2+i)=0;
            i++;
        }
        else if (str.cmpKWord("push",i))set_push(getBool(str,i));
        else if (str.cmpKWord("doNotOutput",i))set_doNotOutput(getBool(str,i));
        else if (str.cmpKWord("doNotSave",i))set_doNotSave(getBool(str,i));
        else if (str.cmpKWord("blockUse",i))set_blockUse(getBool(str,i));
        else if (str.cmpKWord("blockNew",i)) set_blockNew(getBool(str,i));
        else if (str.cmpKWord("beginGroup",i)) beginGroup=getBool(str,i);
        else if (str.cmpKWord("sameAsLast",i)) set_sameAsLast(getBool(str,i));
        else if (str.cmpKWord("axis",i)) set_axis(getBool(str,i));
        else if (str.cmpKWord("startAtIndex",i)){startAtIndex=getInt(str,i);}
        else if (str.cmpKWord("endGroup",i)) endGroup=getBool(str,i);
        else if (str.cmpKWord("disable",i)) set_disable(getBool(str,i));
        else if (str.cmpKWord("disableLen",i)) set_disableLen(getBool(str,i));

        else if (str.cmpKWord("initElLen",i)){initElLen=getInt(str,i);}
        else if (str.cmpKWord("incElLen",i)){incElLen=getInt(str,i);}

        else if (str.cmpKWord("rangeFrom",i)){rangeFrom=getFloat(str,i);}
        else if (str.cmpKWord("rangeTo",i)){rangeTo=getFloat(str,i);}

        else if	(str.cmpKWord("initElVal",i)){
            initElLen=field.countItems(str,i);
            initElVal.getField(str,i);
        }
        else if	(str.cmpKWord("incElVal",i)){
            str.stb(i);
            incElLen=field.countItems(str,i);
            incElVal.getField(str,i);
        }


        else throw;
        str.stb(i);
    }
//	else if (str.cmpKWord("cycleInterval",i)) getFieldB(this, 0, str, i);
}
void DynamicMField::read(M_Properties &properties,const String& str,int& i){
    bool groupHasBegan=false;
    bool wasMF;
    int groupCntr=-1;
    if(str[i]!='[')throw this;
    i++;
    str.stb(i);
    Field *field;
    while(str[i]!=']'){
        Properties sprop;
        wasMF=false;
        if(str.cmpKWord(g_initializeOnly,i)||str.cmpKWord(g_field,i)){
            sprop.flags = UNREADABLE | UNWRITEABLE;
            read2(field, sprop, wasMF, str,i);
            field->read(str,i);
        }
        else if(str.cmpKWord(g_inputOnly,i)||str.cmpKWord(g_eventIn,i)){
            sprop.flags = UNREADABLE;
            read2(field, sprop, wasMF, str,i);
        }
        else if(str.cmpKWord(g_outputOnly,i)||str.cmpKWord(g_eventOut,i)){
            sprop.flags = UNWRITEABLE;
            read2(field, sprop, wasMF, str,i);
        }
        else if(str.cmpKWord(g_inputOutput,i)||str.cmpKWord(g_exposedField,i)){
            read2(field, sprop, wasMF, str,i);
            field->read(str,i);
        }
        else throw this;
/*
        if(wasMF&&(field->isReadable()^field->isWritable())==false&&field->getType()!=MFNODE){
            sprop.groupIndex=groupCntr;
            groupCntr++;
        }
        else{
            sprop.groupIndex=-1;
        }
*/
        *this+=field;
        str.stbc(i);

        bool beginGroup=false;
        bool endGroup=false;
        if(str[i]=='#'){
            i++;
            if(str.cmpKWord("Properties",i)){
                sprop.read(*field, str, i, beginGroup, endGroup);
            }
            i=str.indexOf('\n', i);
        }
        if( wasMF&&(sprop.isReadable()^sprop.isWritable())==false&&field->getType()!=MFNODE){
            if(groupHasBegan==false&&beginGroup==false){
                sprop.groupIndex=++groupCntr;
            }
            else if(beginGroup){
                sprop.groupIndex=++groupCntr;
                groupHasBegan=true;
            }
            else if(groupHasBegan){
                sprop.groupIndex=groupCntr;
            }
            if(endGroup){
                groupHasBegan=false;
            }
        }

        properties+=sprop;

        str.stb(i);
    }
    i++;
}

void ProtoInterface::getFields(const String& str,int& i){
    str.stb(i);

    DynamicMField::read(properties, str,i);

    str.stb(i);
}
void ProtoDeclare::getFields(const String& str,int& i){

    protoInterface->getFields(str,i);

    if(str[i]!='{'){
        throw this;
    }
    str.stb(++i);
    i++;
}




template<> inline int SFBool::countItems(const String& str, int i){throw this;}
inline int SFColor::countItems(const String& str, int i){throw this;}
template<> inline int SFFloat::countItems(const String& str, int i){throw this;}
template<> inline int SFInt32::countItems(const String& str, int i){throw this;}
inline int SFNode::countItems(const String& str, int i){throw this;}
inline int SFRotation::countItems(const String& str, int i){throw this;}
inline int SFString::countItems(const String& str, int i){throw this;}
template<> inline int SFTime::countItems(const String& str, int i){throw this;}
inline int SFVec2f::countItems(const String& str, int i){throw this;}
inline int SFVec3f::countItems(const String& str, int i){throw this;}

template<> inline int MFBool::countItems(const String& str, int i)	{str.stb(i);i++; M_Bool dummy; dummy.getField(str,i); return dummy.length;}
template<> inline int MFColor::countItems(const String& str, int i){str.stb(i);i++; M_Color3f dummy; dummy.getField(str,i); return dummy.length;}
template<> inline int MFFloat::countItems(const String& str, int i){str.stb(i);i++; M_Float dummy; dummy.getField(str,i); return dummy.length;}
template<> inline int MFInt32::countItems(const String& str, int i){str.stb(i);i++; M_Int dummy; dummy.getField(str,i); return dummy.length;}
template<> inline int MFNode::countItems(const String& str, int i)	{str.stb(i);i++; M_Node dummy; dummy.getField(str,i); return dummy.length;}
template<> inline int MFRotation::countItems(const String& str, int i){str.stb(i);i++; M_Rotation dummy; dummy.getField(str,i); return dummy.length;}
template<> inline int MFString::countItems(const String& str, int i){str.stb(i);i++; M_String dummy; dummy.getField(str,i); return dummy.length;}
template<> inline int MFTime::countItems(const String& str, int i){throw this;}
template<> inline int MFVec2f::countItems(const String& str, int i){str.stb(i);i++; M_Vec2f dummy; dummy.getField(str,i); return dummy.length;}
template<> inline int MFVec3f::countItems(const String& str, int i){str.stb(i);i++; M_Vec3f dummy; dummy.getField(str,i); return dummy.length;}


//lowLevelCode
float atof2(const char* str){
//	return atof(str);
#ifdef DIRECT3D
    setMathHiPresision();
#endif
    long double num=0.0;
    int i=0;
    long double negFlag=1;
    if(str[i]=='-'){
        negFlag=-1;
        i++;
    }
    while(1){
        if(str[i]=='.'){
            i++;
            float counter=.1;
            while(str[i]>='0'&&str[i]<='9'){
                num+=counter*(float)(str[i]-'0');
                counter*=.1;
                i++;
            }
            if(str[i]=='E'||str[i]=='e'){
                num=0;
                negFlag=0;
            }
            break;
        }
        else if (str[i]>='0'&&str[i]<='9'){
            num*=10.;
            num+=(float)(str[i]-'0');
        }
        else{
            if(str[i]=='E'||str[i]=='e'){
                num=0;
                negFlag=0;
            }
            break;
        }
        i++;
    }
#ifdef DIRECT3D
    restoreMathPresision();
#endif
    return num*negFlag;
}
double atod(const char* str){
#ifdef DIRECT3D
    setMathHiPresision();
#endif
    long double num=0;
    int i=0;
    long double negFlag=1;
    if(str[i]=='-'){
        negFlag=-1;
        i++;
    }
    while(1){
        if(str[i]=='.'){
            i++;
            double counter=.1;
            while(str[i]>='0'&&str[i]<='9'){
                num+=counter*(double)(str[i]-'0');
                counter*=.1;
                i++;
            }
            if(str[i]=='E'||str[i]=='e'){
                num=0;
                negFlag=0;
            }
            break;
        }
        else if (str[i]>='0'&&str[i]<='9'){
            num*=10.;
            num+=(double)(str[i]-'0');
        }
        else{
            if(str[i]=='E'||str[i]=='e'){
                num=0;
                negFlag=0;
            }
            break;
        }
        i++;
    }
#ifdef DIRECT3D
    restoreMathPresision();
#endif
    return num*negFlag;
}

void searchTillNextWord(char*& p){
    while(*p>' '&&*p!=','&&*p!=']')p++;
    if(*p==0)return;
    searchTillText(p);
}
void searchTillNextVec3f(char*& p){
    for(int i=0;i<3;i++){
        searchTillNextWord(p);
    }
}
bool cmpStr(char* str ,char*& p){
    char* p2=p;
    while(*str!='\0'){
        if(*str++!=*p2++){
            return false;
        };
    }
    if(*p2>' '&&*p2!='{'&&*p2!='}'&&*p2!='['&&*p2!=']'){
        return false;
    }
    p=p2;
    return true;
}

int CmpRest(const char* match ,char*& from,int StrLen){
    for(int i=0;i<StrLen&&*match!=0;i++){
        if (*match++!=*from++)return 0;
    }
    return 1;
}
int countUpSingleIntegers(char* p){
    int i=0;
    searchTillText(p);
    p++;
    searchTillText(p);
    while(*p!=']'){
        searchTillNextWord(p);
        i++;
    }
    p++;
    return i;
}

int search(const char* match ,char*& from,char* BufEnd){
    int StrLen=strlen(match);
    while(from<BufEnd){
    if(*from++==*match)
        if(CmpRest(&match[1], from,StrLen-1)){
            return 1;
        }
    }
    return 0;
}
void searchTillText(char*& p){
    while(*p<=' '||*p==','){
        if(*p==0){
            return;
        }
        p++;
    }
    while(*p=='#'){
        search("\n",p, p+0x1000000);
        searchTillText(p);
    }
}
void searchTillText(char*& p, char* charBuffEnd){
    while(*p<=' '||*p==','){
        if(p>=charBuffEnd)return;
        p++;
    }
    while(*p=='#'){
        search("\n",p, p+0x1000000);
        searchTillText(p);
    }
}


bool g_removeFirstZero = true;

void removeZeros(char * str){
    int k;
    if(g_removeFirstZero && str[0] == '0' && str[1]=='.'){
        for(k = 0; str[k] != 0; k++){
            str[k] = str[k + 1];
        }
        k-=2;
        while(k > 0 && str[k] == '0'){
            str[k--] = 0;
            if(str[k] == '.'){
                str[k] = '0';
                break;
            }
        }
    }
    else if(g_removeFirstZero && str[0] == '-' && str[1] == '0' && str[2] == '.'){
        for(k = 1; str[k] != 0; k++){
            str[k] = str[k + 1];
        }
        k-=2;
        while(k > 0 && str[k] == '0'){
            str[k--]=0;
            if(str[k] == '.'){
                str[0] = '0';
                str[1] = 0;
                break;
            }
        }
    }
    else{
        k = strlen(str) - 1;
        while(k > 0 && str[k] == '0'){
            str[k--] = 0;
            if(str[k] == '.'){
                str[k] = 0;
                break;
            }
        }
    }
    k = strlen(str);
    if(k == 2 && str[0] == '-' && str[1] == '0'){
        str[0] = '0';
        str[1] = 0;
    }
}

int countUpVec3f(char* p){
    searchTillText(p);
    if('['!=*p)return -1;
    p++;
    int i=0;
    while(1){
        searchTillText(p);
        if(*p==']')break;
        searchTillNextVec3f(p);
        i++;
    }
    return i;
}
Vec2f getField(char*& p){
    Vec2f v;
    searchTillText(p);
    v.x=atof2(p);
    searchTillNextWord(p);
    v.y=atof2(p);
    searchTillNextWord(p);
    return v;
}


void searchTillNextVec2f(char*& p){
    for(int i=0;i<2;i++){
        searchTillNextWord(p);
    }
}

int countUpVec2f(char* p){
    searchTillText(p);
    if('['!=*p)return -1;
    p++;
    int i=0;
    while(1){
        searchTillText(p);
        if(*p==']')break;
        searchTillNextVec2f(p);
        i++;
    }
    return i;
}
Color3f hexToColor(unsigned hex){
    Color3f color;
    color.b=(hex&0xff)/255.;
    hex>>=8;
    color.g=(hex&0xff)/255.;
    hex>>=8;
    color.r=(hex&0xff)/255.;
    return color;
}
int countWords(char* p){
    int i=0;
    searchTillText(p);
    while(*p!=0){
        searchTillNextWord(p);
        i++;
    }
    return i;
}

float getFloat(const String& str, int& i){
    str.stb(i);
    float value=atof2(str.c_str()+i);
    str.stwo(i);
    return value;
}
float getFloat2(const String& str, int& i){
    str.stb(i);
    float value=atof2(str.c_str()+i);
    str.endOfNum(i);
    return value;
}
double getDouble(const String& str, int& i){
    str.stb(i);
    float value=atod(str.c_str()+i);
    str.endOfNum(i);
    return value;
}
int hexStrToInt(const String& str, int& i){
    int value=0;
    while(true) {
        char ch=str[i];
        if(ch>='0'&&ch<='9'){
            value<<=4;
            value|=str[i]-'0';
        }
        else if(ch>='a'&&ch<='f'){
            value<<=4;
            value|=str[i]-'a'+10;
        }
        else if(ch>='A'&&ch<='F'){
            value<<=4;
            value|=str[i]-'A'+10;
        }
        else {
            break;
        }
        i++;
    }
    return value;
}
int getInt(const String& str, int& i){
    str.stb(i);
    if(str[i]=='0'){
        if(++i<str.length){
            if(str[i]=='x'||str[i]=='X'){
                return hexStrToInt(str, ++i);
            }
        }
        i--;
    }
    int value=atoi(str.c_str()+i);
    str.stwo(i);
    return value;
}
int getInt2(const String& str, int& i){
    str.stb(i);
    if(str[i]=='0'){
        if(++i<str.length){
            if(str[i]=='x'||str[i]=='X'){
                return hexStrToInt(str, ++i);
            }
        }
        i--;
    }
    int value=atoi(str.c_str()+i);
    if(str[i]=='-'){
        i++;
    }
    str.stwo2(i);
    return value;
}
bool getBool(const String& str, int& i){
    str.stb(i);
    if(str.cmpKWord("TRUE",i))return true;
    else if(str.cmpKWord("FALSE",i))return false;
    else throw 0;
}

void String::getField(const String& str, int& i){
    *this=str.fromQuotes(i);
}
template<>
void SFBool::read(const String& str, int& i){
    str.stb(i);
    if		(str.cmpKWord("TRUE",i)) refValue()=true;
    else if	(str.cmpKWord("FALSE",i))refValue()=false;
    else throw 0;
}
void SFColor::read(const String& str, int& i){
    refValue().r=getFloat(str, i);
    str.stb(i);
    refValue().g=getFloat(str, i);
    str.stb(i);
    refValue().b=getFloat(str, i);
}
template<>
void SFFloat::read(const String& str, int& i){
    refValue()=getFloat(str, i);
}
template<>
void SFInt32::read(const String& str, int& i){
    refValue()=getInt(str, i);
}
void SFRotation::read(const String& str, int& i){
    Vec3f axis;
    axis.x=getFloat(str, i);
    str.stb(i);
    axis.y=getFloat(str, i);
    str.stb(i);
    axis.z=getFloat(str, i);
    str.stb(i);
    refValue()=Rotation(axis.normalize(),getFloat(str, i));
}
void SFString::read(const String& str, int& i){
    refValue()=str.fromQuotes(i);
}
template<>
void SFTime::read(const String& str, int& i){
    refValue()=getFloat(str, i);
}
void SFVec2f::read(const String& str, int& i){
    refValue().x=getFloat(str, i);
    str.stb(i);
    refValue().y=getFloat(str, i);
}
void SFVec3f::read(const String& str, int& i){
    refValue().x=getFloat(str, i);
    str.stb(i);
    refValue().y=getFloat(str, i);
    str.stb(i);
    refValue().z=getFloat(str, i);
}
void SFNode::read(const String &str, int &i){
    refValue()=getNode2( str, i);
}

template<>
void MFNode::read(const String &str, int &i){
    NodePtr node;

    bool brackFlag=false;
    str.stb(i);
    if(str[i]=='['){
        brackFlag=true;
        i++;
    }
    checkBackgroundColor(str, i);
//	str.stb(i);
    while(str[i]!=']'){
        refValue()+=getNode2(str, i);
        str.stb(i);
        if(!brackFlag) break;
    }
    if(brackFlag) i++;
}
template<>
void MFString::read(const String& str,int& i){
    str.stb(i);
    String dummy;
    if(str[i]=='['){
        i++;
        str.stb(i);
        int k,ctr;


        for(k=i,ctr=0;str[k]!=']';ctr++){
            dummy.getField(str, k);
            str.stb(k);
        }
        refValue().length=ctr;
        for(ctr=0;str[i]!=']';ctr++){
            refValue()[ctr].getField(str, i);
            str.stb(i);
        }
        i++;
    }
    else{
        dummy.getField(str, i);
        refValue()=dummy;
    }
}
template<>
void MFColor::read(const String& str,int& i){
    str.stb(i);
    Color3f dummy;
    if(str[i]=='['){
        i++;
        str.stb(i);
        int k,ctr;


        for(k=i,ctr=0;str[k]!=']';ctr++){
            dummy.getField(str, k);
            str.stb(k);
        }
        M_Color3f tmpThis; tmpThis.length=ctr;

        for(ctr=0;str[i]!=']';ctr++){
            tmpThis[ctr].getField(str, i);
            str.stb(i);
        }
        refValue()=tmpThis;
        i++;
    }
    else{
        dummy.getField(str, i);
        refValue()=dummy;
    }
}
template<>
void MFBool::read(const String& str,int& i){
    str.stb(i);
    if(str[i]=='['){
        i++;
        str.stb(i);
        int k,ctr;


        for(k=i,ctr=0;str[k]!=']';ctr++){
            getFloat(str, k);
            str.stb(k);
        }
        M_Bool tmpThis; tmpThis.length=ctr;

        for(ctr=0;str[i]!=']';ctr++){
            tmpThis[ctr]=getBool(str, i);
            str.stb(i);
        }
        refValue()=tmpThis;
        i++;
    }
    else{
        refValue()=getBool(str, i);
    }
}
template<>
void MFFloat::read(const String& str,int& i){
    str.stb(i);
    if(str[i]=='['){
        i++;
        str.stb(i);
        int k,ctr;


        for(k=i,ctr=0;str[k]!=']';ctr++){
            getFloat(str, k);
            str.stb(k);
        }
        M_Float tmpThis; tmpThis.length=ctr;

        for(ctr=0;str[i]!=']';ctr++){
            tmpThis[ctr]=getFloat(str, i);
            str.stb(i);
        }
        refValue()=tmpThis;
        i++;
    }
    else{
        refValue()=getFloat(str, i);
    }
}
template<>
void MFInt32::read(const String& str, int& i){
    str.stb(i);
    refValue().length=0;
    if(str[i]=='['){
        i++;
        str.stb(i);
        int k,ctr;

        for(k=i,ctr=0;str[k]!=']';ctr++){
            getInt(str, k);
            str.stb(k);
        }
        M_Int tmpThis; tmpThis.length=ctr;

        for(ctr=0;str[i] != ']'; ctr++){
            tmpThis[ctr] = getInt(str, i);
            str.stb(i);
        }
        refValue()=tmpThis;
        i++;
    }
    else{
        refValue()=getInt(str,i);
    }
}

template<>
void MFRotation::read(const String& str,int& i){
    str.stb(i);
    Rotation tmp;
    if(str[i]=='['){
        i++;
        str.stb(i);
        for(;str[i]!=']';){
            tmp.getField(str, i);
            refValue()+=(tmp);
            str.stb(i);
        }
        i++;
    }
    else{
        tmp.getField(str, i);
        refValue()=tmp;
    }
}
template<>
void MFTime::read(const String& str,int& i){
    str.stb(i);
    if(str[i]=='['){
        i++;
        str.stb(i);
        int k,ctr;


        for(k=i,ctr=0;str[k]!=']';ctr++){
            getFloat(str, k);
            str.stb(k);
        }
        M_Double tmpThis; tmpThis.length=ctr;

        for(ctr=0;str[i]!=']';ctr++){
            tmpThis[ctr]=getFloat(str, i);
            str.stb(i);
        }
        refValue()=tmpThis;
        i++;
    }
    else{
        refValue()=getFloat(str, i);
    }
}
template<>
void MFVec2f::read(const String& str,int& i){
    str.stb(i);
    Vec2f dummy;
    if(str[i]=='['){
        i++;
        str.stb(i);
        int k,ctr;


        for(k=i,ctr=0;str[k]!=']';ctr++){
            dummy.getField(str, k);
            str.stb(k);
        }
        M_Vec2f tmpThis; tmpThis.length=ctr;

        for(ctr=0;str[i]!=']';ctr++){
            tmpThis[ctr].getField(str, i);
            str.stb(i);
        }
        refValue()=tmpThis;
        i++;
    }
    else{
        dummy.getField(str, i);
        refValue()=dummy;
    }
}
template<>
void MFVec3f::read(const String& str,int& i){
    str.stb(i);
    Vec3f dummy;
    if(str[i]=='['){
        i++;
        str.stb(i);
        int k,ctr;


        for(k=i,ctr=0;str[k]!=']';ctr++){
            dummy.getField(str, k);
            str.stb(k);
        }
        M_Vec3f tmpThis; tmpThis.length=ctr;

        for(ctr=0;str[i]!=']';ctr++){
            tmpThis[ctr].getField(str, i);
            str.stb(i);
        }
        refValue()=tmpThis;
        i++;
    }
    else{
        dummy.getField(str, i);
        refValue()=dummy;
    }
}

void String::stw_stb(int& position)const{
    stw(position);
    stb(position);
}
void String::stwo_stb(int& position)const{
    stwo(position);
    stb(position);
}

bool String::cmpKWord(const char* str,int& index)const{
    const char* p2=c_str()+index;
    while(*str!='\0'){
        if(*str++!=*p2++){
            return false;
        };
    }
    if(*p2>' '&&*p2!=','&&*p2!='{'&&*p2!='}'&&*p2!='['&&*p2!=']'&&*p2!='#'&&*p2!='\"'){
        return false;
    }
    index= p2-c_str();
    return true;
}

int String::indexOf( String searchString, int position)const{
    char* match=searchString.pointer->rawChar;
    char* from=&pointer->rawChar[position];
    int newLength=pointer->rawLen-position;
    for(int i=0;i<newLength;i++){
        if(*from++==*match){
        char* match2=match+1;
        if(*match2==0)return position+i;
        char* from2=from;
            for(int k=i;k<newLength;k++){
                if (*match2++!=*from2++)break;
                if(*match2==0)return position+i;
            }

        }
    }
    return -1;
}

String String::extractName(int& i)const{
    stb(i);
    int start = i;
    endOfName(i);
    return substr(start,i-start);
}

void String::stwo(int& position)const{
    char* rawChar=pointer->rawChar;
    char* p=rawChar+position;
    char* endAddress=rawChar+pointer->rawLen;
    while(
        (p  <	endAddress	&& *p == '_') ||
        (*p == '-' || *p	== '+'|| *p	== '.')|| 	//+ - . needed for legacy reasons ???????
        (*p >= '0' && *p	<= '9')||
        (*p >= 'A' && *p	<= 'Z')||
        (*p >= 'a' && *p	<= 'z')
        ){
        p++;
    }
    position=p-rawChar;
}
void String::stwo2(int& position)const{
    char* rawChar=pointer->rawChar;
    char* p=rawChar+position;
    char* endAddress=rawChar+pointer->rawLen;
    while(
        (p  <	endAddress	&& *p == '_')||
        (*p >= '0' && *p	<= '9')||
        (*p >= 'A' && *p	<= 'Z')||
        (*p >= 'a' && *p	<= 'z')
        ){
        p++;
    }
    position=p-rawChar;
}
String String::fromQuotes(int& position)const{
    stb(position);
    if((*this)[position++]!='\"')throw this;

    int begin=position;

    while((*this)[position++]!='\"');
    return substr( begin, position-begin-1);
}


/*
void M_RedirectCoord::getField(const String& str,int& i){
    str.stb(i);
    if(str[i]=='['){
        i++;
        str.stb(i);
        int k,ctr;

        for(k=i,ctr=0;str[k]!=']';ctr++){
            getInt(str, k);
            str.stb(k);
        }
        length=ctr;

        for(ctr=0;str[i]!=']';ctr++){
            length.pointer[ctr].i=getInt(str, i);
            str.stb(i);
        }
        i++;
    }
    else{
        length=1;
        length.pointer[0].i=getInt(str, i);
    }
}
*/
template<>
void M_Bool::toString(String& str)const{
    if(length==1){
        str+=length.pointer[0];
    }
    else {
        str+=g_vrmlSquareOpenBracket;
        for(int i=0;i<length;i++){
            str+=length.pointer[i];
            str+=" ";
        }
        str+=g_vrmlSquareCloseBracket;
    }
}
void String::toString(String& str){
    str+='\"';
    str+=pointer->rawChar;
    str+='\"';
}
template<>
void M_String::toString(String& str)const{
    if(length==1){
        str+='\"';
        str+=length.pointer[0];
        str+='\"';
    }
    else {
        str+=g_vrmlSquareOpenBracket;
        if(length){
            int i=0;
            while(1){
                str+='\"';
                str+=length.pointer[i];
                str+='\"';
                i++;
                if(i<length){
                    str+=" ";
                }
                else break;
            }
        }
        str+=g_vrmlSquareCloseBracket;
    }
}
String& String::operator= (const char& par){
    unsigned lEn=1;

    if(pointer->refCount>1||lEn>pointer->bufLen){
        if(--pointer->refCount==0){
            delete pointer;
        }
        pointer=createData(lEn+defaultBufLen);
    }
    pointer->rawLen=lEn;
    pointer->rawChar[0]=par;
    return *this;
}
String& String::operator= (float par1){
    char tmp[50];
    sprintf(tmp,"%.6f",par1);
    removeZeros(tmp);
    unsigned lEn=strlen(tmp);

    if(pointer->refCount>1||lEn>pointer->bufLen){
        if(--pointer->refCount==0){
            delete pointer;
        }
        pointer=createData(lEn+defaultBufLen);
    }
    pointer->rawLen=lEn;
    for(unsigned i=0;i<pointer->rawLen+1;i++)pointer->rawChar[i]=tmp[i];
    return *this;
}
String& String::operator= (int par1){
    char tmp[50];
    sprintf(tmp,"%i",par1);
    unsigned lEn=strlen(tmp);

    if(pointer->refCount>1||lEn>pointer->bufLen){
        if(--pointer->refCount==0){
            delete pointer;
        }
        pointer=createData(lEn+defaultBufLen);
    }
    pointer->rawLen=lEn;
    for(unsigned i=0;i<pointer->rawLen+1;i++)pointer->rawChar[i]=tmp[i];
    return *this;
}
void String::cat(const String &par1){
    unsigned i;
    unsigned newLength=pointer->rawLen+par1.pointer->rawLen;
    if(pointer->refCount>1||newLength>pointer->bufLen){

        Data* newPointer=createData( newLength+defaultBufLen);
        newPointer->rawLen=pointer->rawLen;
        for(i=0;i<pointer->rawLen+1;i++){
            newPointer->rawChar[i]=pointer->rawChar[i];
        }

        if(--pointer->refCount==0){
            delete pointer;
        }
        pointer=newPointer;
    }

    for(i=0;i<par1.pointer->rawLen+1;i++){
        pointer->rawChar[pointer->rawLen+i]=par1.pointer->rawChar[i];
    }
    pointer->rawLen=newLength;
}
void String::cat(const char& par1){
    Data* newPointer;
    unsigned i;
    if(pointer->refCount>1){
        --pointer->refCount;
        newPointer=createData( pointer->rawLen+defaultBufLen);
        newPointer->rawLen=pointer->rawLen;
        for(i=0; i<pointer->rawLen+1; i++){
            newPointer->rawChar[i]=pointer->rawChar[i];
        }
        pointer=newPointer;
    }
    else if(pointer->rawLen==pointer->bufLen){
        newPointer=createData( pointer->rawLen+defaultBufLen);
        newPointer->rawLen=pointer->rawLen;
        for(i=0; i<pointer->rawLen+1; i++){
            newPointer->rawChar[i]=pointer->rawChar[i];
        }
        if(--pointer->refCount==0){
            delete pointer;
        }
        pointer=newPointer;
    }
    pointer->rawChar[pointer->rawLen++]=par1;
    pointer->rawChar[pointer->rawLen]=0;
}
void String::cat(const unsigned char& par){
    cat(static_cast<const char>(par));
}
//needed gzip 0


bool String::findWord(const String str, int& i)const{
    do{
        i=indexOf(str,i);
        int j=i;
        if(cmpKWord(str,j))return true;
        i++;
    }while(i<length);
    return false;
}

void String::stnl(int& position)const{
    unsigned char* p=(unsigned char*)&pointer->rawChar[position];
    while(*p!='\n'){
        p++;
    }
    while(*p<' '){
        p++;
    }
    position=p-(unsigned char*)pointer->rawChar;
}

void String::stb(int& position)const{
    unsigned char* p=(unsigned char*)&pointer->rawChar[position];
    int newLength=pointer->rawLen-position-1;
    for(int i=0;i<newLength;i++){
        if(*p>' '&&*p!=','){
            if(*p=='#'){
                while(*p>=' '||*p=='\t')p++;//search for new line
            }
            else{
                break;
            }
        }
        p++;
    }
    position=p-(unsigned char*)pointer->rawChar;
}
void String::stbCpp(int& position)const{
    unsigned char* p=(unsigned char*)&pointer->rawChar[position];
    int newLength=pointer->rawLen-position-1;
    for(int i=0;i<newLength;i++){
        if(*p>' '){
            if(*p=='/'&&p[1]=='/'){
                p++;
                while(*p>=' '||*p=='\t')p++;//seerch for new line
            }
            else{
                break;
            }
        }
        p++;
    }
    position=p-(unsigned char*)pointer->rawChar;
}
void String::stbnlCpp(int& position)const{
    unsigned char* p=(unsigned char*)&pointer->rawChar[position];
    int newLength=pointer->rawLen-position-1;
    for(int i=0;i<newLength;i++){
        if( *p=='\n'){
            break;
        }
        else if(*p>' '){
            if(*p=='/'&&p[1]=='/'){
                p++;
                while(*p>=' '||*p=='\t')p++;//search for new line
            }
            else{
                break;
            }
        }
        p++;
    }
    position=p-(unsigned char*)pointer->rawChar;
}
void String::stbc(int& position)const{
    unsigned char* p=(unsigned char*)&pointer->rawChar[position];
    int newLength=pointer->rawLen-position-1;
    for(int i=0;i<newLength;i++){
        if(*p>' '&&*p!=','){
            break;
        }
        p++;
    }
    position=p-(unsigned char*)pointer->rawChar;
}

void String::stw(int& position)const{
    char* p=&pointer->rawChar[position];
    int newLength=pointer->rawLen-position;
    for(int i=0;i<newLength;i++){
        if(*p<=' '||*p==','){
            position=p-pointer->rawChar;
            return;
        }
        p++;
    }
    throw this;
}

void String::endOfName(int& index)const{
    char* p=&pointer->rawChar[index];
    int newLength=pointer->rawLen-index;
    int i=0;
    while(i < newLength	&& isLegalNameChar(*p) ){
        p++;
    }
    index=p-pointer->rawChar;
}
void String::endOfNum(int& position)const{
    char* rawChar=pointer->rawChar;
    char* p=rawChar+position;
    char* endAddress=rawChar+pointer->rawLen;
    int dotCounter=0;
    if(*p == '-'){
        p++;
        while (p<endAddress && *p<=' '){
            p++;
        }
    }
    if(*p == '.'){
        dotCounter++;
        p++;
    }
    while(p < endAddress){
here:
        if(*p=='E'||*p=='e'){
            p++;
            p++;
            while(p < endAddress && *p >= '0' && *p	<= '9'){
                p++;
            }
            break;
        }
        if(*p < '0' || *p	> '9')break;
        p++;
    }
    if(p < endAddress && *p=='.' && dotCounter==0){
        p++;
        goto here;
    }
    position=p-rawChar;
}
bool String::cmpKWord(const String& str,int& index)const{
    return cmpKWord(str.pointer->rawChar, index);
}
String String::getWord(int& i)const{
    stb(i);
    int start=i;
    stwo(i);
    return substring(start,i-1);
}
String String::getWord2(int& i)const{
    stb(i);
    int start=i;
    stwo2(i);
    return substring(start,i-1);
}


template <> Vec2f Vec2f::normalize()const{
    float dist = sqrt(x * x + y * y);
    if(dist == 0)	return Vec2f(0, 0);
    float sc = 1. / dist;

    return (*this) * sc;
}

template <class T> Vec3<T> Vec3<T>::normalize()const{
    float dist=sqrt(x * x + y * y + z * z);
    if(dist==0)	return Vec3<T>(0, 0, 0);
    float sc = 1./dist;

    return (*this) * sc;
}

void Node::getFields(const String &str,int &i){
    str.stb(i);

    M_Properties &properties=g_lookupProtoInterface[getType()]->properties;

    while(str[i]!='}'){
        int fieldIdx = 0;
        for(; fieldIdx < getFLen(); fieldIdx++){
            if(str.cmpKWord(properties[fieldIdx].name,i)){
                getField(fieldIdx, str, i);
                break;
            }
        }
		str.stb(i);
    }  
}

void Node::getNode(const String& str,int& i){
	checkBackgroundColor(str, i);
//	str.stb(i);
	if(str[i]!='{')throw this;
	i++;
	str.stb(i);
	while(str[i]!='}'){
		getFields(str, i);
		str.stb(i);
	}
	i++;
	checkBackgroundColor(str, i);
}

bool equals(const char* pa, const char* pb, const int lenA, const int lenB){
    if(lenA != lenB){
        return false;
    }
    for(int i = 0; i < lenA; i++){
        if(pa[i] != pb[i]){
            return false;
        }
    }
    return true;
}
bool notEquals(const char* pa, const char* pb, const int lenA, const int lenB){
    if(lenA != lenB){
        return true;
    }
    for(int i = 0; i < lenA; i++){
        if(pa[i] != pb[i]){
            return true;
        }
    }
    return false;
}
bool greaterThan(const char* pa, const char* pb, const int lenA, const int lenB){
    int len = lenA;

    if(lenA > lenB){
        len = lenB;
    }
    for(int i = 0; i < len; i++){
        char a = pa[i];
        char b = pb[i];
        if(a == b){
            continue;
        }
        return a > b;
    }
    return false;
}
bool lessThan(const char* pa, const char* pb, const int lenA, const int lenB){
    int len = lenA;

    if(lenA > lenB){
        len = lenB;
    }
    for(int i = 0; i < len; i++){
        char a = pa[i];
        char b = pb[i];
        if(a == b){
            continue;
        }
        return a < b;
    }
    return false;
}
bool greaterEqual(const char* pa, const char* pb, const int lenA, const int lenB){
    if(greaterThan(pa, pb, lenA, lenB)){
        return true;
    }
    return equals(pa, pb, lenA, lenB);
}
bool lessEqual(const char* pa, const char* pb, const int lenA, const int lenB){
    if(lessThan(pa, pb, lenA, lenB)){
        return true;
    }
    return equals(pa, pb, lenA, lenB);
}

inline void Node::getField(const int& fieldIdx, const String& str,int& i){
    checkBackgroundColor(str, i);
    getField(fieldIdx)->read(str,i);
}

void createSmlsFromString(M_Node& nodes, const String& str){
    g_bColor=Color3f();
    g_defLookUp.length=0;
    g_protoDeclare=NULL;
    int i=0;

    String& str2=const_cast<String&>(str);
    str2.requestMoreIfNeeded(2);
    char* p=const_cast<char*>(str2.c_str());
    p[str2.length+1]=']'; //insert after end of str charecter

    getNodesFromStr(nodes, str2, i);

}

String readFile(String filename){
    static int maxBufSize=0x4000;
    if(filename=="") return "";
    int bufSize=0;
    char* buf;

    FILE* file;


    while(1){
        buf=new char[maxBufSize+2];
        if((file = fopen(filename.c_str(), "rb"))!=NULL){
            bufSize = fread(buf, 1, maxBufSize, file);
            fclose(file);
            if(bufSize==maxBufSize){
                maxBufSize*=2;
                delete buf;
            }
            else break;
        }
        else {
            break;
        }
    }
    buf[bufSize]=0;
    return buf;
}

//http://www.3dkingdoms.com/weekly/quat.h
//http://forum.onlineconversion.com/showthread.php?t=5408
//assumes Quaternion is normalised
Vec3d Quaternion::eulerYZX(){ //adpated from http://www.euclideanspace.com/maths/geometry/rotations/conversions/quaternionToEuler/index.htm
    double x2 = x * x;
    double y2 = y * y;
    double z2 = z * z;
    double test= x * y + z * w;
    if(test > 0.49999999) {
        return Vec3d (atan2(x, w), PI*.5, 0);
    }
    else if(test < -0.49999999) {
        return Vec3d(-atan2(x, w), -PI*.5, 0);
    }
    return Vec3d(
        atan2(2 * y * w - 2 * x * z, 1 - 2 * y2 - 2 * z2),
        asin(2 * x * y + 2 * z * w),
        atan2(2 * x * w - 2 * y * z, 1 - 2 * x2 - 2 * z2)
    );
}



//Some quaternion/rotation maths C code that has been adapted was originally written by: Gavin Bell

Quaternion invert(const Quaternion &srcQuat){
    Quaternion  destQuat;
    float srcQuatNorm;

    srcQuatNorm = 1.0 / (srcQuat.x*srcQuat.x + srcQuat.y*srcQuat.y + srcQuat.z*srcQuat.z + srcQuat.w*srcQuat.w);

    destQuat.x = -srcQuat.x * srcQuatNorm;
    destQuat.y = -srcQuat.y * srcQuatNorm;
    destQuat.z = -srcQuat.z * srcQuatNorm;
    destQuat.w =  srcQuat.w * srcQuatNorm;
    return(destQuat);

}

void rotationToQuaternion(const Rotation &rot, Quaternion &result){
    result.x = rot.x*sin(rot.angle / 2.0);
    result.y = rot.y*sin(rot.angle / 2.0);
    result.z = rot.z*sin(rot.angle / 2.0);
    result.w = cos(rot.angle / 2.0);
}

void rotationToQuaternion(const Rotation4d &rot, Quaternion &result){
    result.x = rot.x*sin(rot.angle / 2.0);
    result.y = rot.y*sin(rot.angle / 2.0);
    result.z = rot.z*sin(rot.angle / 2.0);
    result.w = cos(rot.angle / 2.0);
}

void quaternionToRotation(Quaternion q, Rotation& rot){

    if(q.w>1.0L){
        q.w=1;
    }
    rot.angle = acos(q[3]) * 2.0;
    if(rot.angle==0){
        rot=Rotation();
    }
    else{
        rot.x = q[0] / sin(rot.angle / 2.0);
        rot.y = q[1] / sin(rot.angle / 2.0);
        rot.z = q[2] / sin(rot.angle / 2.0);
    }
}

void quaternionToRotation(Quaternion q, Rotation4d& rot){

    if(q.w>1.0L){
        q.w=1;
    }
    rot.angle = acos(q[3]) * 2.0;
    if(rot.angle==0){
        rot=Rotation4d();
    }
    else{
        rot.x = q[0] / sin(rot.angle / 2.0);
        rot.y = q[1] / sin(rot.angle / 2.0);
        rot.z = q[2] / sin(rot.angle / 2.0);
    }
}

void multiplyQuaternion(Quaternion& q1, Quaternion& q2, Quaternion& result){
    float tmp[4];
    tmp[0] = q2[3] * q1[0] + q2[0] * q1[3] + q2[1] * q1[2] - q2[2] * q1[1];
    tmp[1] = q2[3] * q1[1] + q2[1] * q1[3] + q2[2] * q1[0] - q2[0] * q1[2];
    tmp[2] = q2[3] * q1[2] + q2[2] * q1[3] + q2[0] * q1[1] - q2[1] * q1[0];
    tmp[3] = q2[3] * q1[3] - q2[0] * q1[0] - q2[1] * q1[1] - q2[2] * q1[2];
    result[0] = tmp[0];
    result[1] = tmp[1];
    result[2] = tmp[2];
    result[3] = tmp[3];
}

Rotation::Rotation (Vec3f fromVector, Vec3f toVector){
//
////////////////////////////////////////////////////////////////////////
    Quaternion quat;
    Vec3f	from = fromVector.normalize();
    Vec3f	to = toVector.normalize();
    Vec3f	axis;
    float	cost;

    cost = from.dot(to);

    // check for degeneracies
    if (cost > 0.99999f) {		// vectors are parallel
        quat[0] = quat[1] = quat[2] = 0.0f;
        quat[3] = 1.0f;
        quaternionToRotation(quat, *this);
        return;
    }
    else if (cost < -0.99999f) {		// vectors are opposite
        // find an axis to rotate around, which should be
        // perpendicular to the original axis
        // Try cross product with (1,0,0) first, if that's one of our
        // original vectors then try  (0,1,0).
        Vec3f tmp = from.cross(Vec3f(1.0f, 0.0f, 0.0f));
        if (tmp.length() < 0.00001f){
            tmp = from.cross(Vec3f(0.0f, 1.0f, 0.0f));
        }

        tmp=tmp.normalize();
        quaternionToRotation(Quaternion(tmp.x, tmp.y, tmp.z, 0.0f), *this);
        return;
    }

    axis = fromVector.cross(toVector);
    axis=axis.normalize();

    // use half-angle formulae
    // sin^2 t = ( 1 - cos (2t) ) / 2
    axis *= sqrt(0.5f * (1.0f - cost));

    // scale the axis by the sine of half the rotation angle to get
    // the normalized quaternion
    quat[0] = axis.x;
    quat[1] = axis.y;
    quat[2] = axis.z;

    // cos^2 t = ( 1 + cos (2t) ) / 2
    // w part is cosine of half the rotation angle
    quat[3] = sqrt(0.5f * (1.0f + cost));

    quaternionToRotation(quat, *this);
}

Rotation4d::Rotation4d (Vec3d fromVector, Vec3d toVector){
//
////////////////////////////////////////////////////////////////////////
    Quaternion quat;
    Vec3d	from = fromVector.normalize();
    Vec3d	to = toVector.normalize();
    Vec3d	axis;
    double	cost;

    cost = from.dot(to);

    // check for degeneracies
    if (cost > 0.99999999999) {		// vectors are parallel
        quat[0] = quat[1] = quat[2] = 0.0f;
        quat[3] = 1.0f;
        quaternionToRotation(quat, *this);
        return;
    }
    else if (cost < -0.99999999999) {		// vectors are opposite
        // find an axis to rotate around, which should be
        // perpendicular to the original axis
        // Try cross product with (1,0,0) first, if that's one of our
        // original vectors then try  (0,1,0).
        Vec3d tmp = from.cross(Vec3d(1.0f, 0.0f, 0.0f));
        if (tmp.length() < 0.00001f){
            tmp = from.cross(Vec3d(0.0, 1.0, 0.0));
        }

        tmp=tmp.normalize();
        quaternionToRotation(Quaternion(tmp.x, tmp.y, tmp.z, 0.0f), *this);
        return;
    }

    axis = fromVector.cross(toVector);
    axis=axis.normalize();

    // use half-angle formulae
    // sin^2 t = ( 1 - cos (2t) ) / 2
    axis *= sqrt(0.5 * (1.0 - cost));

    // scale the axis by the sine of half the rotation angle to get
    // the normalized quaternion
    quat[0] = axis.x;
    quat[1] = axis.y;
    quat[2] = axis.z;

    // cos^2 t = ( 1 + cos (2t) ) / 2
    // w part is cosine of half the rotation angle
    quat[3] = sqrt(0.5f * (1.0f + cost));

    quaternionToRotation(quat, *this);
}


Rotation Rotation::operator -()const{
    Rotation result;
    Quaternion quat;
    rotationToQuaternion(*this, quat);
    quat=invert(quat);
    quaternionToRotation(quat, result);
    result.setAxis(result.getAxis().normalize());

    return result;
}

Rotation4d Rotation4d::operator -()const{
    Rotation4d result;
    Quaternion quat;
    rotationToQuaternion(*this, quat);
    quat=invert(quat);
    quaternionToRotation(quat, result);
    result.setAxis(result.getAxis().normalize());

    return result;
}

Rotation Rotation::operator *=(const Rotation &par1){
    Quaternion q1;
    Quaternion q2;
    Quaternion q3;

    rotationToQuaternion(*this, q1);
    rotationToQuaternion(par1, q2);
    multiplyQuaternion(q1, q2, q3);
    quaternionToRotation(q3, *this);
    setAxis(getAxis().normalize());

    return *this;
}

Rotation Rotation::operator *(const Rotation &par1)const{
    Quaternion q1;
    Quaternion q2;
    Quaternion q3;
    Rotation result;

    rotationToQuaternion(*this, q1);
    rotationToQuaternion(par1, q2);
    multiplyQuaternion(q1, q2, q3);
    quaternionToRotation(q3, result);
    result.setAxis(result.getAxis().normalize());

    return result;
}

Rotation4d Rotation4d::operator *(const Rotation4d &par1)const{
    Quaternion q1;
    Quaternion q2;
    Quaternion q3;
    Rotation4d result;

    rotationToQuaternion(*this, q1);
    rotationToQuaternion(par1, q2);
    multiplyQuaternion(q1, q2, q3);
    quaternionToRotation(q3, result);
    result.setAxis(result.getAxis().normalize());

    return result;
}

Rotation Rotation::multiply(const Rotation &par1)const{
    Quaternion q1;
    Quaternion q2;
    Quaternion q3;
    Rotation result;

    rotationToQuaternion(*this, q1);
    rotationToQuaternion(par1, q2);
    multiplyQuaternion(q1, q2, q3);
    quaternionToRotation(q3, result);
    result.setAxis(result.getAxis().normalize());

    return result;
}

Vec3f Rotation::getAxis()const{
    return Vec3f(x, y, z);
}

Vec3d Rotation4d::getAxis()const{
    return Vec3d(x, y, z);
}

void rotateVectorX(Vec3f& v,float angle){
    float newY;
    newY = (float)(v.y * cos(angle) - v.z * sin(angle));
    v.z = (float)(v.y * sin(angle) + v.z * cos(angle));
    v.y=newY;
}

void rotateVectorX(Vec3d& v,float angle){
    double newY;
    newY = v.y * cos(angle) - v.z * sin(angle);
    v.z = v.y * sin(angle) + v.z * cos(angle);
    v.y=newY;
}

void rotateVectorY(Vec3f& v,float angle){
    float newZ;
    newZ = (float)(v.z * cos(angle) - v.x * sin(angle));
    v.x = (float)(v.z * sin(angle) + v.x * cos(angle));
    v.z=newZ;
}

void rotateVectorY(Vec3d& v, double angle){
    double newZ;
    newZ = v.z * cos(angle) - v.x * sin(angle);
    v.x = v.z * sin(angle) + v.x * cos(angle);
    v.z = newZ;
}

void rotateVectorZ(Vec3f& v,float angle){
    float newX;
    newX=(float)(v.x * cos(angle) - v.y * sin(angle));
    v.y=(float)(v.x * sin(angle) + v.y * cos(angle));
    v.x = newX;
}

void rotateVectorZ(Vec3d& v, double angle){
    double newX;
    newX = v.x * cos(angle) - v.y * sin(angle);
    v.y = v.x * sin(angle) + v.y * cos(angle);
    v.x = newX;
}

Vec3f Rotation::operator *(const Vec3f& vec)const{
        Vec3f axis=getAxis(); //put the axis rotation into the vector axis

        //unwind the axis to 1 0 0
        float zAngle=atan2(axis.y,axis.x);	  //get the z angle necessary for unrotating the axis
        rotateVectorZ(axis,-zAngle);

        float yAngle=-atan2( axis.z, axis.x);	//get the y angle necessary for unrotating the axis
        rotateVectorY( axis, -yAngle);
        //axis should allways = 1 0 0 here because we have unwound the vector so that it points to the right, that is the axis the x axis or (1 0 0)

        //rotate the second VisibleObject only on the x y and x rotations to simulate rotating an object on any given axis to any given angle
        Vec3f retvec=vec;
        rotateVectorZ(retvec,-zAngle);//unwind vec to the x axis
        rotateVectorY(retvec,-yAngle);

        rotateVectorX(retvec, angle);//rotate the vector on the x axis with the angle component exactly the same as if it was rotated on any axis

        rotateVectorY(retvec, yAngle);// wind vec back to the specified axis
        rotateVectorZ(retvec, zAngle);
        return retvec;
}

Vec3d Rotation4d::operator *(const Vec3d& vec)const{
        Vec3d axis=getAxis(); //put the axis rotation into the vector axis

        //unwind the axis to 1 0 0
        double zAngle=atan2(axis.y,axis.x);	  //get the z angle necessary for unrotating the axis
        rotateVectorZ(axis,-zAngle);

        double yAngle=-atan2( axis.z, axis.x);	//get the y angle necessary for unrotating the axis
        rotateVectorY(axis, -yAngle);
        //axis should allways = 1 0 0 here because we have unwound the vector so that it points to the right, that is the axis the x axis or (1 0 0)

        //rotate the second VisibleObject only on the x y and x rotations to simulate rotating an object on any given axis to any given angle
        Vec3d retvec=vec;
        rotateVectorZ(retvec,-zAngle);//unwind vec to the x axis
        rotateVectorY(retvec,-yAngle);

        rotateVectorX(retvec, angle);//rotate the vector on the x axis with the angle component exactly the same as if it was rotated on any axis

        rotateVectorY(retvec, yAngle);// wind vec back to the specified axis
        rotateVectorZ(retvec, zAngle);
        return retvec;
}


// Construct a rotation of a given angle about a given axis.
// Derived from Eric Haines's SPD (Standard Procedural
// Database).
Matrix rotationMatrix(Rotation rot){
    float angle=rot.angle;
    Vec3f axis=rot.getAxis();
    Matrix M;

    if (angle == 0.0) {
        return Matrix();
    }
    axis.normalize();

    float cosine = cos(angle);
    float sine = sin(angle);
    float one_minus_cosine = 1 - cosine;

    M.m[0][0] = axis.x * axis.x + (1.0 - axis.x * axis.x) * cosine;
    M.m[0][1] = axis.x * axis.y * one_minus_cosine + axis.z * sine;
    M.m[0][2] = axis.x * axis.z * one_minus_cosine - axis.y * sine;
    M.m[0][3] = 0;

    M.m[1][0] = axis.x * axis.y * one_minus_cosine - axis.z * sine;
    M.m[1][1] = axis.y * axis.y + (1.0 - axis.y * axis.y) * cosine;
    M.m[1][2] = axis.y * axis.z * one_minus_cosine + axis.x * sine;
    M.m[1][3] = 0;

    M.m[2][0] = axis.x * axis.z * one_minus_cosine + axis.y * sine;
    M.m[2][1] = axis.y * axis.z * one_minus_cosine - axis.x * sine;
    M.m[2][2] = axis.z * axis.z + (1.0 - axis.z * axis.z) * cosine;
    M.m[2][3] = 0;

    M.m[3][0] = 0;
    M.m[3][1] = 0;
    M.m[3][2] = 0;
    M.m[3][3] = 1;

    return M;
}

Matrix scaleMatrix(Vec3f sc){
    Matrix mat;
    mat.f11 = mat.f22 = mat.f33 = mat.f44 = 1.0f;
    mat.f12 = mat.f13 = mat.f14 = mat.f41 = 0.0f;
    mat.f21 = mat.f23 = mat.f24 = mat.f42 = 0.0f;
    mat.f31 = mat.f32 = mat.f34 = mat.f43 = 0.0f;
    mat.f11= sc.x;
    mat.f22= sc.y;
    mat.f33= sc.z;
    return mat;
}
Matrix Matrix::operator *(const Matrix mat)const{
    Matrix tmp;

    for (int i=0; i<4; i++){
        for (int j=0; j<4; j++){
            tmp.m[i][j]=0;
            for (int k=0; k<4; k++){
                tmp.m[i][j]+=this->m[i][k] * mat.m[k][j];
            }
        }
    }
    return tmp;
}
