/* Seamless3d

Copyright (C) 2001-2015 Graham John Perrett (AKA thyme) thyme@seamless3d.com www.seamless3d.com

Permission is hereby granted, free of charge, to any person obtaining a copy of this
software and associated documentation files (the "Software"), to deal in the Software
without restriction, including without limitation the rights to use, copy, modify,
merge, publish, distribute, sublicense, and/or sell copies of the Software, and to
permit persons to whom the Software is furnished to do so, subject to the following
conditions:

The above copyright notice and this permission notice shall be included in all copies or
substantial portions of the Software.

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR IMPLIED,
INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR
PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE
LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT
OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR
OTHER DEALINGS IN THE SOFTWARE.
*/

#define SEAMLESS3DGUI
//#define DIRECT3D
#define NOZLIB

//#define BIND_DEVELOPER_SCRIPT 1

#define PROG_ROB 1
//#define DEMO_TO_MOVIE
#define RECORD_SOUNDMOVIE 1

#ifndef NOZLIB
#include "zlib.h"
#endif

#ifdef WIN32
#pragma warning (disable : 4244 4305 4100 4355 4996)
#endif
#include <stdio.h>
#include <math.h>
#include <string.h>
#include <stdlib.h>

#define PI 3.14159265358979323846
#define TINY_AMOUNT 0.00000001
#define FIELDLOOKUPLEN 22
#define LOOKUPLEN 264
//extern const char nodeLookUpStr[LOOKUPLEN][24];
extern char floatFormat[];
extern char floatFormat2[];
extern char g_vrmlSquareOpenBracket[];
extern char g_vrmlSquareCloseBracket[];
extern char g_vrmlQuote[];
extern const char g_exposedField[];
extern const char g_eventIn[];
extern const char g_eventOut[];
extern const char g_field[];
extern const char g_inputOutput[];
extern const char g_inputOnly[];
extern const char g_outputOnly[];
extern const char g_initializeOnly[];
extern const char* g_exposedFieldPtr;
extern const char* g_eventInPtr;
extern const char* g_eventOutPtr;
extern const char* g_fieldPtr;


#define nullPtr 0
extern char g_exceptioninNode[];


enum FieldType{
    SFBOOL,
    SFCOLOR,
    SFFLOAT,
    SFIMAGE,
    SFINT32,
    SFNODE,
    SFROTATION,
    SFSTRING,
    SFTIME,
    SFVEC2F,
    SFVEC3F,
    MFBOOL,
    MFCOLOR,
    MFFLOAT,
    MFIMAGE,
    MFINT32,
    MFNODE,
    MFROTATION,
    MFSTRING,
    MFDOUBLE,
    MFVEC2F,
    MFVEC3F,

    MROUTE,
    MFPROPERTIES,
    MFTRIANGLE,
    MFPART
};

class S_Triangle;

class Properties;
class Node;
class Art;
class ProtoInterface;
class Field;

class String;
class Matrix;
class Single{};
class Global{};
class Multiple{};
class Local{};

class BIProto;
class BIField;

template <class T>class Ptr;
typedef Ptr <Node> NodePtr;
typedef Ptr <ProtoInterface>			ProtoInterfacePtr;

template <typename T > class M_;
template <class T, class S = Single> class FD;
template <class T, class S = Single, class V = Global> class V_;



#ifdef DIRECT3D
#ifdef DX8
struct IDirect3DTexture8;
typedef IDirect3DTexture8 IDirect3DTexture;
#endif
#ifdef DX9
struct IDirect3DTexture9;
typedef IDirect3DTexture9 IDirect3DTexture;
#endif
#endif

class Rotation;
class Rotation4d;

//extern bool g_autoMFLength;
extern bool g_writeGzipped;

class Color3f  {
public:
    float r;
    float g;
    float b;
    Color3f(){
        r=1;
        g=1;
        b=1;
    }
    Color3f(float nr,float ng,float nb){
        r=nr;
        g=ng;
        b=nb;
    }
    Color3f	operator /(const float v)const{
        return (Color3f(r / v, g / v, b / v));
    }
    Color3f	operator +(const Color3f v)const{
        return (Color3f(r + v.r, g + v.g, b + v.b));
    }
    Color3f	operator -(const Color3f v)const{
        return (Color3f(r - v.r, g - v.g, b - v.b));
    }
    Color3f	operator -()const{
        return (Color3f(-r, -g, -b));
    }
    Color3f	invert()const{
        return (Color3f(1 - r, 1 - g, 1 - b));
    }

    Color3f	operator *(const Color3f& par)const{
        return (Color3f(r * par.r, g * par.g, b * par.b));
    }
    Color3f	operator *(const float par)const{
        return (Color3f(r * par, g * par, b * par));
    }
    friend Color3f operator *(const float par0, const Color3f& par1){
        return (Color3f(par0 * par1.r, par0 * par1.g, par0 * par1.b ));
    }

    Color3f	operator/(const Color3f v)const{
        return (Color3f(r / v.r, g / v.g, b / v.b));
    }

    friend bool	operator !=(const Color3f &par1, const Color3f &par2){
        if(
            par1.r == par2.r &&
            par1.g == par2.g &&
            par1.b == par2.b
            )
            return false;
        else return true;
    }
    friend bool	operator ==(const Color3f &par1, const Color3f &par2){
        if(
            par1.r == par2.r &&
            par1.g == par2.g &&
            par1.b == par2.b
            )
            return true;
        else return false;
    }

    unsigned colorToHex()const;
    unsigned colorToHex(const float alpha)const;
    void toString(String& str)const;
    int fromString(const String& str, int i=0);
    void getField(const String& str, int& i);
    void toStrCpp(String& str)const;
};

template <typename T>class Vec3{
public:

    T x;
    T y;
    T z;
    Vec3(){
        x = 0;
        y = 0;
        z = 0;
    }

    Vec3(T nx, T ny, T nz){
        x = nx;
        y = ny;
        z = nz;
    }

    Vec3 operator +(const Vec3& v)const{
        return (Vec3(x+v.x, y+v.y, z+v.z));
    }
    Vec3 operator -(const Vec3& v)const{
        return (Vec3(x-v.x, y-v.y, z-v.z));
    }
    Vec3 operator -()const{
        return (Vec3(-x, -y, -z));
    }

    Vec3 operator *(const Vec3& par)const{
        return (Vec3(x * par.x, y * par.y, z * par.z));
    }
    Vec3 operator *(float par)const{
        return (Vec3(x * par, y * par, z * par));
    }
//	friend Vec3 operator *(const float& par0, const Vec3& par1){
//		return (Vec3(par0*par1.x, par0*par1.y, par0*par1.z ));
//	}

    Vec3 operator /(const Vec3& v)const{
        return (Vec3(x / v.x, y / v.y, z / v.z));
    }
    Vec3 operator /(const float par)const{
        return (Vec3(x / par, y / par, z / par));
    }

    Vec3& operator -=(const Vec3& par1){
        x -= par1.x;
        y -= par1.y;
        z -= par1.z;
        return *this;
    }
    Vec3& operator +=(const Vec3& par1){
        x += par1.x;
        y += par1.y;
        z += par1.z;
        return *this;
    }
    Vec3& operator /=(const Vec3& par1){
        x /= par1.x;
        y /= par1.y;
        z /= par1.z;
        return *this;
    }
    Vec3& operator *=(const Vec3& par1){
        x *= par1.x;
        y *= par1.y;
        z *= par1.z;
        return *this;
    }
    Vec3& operator /=(const float& par1){
        x /= par1;
        y /= par1;
        z /= par1;
        return *this;
    }
    Vec3& operator *=(const float& par1){
        x *= par1;
        y *= par1;
        z *= par1;
        return *this;
    }
    friend bool	operator !=(const Vec3& par1, const Vec3& par2){
        if(
            par1.x==par2.x&&
            par1.y==par2.y&&
            par1.z==par2.z
            )
            return false;
        else return true;
    }
    friend bool	operator ==(const Vec3& par1, const Vec3& par2){
        if(
            par1.x==par2.x&&
            par1.y==par2.y&&
            par1.z==par2.z
            )
            return true;
        else return false;
    }
    float dot(const Vec3& v) const{
        return (x * v.x +	y * v.y +  z * v.z);
    }

    Vec3 mirrorX()const{
       return Vec3(-x, y, z);
    }

    Vec3 cross ( const Vec3& v)const{
       return	Vec3(
                    y * v.z - z * v.y,
                    z * v.x - x * v.z,
                    x * v.y - y * v.x
                );
    }
    void getField(const String& str, int& i);
    void toString(String& str) const;

    Vec3 operator *=(const Matrix matrix);
    Vec3 normalize()const;
    T length()const{
        return sqrt(x * x + y * y + z * z);
    }


    void toStrCpp(String& str)const;
    void toPOVRayString(String& str)const;


};

template <typename T>class Vec2{
public:
    T x;
    T y;
    Vec2():
        x(0),
        y(0)
        {}

    Vec2(T nx,T ny){
        x=nx;
        y=ny;
    }
    friend bool	operator ==(const Vec2 &par1, const Vec2 &par2){
        if(
            par1.x==par2.x&&
            par1.y==par2.y
            )
            return true;
        else return false;
    }
    friend bool	operator !=(const Vec2 &par1, const Vec2 &par2){
        if(
            par1.x==par2.x&&
            par1.y==par2.y
            )
            return false;
        else return true;
    }
//////////
    friend bool	operator <(const Vec2 &parA, const Vec2 &parB){
        if(
            parA.x<parB.x&&
            parA.y<parB.y
            )
            return true;
        else return false;
    }
    friend bool	operator >=(const Vec2 &parA, const Vec2 &parB){
        if(
            parA.x>=parB.x&&
            parA.y>=parB.y
            )
            return true;
        else return false;
    }


    Vec2& operator+=(const Vec2& par){
        x += par.x;
        y += par.y;
        return *this;
    }
    Vec2& operator-=(const Vec2& par){
        x -= par.x;
        y -= par.y;
        return *this;
    }
    Vec2& operator*=(const Vec2& par){
        x *= par.x;
        y *= par.y;
        return *this;
    }
    Vec2& operator/=(const Vec2& par){
        x /= par.x;
        y /= par.y;
        return *this;
    }
    Vec2& operator/=(const float par){
        x /= par;
        y /= par;
        return *this;
    }
    Vec2& operator*=(const float par){
        x *= par;
        y *= par;
        return *this;
    }
    Vec2 operator -()const{
        return (Vec2(-x, -y));
    }
    Vec2 operator -(const Vec2& v)const{
        return (Vec2(x-v.x, y-v.y));
    }
    Vec2	operator +(const Vec2& v)const{
        return (Vec2(x+v.x, y+v.y));
    }
    Vec2	operator *(const Vec2& v)const{
        return Vec2(x*v.x, y*v.y);
    }
    Vec2	operator /(const Vec2& v)const{
        return Vec2(x/v.x, y/v.y);
    }
    Vec2	operator /(const float& par)const{
        return Vec2(x/par, y/par);
    }
    Vec2	operator *(const double& v)const{
        return Vec2(x*v, y*v);
    }
    Vec2	operator >>(const int& par)const{
        return Vec2(x >> par, y >> par);
    }
    Vec2 rotate(double angle)const{
        double newX;
        double newY;
        double cosAngle=(double)cos(angle);
        double sinAngle=(double)sin(angle);
        newX=x*cosAngle-y*sinAngle;
        newY=x*sinAngle+y*cosAngle;
        return Vec2(newX,newY);
    }
    float getAngle(){
        float hype=sqrt(x*x + y*y);
        float angle = acos(x/hype);
        if(y < 0){
            angle = -angle;
        }
        return angle;
    }
    float getAngle2(){
        float hype=sqrt(x*x + y*y);
        float angle = acos(x/hype);
        if(y < 0){
            angle = PI*2 - angle;
        }
        return angle;
    }
    float getAngle3(){
        float hype=sqrt(x*x + y*y);
        float angle = asin(y/hype) + PI*.5;
        if(x > 0){
            angle = PI*2 - angle;
        }
        return angle;
    }

    // return the principal angle of v (-PI < angle <= PI)
    double angle() const {
        return atan2(y, x);
    }
    Vec2 normalize()const;

    double length() const {
        return sqrt(x * x + y * y);
    }
    Vec2 mirrorX()const{
       return Vec2(-x, y);
    }

    void getField(const String& str, int& i);
    void toString(String& str) const;
    void toStrCpp(String& str)const;
    void toPOVRayString(String& str)const;
    void toPOVRayString(String& str, const Node* node)const;

};


typedef Vec2<float> Vec2f;
typedef Vec2<int> Vec2i;
typedef Vec3<float> Vec3f;
typedef Vec3<double> Vec3d;

struct NodeIconLookUpItem{
    Vec2i size;
    Vec2i srcPos;
};


class Quaternion{
public:
    union {
        struct{
            double x, y, z, w;
        };
        float qArray[4];
    };

    operator double *() { return &x; }
    Quaternion(){
    }
    Quaternion(double x, double y, double z, double w):
        x(x),
        y(y),
        z(z),
        w(w)
    {}
    Rotation rotation();
    Rotation4d rotation4d();
    Vec3d eulerXYZ();
    Vec3d eulerYZX();

};
class Rotation {
public:
    float x;
    float y;
    float z;
    float angle;
    Rotation (){
        x=0;
        y=0;
        z=1;
        angle=0;
    }
    Rotation (const float i_x, const float i_y, const float i_z, const float i_angle){
        x=i_x;
        y=i_y;
        z=i_z;
        angle=i_angle;
    }
    Rotation (const Vec3f vec, const float i_angle){
        x=vec.x;
        y=vec.y;
        z=vec.z;
        angle=i_angle;
    }
    Rotation (Vec3f fromVector, Vec3f toVector);

    friend bool	operator !=(const Rotation &par1, const Rotation &par2){
        if(
            par1.x==par2.x&&
            par1.y==par2.y&&
            par1.z==par2.z&&
            par1.angle==par2.angle
            )
            return false;
        else return true;
    }
    friend bool	operator ==(const Rotation &par1, const Rotation &par2){
        if(
            par1.x==par2.x&&
            par1.y==par2.y&&
            par1.z==par2.z&&
            par1.angle==par2.angle
            )
            return true;
        else return false;
    }
    Rotation	operator *=(const Rotation& par1);
    Rotation	operator *(const Rotation& par1)const;
    Rotation	multiply(const Rotation& par1)const;
    Vec3f operator *(const Vec3f& vec)const;

    Quaternion quaternion()const{
        return Quaternion(x*sin(angle / 2.0), y*sin(angle / 2.0), z*sin(angle / 2.0), cos(angle / 2.0));
    }

    void toStrCpp(String& str)const;
    void toString(String& str)const;
    Vec3f getAxis()const;
    Vec3f setAxis(const Vec3f& par){
        x=par.x;
        y=par.y;
        z=par.z;
        return par;
    };
    Rotation operator -()const;
    void getField(const String& str, int& i);
    Rotation slerp(const Rotation dest, const float amount)const;
};
inline Rotation Quaternion::rotation(){
    Rotation rot;
    if(w>1.0L){
        w = 1;
    }
    rot.angle = acos(w) * 2.0;
    if(rot.angle==0){
        rot=Rotation();
    }
    else{
        rot.x = x / sin(rot.angle / 2.0);
        rot.y = y / sin(rot.angle / 2.0);
        rot.z = z / sin(rot.angle / 2.0);
    }
    return rot;
}


class Rotation4d {
public:
    double x;
    double y;
    double z;
    double angle;
    Rotation4d (){
        x=0;
        y=0;
        z=1;
        angle=0;
    }
    Rotation4d (const double i_x, const double i_y, const double i_z, const double i_angle){
        x=i_x;
        y=i_y;
        z=i_z;
        angle=i_angle;
    }
    Rotation4d (const Vec3d vec, const double i_angle){
        x=vec.x;
        y=vec.y;
        z=vec.z;
        angle=i_angle;
    }
    Rotation4d (Vec3d fromVector, Vec3d toVector);

    friend bool	operator !=(const Rotation4d &par1, const Rotation4d &par2){
        if(
            par1.x==par2.x&&
            par1.y==par2.y&&
            par1.z==par2.z&&
            par1.angle==par2.angle
            )
            return false;
        else return true;
    }
    friend bool	operator ==(const Rotation4d &par1, const Rotation4d &par2){
        if(
            par1.x==par2.x&&
            par1.y==par2.y&&
            par1.z==par2.z&&
            par1.angle==par2.angle
            )
            return true;
        else return false;
    }
    Rotation4d	operator *=(const Rotation4d& par1);
    Rotation4d	operator *(const Rotation4d& par1)const;
    /*
    void* getMultiplyAddress(){
        (Rotation4d (__thiscall * )(const Rotation4d &) const)* p;
        p=&multiply;
        return p;
    }
    */
    Rotation4d	multiply(const Rotation4d& par1)const;
    Vec3d operator *(const Vec3d& vec)const;

    Quaternion quaternion()const{
        return Quaternion(x*sin(angle / 2.0), y*sin(angle / 2.0), z*sin(angle / 2.0), cos(angle / 2.0));
    }

    void toStrCpp(String& str)const;
    void toString(String& str)const;
    Vec3d getAxis()const;
    Vec3d setAxis(const Vec3d& par){
        x=par.x;
        y=par.y;
        z=par.z;
        return par;
    };
    Rotation4d operator -()const;
    void getField(const String& str, int& i);
    Rotation4d slerp(const Rotation4d dest, const double amount)const;
};


typedef M_ <Vec3f> M_Vec3f;


#define STANDARDBUFLEN 20

template <typename T>class M_ {
class Length{
	friend class M_;
protected:
	unsigned bufLen;
	unsigned rawLen;
	T* pointer;
public:
	Length():
		rawLen(0), bufLen(0), pointer(0)
	{
	}
	Length(const T& f0):
		rawLen(1),
		bufLen(STANDARDBUFLEN),
		pointer(new T[STANDARDBUFLEN])
	{
		pointer[0] = f0;
	}
	Length(const T& f0,const T& f1):
		rawLen(2),
		bufLen(STANDARDBUFLEN),
		pointer(new T[STANDARDBUFLEN])
	{
		pointer[0] = f0;
		pointer[1] = f1;
	}
	Length(const T& f0,const T& f1,const T& f2):
		rawLen(3),
		bufLen(STANDARDBUFLEN),
		pointer(new T[STANDARDBUFLEN])
	{
		pointer[0] = f0;
		pointer[1] = f1;
		pointer[2] = f2;
	}
	Length(const T& f0,const T& f1,const T& f2, const T& f3):
		rawLen(4),
		bufLen(STANDARDBUFLEN),
		pointer(new T[STANDARDBUFLEN])
	{
		pointer[0] = f0;
		pointer[1] = f1;
		pointer[2] = f2;
		pointer[3] = f3;
	}
	Length(const T& f0,const T& f1,const T& f2, const T& f3, const T& f4):
		rawLen(5),
		bufLen(STANDARDBUFLEN),
		pointer(new T[STANDARDBUFLEN])
	{
		pointer[0] = f0;
		pointer[1] = f1;
		pointer[2] = f2;
		pointer[3] = f3;
		pointer[4] = f4;
	}
	Length(const T& f0,const T& f1,const T& f2, const T& f3, const T& f4, const T& f5):
		rawLen(6),
		bufLen(STANDARDBUFLEN),
		pointer(new T[STANDARDBUFLEN])
	{
		pointer[0] = f0;
		pointer[1] = f1;
		pointer[2] = f2;
		pointer[3] = f3;
		pointer[4] = f4;
		pointer[5] = f5;
	}
	Length(const T& f0,const T& f1,const T& f2, const T& f3, const T& f4, const T& f5, const T& f6):
		rawLen(7),
		bufLen(STANDARDBUFLEN),
		pointer(new T[STANDARDBUFLEN])
	{
		pointer[0] = f0;
		pointer[1] = f1;
		pointer[2] = f2;
		pointer[3] = f3;
		pointer[4] = f4;
		pointer[5] = f5;
		pointer[6] = f6;
	}
	Length(const T& f0,const T& f1,const T& f2, const T& f3, const T& f4, const T& f5, const T& f6, const T& f7):
		rawLen(8),
		bufLen(STANDARDBUFLEN),
		pointer(new T[STANDARDBUFLEN])
	{
		pointer[0] = f0;
		pointer[1] = f1;
		pointer[2] = f2;
		pointer[3] = f3;
		pointer[4] = f4;
		pointer[5] = f5;
		pointer[6] = f6;
		pointer[7] = f7;
	}
	Length(const T& f0,const T& f1,const T& f2, const T& f3, const T& f4, const T& f5, const T& f6, const T& f7, const T& f8):
		rawLen(9),
		bufLen(STANDARDBUFLEN),
		pointer(new T[STANDARDBUFLEN])
	{
		pointer[0]=f0;
		pointer[1]=f1;
		pointer[2]=f2;
		pointer[3]=f3;
		pointer[4]=f4;
		pointer[5]=f5;
		pointer[6]=f6;
		pointer[7]=f7;
		pointer[8]=f8;
	}
	Length(const T& f0,const T& f1,const T& f2, const T& f3, const T& f4, const T& f5, const T& f6, const T& f7, const T& f8, const T& f9):
		rawLen(10),
		bufLen(STANDARDBUFLEN),
		pointer(new T[STANDARDBUFLEN])
	{
		pointer[0]=f0;
		pointer[1]=f1;
		pointer[2]=f2;
		pointer[3]=f3;
		pointer[4]=f4;
		pointer[5]=f5;
		pointer[6]=f6;
		pointer[7]=f7;
		pointer[8]=f8;
		pointer[9]=f9;
	}
	Length(const T& f0,const T& f1,const T& f2, const T& f3, const T& f4, const T& f5, const T& f6, const T& f7, const T& f8, const T& f9, const T& f10):
		rawLen(11),
		bufLen(STANDARDBUFLEN),
		pointer(new T[STANDARDBUFLEN])
	{
		pointer[0]=f0;
		pointer[1]=f1;
		pointer[2]=f2;
		pointer[3]=f3;
		pointer[4]=f4;
		pointer[5]=f5;
		pointer[6]=f6;
		pointer[7]=f7;
		pointer[8]=f8;
		pointer[9]=f9;
		pointer[10]=f10;
	}
	Length(const T& f0,const T& f1,const T& f2, const T& f3, const T& f4, const T& f5, const T& f6, const T& f7, const T& f8, const T& f9, const T& f10, const T& f11):
		rawLen(12),
		bufLen(STANDARDBUFLEN),
		pointer(new T[STANDARDBUFLEN])
	{
		pointer[0]=f0;
		pointer[1]=f1;
		pointer[2]=f2;
		pointer[3]=f3;
		pointer[4]=f4;
		pointer[5]=f5;
		pointer[6]=f6;
		pointer[7]=f7;
		pointer[8]=f8;
		pointer[9]=f9;
		pointer[10]=f10;
		pointer[11]=f11;
	}
	Length(const T& f0,const T& f1,const T& f2, const T& f3, const T& f4, const T& f5, const T& f6, const T& f7, const T& f8, const T& f9, const T& f10, const T& f11, const T& f12):
		rawLen(13),
		bufLen(STANDARDBUFLEN),
		pointer(new T[STANDARDBUFLEN])
	{
		pointer[0]=f0;
		pointer[1]=f1;
		pointer[2]=f2;
		pointer[3]=f3;
		pointer[4]=f4;
		pointer[5]=f5;
		pointer[6]=f6;
		pointer[7]=f7;
		pointer[8]=f8;
		pointer[9]=f9;
		pointer[10]=f10;
		pointer[11]=f11;
		pointer[12]=f12;
	}
	Length(const T& f0,const T& f1,const T& f2, const T& f3, const T& f4, const T& f5, const T& f6, const T& f7, const T& f8, const T& f9, const T& f10, const T& f11, const T& f12, const T& f13):
		rawLen(14),
		bufLen(STANDARDBUFLEN),
		pointer(new T[STANDARDBUFLEN])
	{
		pointer[0]=f0;
		pointer[1]=f1;
		pointer[2]=f2;
		pointer[3]=f3;
		pointer[4]=f4;
		pointer[5]=f5;
		pointer[6]=f6;
		pointer[7]=f7;
		pointer[8]=f8;
		pointer[9]=f9;
		pointer[10]=f10;
		pointer[11]=f11;
		pointer[12]=f12;
		pointer[13]=f13;
	}
	Length(const T& f0,const T& f1,const T& f2, const T& f3, const T& f4, const T& f5, const T& f6, const T& f7, const T& f8, const T& f9, const T& f10, const T& f11, const T& f12, const T& f13, const T& f14, const T& f15):
		rawLen(16),
		bufLen(STANDARDBUFLEN),
		pointer(new T[STANDARDBUFLEN])
	{
		pointer[0]=f0;
		pointer[1]=f1;
		pointer[2]=f2;
		pointer[3]=f3;
		pointer[4]=f4;
		pointer[5]=f5;
		pointer[6]=f6;
		pointer[7]=f7;
		pointer[8]=f8;
		pointer[9]=f9;
		pointer[10]=f10;
		pointer[11]=f11;
		pointer[12]=f12;
		pointer[13]=f13;
		pointer[14]=f14;
		pointer[15]=f15;
	}
	Length(const M_& param):
		rawLen(param.length.rawLen),
		bufLen(param.length.bufLen),
		pointer(new T[bufLen])
	{
		for(unsigned i=0;i<rawLen;i++)pointer[i]=param.length.pointer[i];
	}
	~Length(){
		delete[] pointer;
	}
	int operator =(const Length& par){
		unsigned newLength=par.rawLen;
		unsigned i;
		if(bufLen<newLength){
			bufLen=newLength*2;
			T* newPointer=new T[bufLen];
			for(i=0;i<rawLen;i++){
				newPointer[i]=pointer[i];
			}
			delete[] pointer;
			pointer=newPointer;
		}
		for(i=rawLen;i<newLength;i++){
			pointer[i]=T();
		}
		rawLen=newLength;
		return rawLen;
	}
	inline int operator =(const int& newLength){
		unsigned i;
		if(bufLen<(unsigned)newLength){
			bufLen=newLength*2;
			T* newPointer=new T[bufLen];
			for(i=0;i<rawLen;i++){
				newPointer[i]=pointer[i];
			}
			delete[] pointer;
			pointer=newPointer;
		}
		for(i=rawLen;i<(unsigned)newLength;i++){
			pointer[i]=T();
		}
		rawLen=newLength;
		return rawLen;
	}
	int operator ++ (int){
		unsigned newLength=rawLen+1;

		if(bufLen<newLength){
			bufLen=newLength*2;
			T* newPointer=new T[bufLen];
			for(unsigned i=0;i<rawLen;i++){
				newPointer[i]=pointer[i];
			}
			delete[] pointer;
			pointer=newPointer;
		}
		pointer[rawLen]=T();
		rawLen=newLength;
		return rawLen-1;
	}
	int operator -- (int){
		rawLen--;
		if((int)rawLen<0)throw this;
		return rawLen;
	}
	int operator -= (int par){
		rawLen-=par;
		if((int)rawLen<0)throw this;
		return rawLen;
	}
	int operator += (int par){
		unsigned newLength=par+rawLen;

		if(bufLen<newLength){
			bufLen=newLength*2;
			T* newPointer=new T[bufLen];
			for(unsigned i=0;i<rawLen;i++){
				newPointer[i]=pointer[i];
			}
			delete[] pointer;
			pointer=newPointer;
		}
		for(unsigned i=rawLen;i<newLength;i++){
			pointer[i]=T();
		}
		rawLen=newLength;
		return newLength;
	}
	operator int ()const{
		return rawLen;
	}
};

public:
	M_():
		length()
	{}
	M_(const T& f0):
		length(f0)
	{}
	M_(const T& f0,const T& f1):
		length(f0,f1)
	{}
	M_(const T& f0,const T& f1,const T& f2):
		length(f0,f1,f2)
	{}
	M_(const T& f0,const T& f1,const T& f2,const T& f3):
		length(f0,f1,f2,f3)
	{}
	M_(const T& f0,const T& f1,const T& f2, const T& f3, const T& f4):
		length(f0,f1,f2,f3,f4)
	{}
	M_(const T& f0,const T& f1,const T& f2, const T& f3, const T& f4, const T& f5):
		length(f0,f1,f2,f3,f4,f5)
	{}
	M_(const T& f0,const T& f1,const T& f2, const T& f3, const T& f4, const T& f5, const T& f6):
		length(f0,f1,f2,f3,f4,f5,f6)
	{}
	M_(const T& f0,const T& f1,const T& f2, const T& f3, const T& f4, const T& f5, const T& f6, const T& f7):
		length(f0,f1,f2,f3,f4,f5,f6,f7)
	{}
	M_(const T& f0,const T& f1,const T& f2, const T& f3, const T& f4, const T& f5, const T& f6, const T& f7, const T& f8):
		length(f0,f1,f2,f3,f4,f5,f6,f7,f8)
	{}
	M_(const T& f0,const T& f1,const T& f2, const T& f3, const T& f4, const T& f5, const T& f6, const T& f7, const T& f8, const T& f9):
		length(f0,f1,f2,f3,f4,f5,f6,f7,f8,f9)
	{}
	M_(const T& f0,const T& f1,const T& f2, const T& f3, const T& f4, const T& f5, const T& f6, const T& f7, const T& f8, const T& f9, const T& f10):
		length(f0,f1,f2,f3,f4,f5,f6,f7,f8,f9,f10)
	{}
	M_(const T& f0,const T& f1,const T& f2, const T& f3, const T& f4, const T& f5, const T& f6, const T& f7, const T& f8, const T& f9, const T& f10, const T& f11):
		length(f0,f1,f2,f3,f4,f5,f6,f7,f8,f9,f10,f11)
	{}
	M_(const T& f0,const T& f1,const T& f2, const T& f3, const T& f4, const T& f5, const T& f6, const T& f7, const T& f8, const T& f9, const T& f10, const T& f11, const T& f12):
		length(f0,f1,f2,f3,f4,f5,f6,f7,f8,f9,f10,f11,f12)
	{}
	M_(const T& f0,const T& f1,const T& f2, const T& f3, const T& f4, const T& f5, const T& f6, const T& f7, const T& f8, const T& f9, const T& f10, const T& f11, const T& f12, const T& f13):
		length(f0,f1,f2,f3,f4,f5,f6,f7,f8,f9,f10,f11,f12,f13)
	{}
	M_(const T& f0,const T& f1,const T& f2, const T& f3, const T& f4, const T& f5, const T& f6, const T& f7, const T& f8, const T& f9, const T& f10, const T& f11, const T& f12, const T& f13, const T& f14, const T& f15):
		length(f0,f1,f2,f3,f4,f5,f6,f7,f8,f9,f10,f11,f12,f13,f14,f15)
	{}
	M_(const M_& param):
		length(param)
	{}
	~M_(){
	}
	M_& operator= (const M_& par){
		if(par.length.pointer==0){
			length.rawLen=0;
		}
		else if(par.length.pointer!=length.pointer){
			if(length.bufLen<par.length.bufLen){
				delete[] length.pointer;
				length.rawLen=par.length.rawLen;
				length.bufLen=length.rawLen*2;
				length.pointer=new T[length.bufLen];
			}
			else{
				length.rawLen=par.length.rawLen;
			}
			for(unsigned i=0;i<length.rawLen;i++)length.pointer[i]=par.length.pointer[i];
		}
		return *this;
	}

	M_& operator +=(T par){
		unsigned newLength=length.rawLen+1;

		if(length.bufLen<newLength){
			length.bufLen=newLength*2;
			T* newPointer=new T[length.bufLen];
			for(unsigned i=0;i<length.rawLen;i++){
				newPointer[i]=length.pointer[i];
			}
			delete[] length.pointer;
			length.pointer=newPointer;
		}
		length.pointer[length.rawLen]=par;
		length.rawLen=newLength;
		return *this;
	}
	M_& operator +=(const M_& par1){
		unsigned i;
		unsigned newLength=par1.length.rawLen+length.rawLen;
		if(length.bufLen<newLength){
			length.bufLen=newLength*2;
			T* newPointer=new T[length.bufLen];
			for(unsigned i=0;i<length.rawLen;i++){
				newPointer[i]=length.pointer[i];
			}
			delete[] length.pointer;
			length.pointer=newPointer;
		}
		for(i=0;i<par1.length.rawLen;i++){
			length.pointer[i+length.rawLen]=par1.length.pointer[i];
		}
		length.rawLen=newLength;
		return *this;
	}
	M_ operator +(const T& par)const{
		M_ m=*this;
		m+=par;
		return m;
	}
	M_ operator +(const M_& par)const{
		M_ m=*this;
		m+=par;
		return m;
	}
    const char *showReason() const { 
		return "Exception in M_ class."; 
	}
	int size()const{
		return length.rawLen;
	}
	void setElement(const int index, const T& val){
		length.pointer[index]=val;
	}
	T& operator [](const int& index){
		if((unsigned)index>=length.rawLen){
			throw this;
		}
		return length.pointer[index];
	}
	const T& operator [](const int& index)const{
		if((unsigned)index>=length.rawLen){
			throw this;
		}
		return length.pointer[index];
	}
	const T* getArray()const{
		return length.pointer;
	}
	T* getArray(){
		return length.pointer;
	}
	bool operator ==(const M_& par)const{
        if(M_::rawLen!=par->rawLen){
			return false;
		}
		for(int i=length; --i>=0;){
			if(length.pointer[i]!=par[i]){
				return false;
			}
		}
		return true;
	}
	bool operator !=(const M_& par)const{
		if(length!=par.length){
			return true;
		}
		for(int i=length; --i>=0;){
			if(length.pointer[i]!=par[i]){
				return true;
			}
		}
		return false;
	}
	void toString(String& str)const;
	void freeMem(){
		length.rawLen=0;
		length.bufLen=0;
		delete[] length.pointer;
		length.pointer=0;
	}
	void getField(const String& str,int& i);

	Length length;
};



typedef M_ <bool> M_Bool;
typedef M_ <Color3f> M_Color3f;
typedef M_ <float> M_Float;
typedef M_ <int> M_Int;
typedef M_ <Rotation> M_Rotation;
typedef M_ <Rotation4d> M_Rotation4d;
typedef M_ <String> M_String;
typedef M_ <double> M_Double;
typedef M_ <Vec2f> M_Vec2f;
typedef M_ <Vec3f> M_Vec3f;
typedef M_ <NodePtr> M_Node;
typedef M_ <ProtoInterfacePtr> M_ProtoInterface;

extern M_ProtoInterface g_lookupProtoInterface;

typedef M_ <M_Color3f> MM_Color;
typedef M_ <M_Int> MM_Int32;

typedef M_ <Properties> M_Properties;

extern M_String nodeLookUpStr;
extern M_String containerFieldLookUpStr;

bool equals(const char* pa, const char* pb, const int lenA, const int lenB);
bool notEquals(const char* pa, const char* pb, const int lenA, const int lenB);
bool lessThan(const char* pa, const char* pb, const int lenA, const int lenB);
bool lessEqual(const char* pa, const char* pb, const int lenA, const int lenB);
bool greaterThan(const char* pa, const char* pb, const int lenA, const int lenB);
bool greaterEqual(const char* pa, const char* pb, const int lenA, const int lenB);

template<class T> inline unsigned amountToRequest();
template<> inline unsigned amountToRequest<char>()         {return 4;}
template<> inline unsigned amountToRequest<Color3f>()		{return 60;}
template<> inline unsigned amountToRequest<int>()			{return 20;}
template<> inline unsigned amountToRequest<unsigned int>()	{return 20;}
template<> inline unsigned amountToRequest<float>()		{return 20;}
template<> inline unsigned amountToRequest<double>()		{return 20;}
template<> inline unsigned amountToRequest<Vec2i>()		{return 38;}
template<> inline unsigned amountToRequest<Vec2f>()		{return 40;}
template<> inline unsigned amountToRequest<Vec3f>()		{return 60;}
template<> inline unsigned amountToRequest<Vec3d>()		{return 70;}
template<> inline unsigned amountToRequest<Rotation>()		{return 80;}

#define defaultBufLen 0x200
void removeZeros(char* str);

inline char upperCaseChar(const char& c){
	if(c >= 'a' && c <= 'z'){
		return c - 0x20;
	}
	return c;
}
inline char lowerCaseChar(const char& c){
	if(c >= 'A' && c <= 'Z'){
		return c + 0x20;
	}
	return c;
}


float getFloat(const String& str, int& i);
float getFloat2(const String& str, int& i);
double getDouble(const String& str, int& i);
int getInt(const String& str, int& i);
int getInt2(const String& str, int& i);
bool getBool(const String& str, int& i);
bool getCppBool(const String& str, int& i);


class String{
public:
    class Data {
    public:
        unsigned refCount;
        unsigned bufLen;
        unsigned rawLen;
        char* rawChar;
    };
    friend Data;
class Length{
private:
    Data*& pointer;
public:
    Length(	Data*& i_pointer):
      pointer(i_pointer)
    {
    }
    operator int ()const{
        return pointer->rawLen;
    }
};

public:
    Data* createData(int i_bufLength=defaultBufLen, int length=0)const{
        Data* data=(Data*)new char[i_bufLength+sizeof(Data)+1];
        data->rawChar=(char*)data+sizeof(Data);
        data->refCount=1;	//must always be created with a pointer to it
        data->bufLen=i_bufLength;
        data->rawLen=length;
        return data;
    }
    String( Data* data):
        length(pointer)
    {
        pointer=data;
    }

    String(int bufSize, int length):
        length(pointer)
    {
        if(bufSize < length)throw this;
        pointer = createData(bufSize, length);
    }

    String(const String& par1):
        length(pointer)
    {
        pointer=par1.pointer;
        pointer->refCount++;
    }
    String(const char* par1=""):
        length(pointer)
    {
        unsigned lEn=strlen(par1);
        pointer=createData(lEn+defaultBufLen);
        pointer->rawLen=lEn;
        for(unsigned i=0;i<pointer->rawLen+1;i++)pointer->rawChar[i]=par1[i];
    }
    String(char par1):
        length(pointer)
    {
        pointer=createData(1+defaultBufLen);
        pointer->rawLen=1;
        pointer->rawChar[0]=par1;
        pointer->rawChar[1]=0;
    }
    String(int par1):
        length(pointer)
    {
        char tmp[50];
        sprintf(tmp,"%i",par1);
        unsigned lEn=strlen(tmp);
        pointer=createData(lEn+defaultBufLen);
        pointer->rawLen=lEn;
        for(unsigned i=0;i<pointer->rawLen+1;i++)pointer->rawChar[i]=tmp[i];
    }
    String(unsigned int par1):
        length(pointer)
    {
        char tmp[50];
        sprintf(tmp,"%u",par1);
        unsigned lEn=strlen(tmp);
        pointer=createData(lEn+defaultBufLen);
        pointer->rawLen=lEn;
        for(unsigned i=0;i<pointer->rawLen+1;i++)pointer->rawChar[i]=tmp[i];
    }
    String(float par1):
        length(pointer)
    {
        char tmp[50];
        sprintf(tmp,floatFormat,par1);
        removeZeros(tmp);
        unsigned lEn=strlen(tmp);
        pointer=createData(lEn+defaultBufLen);
        pointer->rawLen=lEn;
        for(unsigned i=0;i<pointer->rawLen+1;i++)pointer->rawChar[i]=tmp[i];
    }
    String(double par1):
        length(pointer)
    {
        char tmp[50];
        sprintf(tmp,floatFormat,par1);
        removeZeros(tmp);
        unsigned lEn=strlen(tmp);
        pointer=createData(lEn+defaultBufLen);
        pointer->rawLen=lEn;
        for(unsigned i=0;i<pointer->rawLen+1;i++)pointer->rawChar[i]=tmp[i];
    }

    ~String(){
        if(--pointer->refCount==0){
            delete pointer;
        }
        pointer=NULL;
    }

    //operator const char*()const{
    //	return c_str();
    //}

    String& operator = (const String& par1){
        par1.pointer->refCount++;
        if(--pointer->refCount==0){
            delete pointer;
        }
        pointer = par1.pointer;
        return *this;
    }
    String& operator = (const char* par1){
        unsigned lEn=strlen(par1);

        if(pointer->refCount>1||lEn>pointer->bufLen){
            if(--pointer->refCount==0){
                delete pointer;
            }
            pointer=createData(lEn+defaultBufLen);
        }
        pointer->rawLen=lEn;
        for(unsigned i=0;i<pointer->rawLen+1;i++)pointer->rawChar[i]=par1[i];
        return *this;
    }
    friend String	operator + (const String& par1, const String& par2){
        return String(par1)+=par2;
    }
    String& operator = (const char& par);
    String& operator = (float par1);
    String& operator = (int par1);

    const char *showReason() const {
        return "Exception in String class.";
    }
    const char& operator ()(const int par1)const{
        if((unsigned)par1>=(unsigned)pointer->rawLen){
            throw this;
        }
        return pointer->rawChar[par1];
    }
    /*
    operator const String ()const{
        return *this;
    }
    */
    const char* c_str()const{
        return pointer->rawChar;
    }
    int getBufLength()const{
        return pointer->bufLen;
    }




////////////////////////////
friend bool	operator ==(const String &parA, const String &parB){
    return equals(parA.pointer->rawChar, parB.pointer->rawChar, parA.length, parB.length);
}
friend bool	operator ==(const char *parA, const String &parB){
    return equals(parA, parB.pointer->rawChar, strlen(parA), parB.length);
}
friend bool	operator ==(const String& parA, const char* parB){
    return equals(parA.pointer->rawChar, parB, parA.length, strlen(parB));
}

friend bool	operator !=(const String &parA, const String &parB){
    return notEquals(parA.pointer->rawChar, parB.pointer->rawChar, parA.length, parB.length);
}
friend bool	operator !=(const char *parA, const String &parB){
    return notEquals(parA, parB.pointer->rawChar, strlen(parA), parB.length);
}
friend bool	operator !=(const String& parA, const char* parB){
    return notEquals(parA.pointer->rawChar, parB, parA.length, strlen(parB));
}

friend bool	operator <(const String &parA, const String &parB){
    return lessThan(parA.pointer->rawChar, parB.pointer->rawChar, parA.length, parB.length);
}
friend bool	operator <(const char *parA, const String &parB){
    return lessThan(parA, parB.pointer->rawChar, strlen(parA), parB.length);
}
friend bool	operator <(const String& parA, const char* parB){
    return lessThan(parA.pointer->rawChar, parB, parA.length, strlen(parB));
}

friend bool	operator <=(const String &parA, const String &parB){
    return lessEqual(parA.pointer->rawChar, parB.pointer->rawChar, parA.length, parB.length);
}
friend bool	operator <=(const char *parA, const String &parB){
    return lessEqual(parA, parB.pointer->rawChar, strlen(parA), parB.length);
}
friend bool	operator <=(const String& parA, const char* parB){
    return lessEqual(parA.pointer->rawChar, parB, parA.length, strlen(parB));
}

friend bool	operator >(const String &parA, const String &parB){
    return greaterThan(parA.pointer->rawChar, parB.pointer->rawChar, parA.length, parB.length);
}
friend bool	operator >(const char *parA, const String &parB){
    return greaterThan(parA, parB.pointer->rawChar, strlen(parA), parB.length);
}
friend bool	operator >(const String& parA, const char* parB){
    return greaterThan(parA.pointer->rawChar, parB, parA.length, strlen(parB));
}

friend bool	operator >=(const String &parA, const String &parB){
    return greaterEqual(parA.pointer->rawChar, parB.pointer->rawChar, parA.length, parB.length);
}
friend bool	operator >=(const char *parA, const String &parB){
    return greaterEqual(parA, parB.pointer->rawChar, strlen(parA), parB.length);
}
friend bool	operator >=(const String& parA, const char* parB){
    return greaterEqual(parA.pointer->rawChar, parB, parA.length, strlen(parB));
}

    bool greaterThanCaseInsensitive(const String& par)const;

    void requestMoreIfNeeded(unsigned more){
        unsigned i;
        if(pointer->refCount>1||pointer->rawLen+more>pointer->bufLen){
            Data* newPointer=createData(pointer->rawLen+more+defaultBufLen, pointer->rawLen);
            for(i=0;i<pointer->rawLen+1;i++){
                newPointer->rawChar[i]=pointer->rawChar[i];
            }
            if(--pointer->refCount==0){
                delete pointer;
            }
            pointer=newPointer;
        }
    }
    void catHex(unsigned int& par){
        requestMoreIfNeeded(20);
        char* p=pointer->rawChar+pointer->rawLen;
        sprintf(p,"%X",par);
        pointer->rawLen+=strlen(p);
    }
    template<typename T>void toValue(T& par){
        int i=0;
        par.getField(*this, i);
    }
    void toValue(bool& par){
        int i=0;
        par=getBool(*this, i);
    }
    void toValue(float& par){
        int i=0;
        par=getFloat(*this, i);
    }
    void toValue(double& par){
        int i=0;
        par=getFloat(*this, i);
    }
    void toValue(int& par){
        int i=0;
        par=getFloat(*this, i);
    }

    int hourMinSecFormatToValue(); //return seconds

    void cat(const String& par1);
    void cat(const bool& par){
        if(par){
            *this+="TRUE";
        }
        else{
            *this+="FALSE";
        }
    }
    void cat(const char& par);
    void cat(const unsigned char& par);
    void cat(const int& par){
        requestMoreIfNeeded(amountToRequest<int>());
        char* p = pointer->rawChar+pointer->rawLen;
        sprintf(p,"%i",par);
        pointer->rawLen += strlen(p);
    }
    void cat(const unsigned int& par){
        requestMoreIfNeeded(amountToRequest<unsigned int>());
        char* p=pointer->rawChar+pointer->rawLen;
        sprintf(p,"%u",par);
        pointer->rawLen+=strlen(p);
    }
    void cat(const float& par){
        requestMoreIfNeeded(amountToRequest<float>());
        char* p=pointer->rawChar+pointer->rawLen;
        sprintf(p,floatFormat,par);
        removeZeros(p);
        pointer->rawLen+=strlen(p);
    }
    void cat(const double& par){
        requestMoreIfNeeded(amountToRequest<double>());
        char* p=pointer->rawChar+pointer->rawLen;
        sprintf(p,floatFormat,par);
        removeZeros(p);
        pointer->rawLen+=strlen(p);
    }
    void cat(const char* par1){
        int length = strlen(par1);
        requestMoreIfNeeded(length);
        for(int i=0;i<length+1;i++){
            pointer->rawChar[pointer->rawLen+i]=par1[i];
        }
        pointer->rawLen+=length;
    }
    void cat(const Vec2i& par){
        requestMoreIfNeeded(amountToRequest<Vec2i>());

        char* p = pointer->rawChar+pointer->rawLen;
        sprintf(p,"%i %i", par.x, par.y);
        pointer->rawLen += strlen(p);

    }

    void cat(const Vec3f& par){
        requestMoreIfNeeded(amountToRequest<Vec3f>());
        char* p=pointer->rawChar+pointer->rawLen;
        sprintf(p,floatFormat,par.x);
        removeZeros(p);
        int length=strlen(p);
        p+=length;
        pointer->rawLen+=length;
        sprintf(p,floatFormat2,par.y);
        removeZeros(p+1);
        length=strlen(p);
        p+=length;
        pointer->rawLen+=length;
        sprintf(p,floatFormat2,par.z);
        removeZeros(p+1);
        pointer->rawLen+=strlen(p);
    }
    void cat(const Vec3d& par){
        requestMoreIfNeeded(amountToRequest<Vec3d>());
        char* p=pointer->rawChar+pointer->rawLen;
        sprintf(p,floatFormat,par.x);
        removeZeros(p);
        int length=strlen(p);
        p+=length;
        pointer->rawLen+=length;
        sprintf(p,floatFormat2,par.y);
        removeZeros(p+1);
        length=strlen(p);
        p+=length;
        pointer->rawLen+=length;
        sprintf(p,floatFormat2,par.z);
        removeZeros(p+1);
        pointer->rawLen+=strlen(p);
    }
    void cat(const Vec2f& par){
        requestMoreIfNeeded(amountToRequest<Vec2f>());
        char* p=pointer->rawChar+pointer->rawLen;
        sprintf(p,floatFormat,par.x);
        removeZeros(p);
        int length=strlen(p);
        p+=length;
        pointer->rawLen+=length;
        sprintf(p,floatFormat2,par.y);
        removeZeros(p+1);
        pointer->rawLen+=strlen(p);
    }
    void cat(const Rotation& par){
        requestMoreIfNeeded(amountToRequest<Rotation>());
        char* p=pointer->rawChar+pointer->rawLen;
        sprintf(p,floatFormat,par.x);
        removeZeros(p);
        int length=strlen(p);
        p+=length;
        pointer->rawLen+=length;
        sprintf(p,floatFormat2,par.y);
        removeZeros(p+1);
        length=strlen(p);
        p+=length;
        pointer->rawLen+=length;
        sprintf(p,floatFormat2,par.z);
        removeZeros(p+1);
        length=strlen(p);
        p+=length;
        pointer->rawLen+=length;
        sprintf(p,floatFormat2,par.angle);
        removeZeros(p+1);
        pointer->rawLen+=strlen(p);
    }
    void cat(const Color3f& par){
        requestMoreIfNeeded(amountToRequest<Color3f>());
        char* p=pointer->rawChar+pointer->rawLen;
        sprintf(p,floatFormat,par.r);
        removeZeros(p);
        int length=strlen(p);
        p+=length;
        pointer->rawLen+=length;
        sprintf(p,floatFormat2,par.g);
        removeZeros(p+1);
        length=strlen(p);
        p+=length;
        pointer->rawLen+=length;
        sprintf(p,floatFormat2,par.b);
        removeZeros(p+1);
        pointer->rawLen+=strlen(p);
    }

    void catColor4f(const Color3f& par){
        requestMoreIfNeeded(amountToRequest<Rotation>());
        char* p=pointer->rawChar+pointer->rawLen;
        sprintf(p,floatFormat,par.r);
        removeZeros(p);
        int length=strlen(p);
        p+=length;
        pointer->rawLen+=length;
        sprintf(p,floatFormat2,par.g);
        removeZeros(p+1);
        length=strlen(p);
        p+=length;
        pointer->rawLen+=length;
        sprintf(p,floatFormat2,par.b);
        removeZeros(p+1);
        length=strlen(p);
        p+=length;
        pointer->rawLen+=length;
        sprintf(p,floatFormat2,1.0);
        removeZeros(p+1);
        pointer->rawLen+=strlen(p);
    }

    void cat(const S_Triangle& par);

    String& operator+=(const char* par){
        cat(par);
        return *this;
    }
    template<class T> String& operator+=(const T& par){
        cat(par);
        return *this;
    }
    template<class T> void cat(const M_<T>& par);
    template<class T> String& operator+=(const M_<T>& par){
        cat(par);
        return *this;
    }
    String& addEqTriCoordIndex(const M_Int& par){
        requestMoreIfNeeded(par.length/3*(8*3+5)+5);
        char* p=pointer->rawChar+pointer->rawLen;
        *p='[';
        p++;
        for(int i=0;i<par.length;i+=3){
            sprintf(p,"%i %i %i -1\n",par[i],par[i+1],par[i+2]);
            p+=strlen(p);
        }
        *p=']';
        p++;
        *p=0;
        pointer->rawLen=p-pointer->rawChar;
        return *this;
    }
    
    void insert(const char& ch, const int& index){
        (*this)+=' ';
        unsigned int i=pointer->rawLen;
        for(; --i>(unsigned)index;){
            (*this)[i]=(*this)[i-1];
        }
        (*this)[i]=ch;
    }
    void deleteChar(const int& index){
        *this=replace(index, 1, "");
    }
    const String substr(const int& index, const int desiredLen=0x7fffffff) const {
        unsigned i;
        unsigned realLen=length-(const unsigned)index; //potential length stage
        if((int)realLen<0)throw this;
        if(realLen>(const unsigned)desiredLen){
            realLen=desiredLen;
        }
        Data* newPointer=createData(realLen+defaultBufLen);
        newPointer->rawLen=realLen;
        for(i=0;i<realLen;i++){
            newPointer->rawChar[i]=pointer->rawChar[index+i];
        }
        newPointer->rawChar[i]=0;
        return newPointer;
    }
    const String substring(const int& startChar, const int endChar=-1) const {
        unsigned strLength,i;
        if(endChar==-1){
            strLength=pointer->rawLen-startChar;
        }
        else{
            strLength=endChar-startChar+1;
        }
        if((unsigned)startChar+strLength>pointer->rawLen)throw this;

        Data* newPointer=createData(strLength+defaultBufLen);
        newPointer->rawLen=strLength;
        for(i=0;i<strLength;i++){
            newPointer->rawChar[i]=pointer->rawChar[startChar+i];
        }
        newPointer->rawChar[i]=0;
        return newPointer;
    }
    String replace(const int& index, const int& desiredLen, const String strPar) const{
        String str=substr(0,index);
        str+=strPar;
        unsigned newIndex=index+desiredLen;
        if(newIndex<(unsigned)length){
            str+=substr(newIndex);
        }
        return str;
    }
    char& operator [](const int& par){
        if((unsigned)par>=(unsigned)pointer->rawLen){
            throw this;
        }
        if(pointer->refCount > 1){
            Data* newPointer = createData(pointer->bufLen, pointer->rawLen);
            unsigned i = 0;
            do{
                newPointer->rawChar[i] = pointer->rawChar[i];
            }while(i++ < pointer->rawLen);

            pointer = newPointer;
        }
        return pointer->rawChar[par];
    }
    const char& operator [](const int& par)const{
        if((unsigned)par>=(unsigned)pointer->rawLen){
            throw this;
        }
        return pointer->rawChar[par];
    }
/*
    int fromText(char*& p,int i=0);
    int readWord(char* p){
        for(length=0;p[length]>' ';++length){
        }
        pointer=new SFChar[length+1];
        for(int i=0;i<length;i++,p++){
            pointer[i]=*p;
        }
        pointer[i]=0;
        return length;
    }
    */
    int indexOf (String searchString, int position=0)const;

    void getField(const String& str, int& i);

    String toUpperCase()const{
        String str;
        str.pointer = createData(pointer->bufLen, pointer->rawLen);
        const char* s = pointer->rawChar;
        char* d = str.pointer->rawChar;
        for(int i = pointer->rawLen + 1; --i >= 0;){
            d[i] = upperCaseChar(s[i]);
        }
        return str;
    }

    String toLowerCase()const{
        String str;
        str.pointer = createData(pointer->bufLen, pointer->rawLen);
        const char* s = pointer->rawChar;
        char* d = str.pointer->rawChar;
        for(int i = pointer->rawLen + 1; --i >= 0;){
            d[i] = lowerCaseChar(s[i]);
        }
        return str;
    }
void toStrCpp(String& str)const;
void toString(String& str);
void endOfName(int& index)const;
void endOfCppLabel(int& index)const;
String extractName(int& i)const;
String extractCppLabel(int& i)const;
//String extractCppOperator(int& i)const;

void stb(String& dest, int& position)const;
void stw(String& dest, int& position)const;

void stb(int& position)const; //searchTillBlack treats coma as white
void stbCpp(int& position)const; //searchTillBlack includes coma as black
void stbnlCpp(int& position)const;//searchTillBlack or new line includes coma as black
void stbc(int& position)const;//searchTillBlackComment
void stw(int& position)const; //searchTillWhite
void stwo(int& position)const; //searchTillWhiteOrOperator //try not to use this
void stwo2(int& position)const; //searchTillWhiteOrOperator //use this
void stw_stb(int& position)const; //searchTillWhite then searchTillBlack
void stwo_stb(int& position)const; //searchTillWhite or Operator then search till black
void stnl(int& position)const;//searchTill new line
//void stNonCppOperator(int& position)const; //searchTillNonOperator (C++)
int getLengthOfCppOperator(int position)const;
void endOfNum(int& position)const;


bool cmpKWord(const char* str,int& index)const;
bool cmpKWord(const String& str,int& index)const;
bool findWord( const String str, int& i)const;
String fromQuotes( int& position)const;
String getWord(int& position)const;
String getWord2(int& position)const;


    void write(FILE* file)const{
        fwrite(pointer->rawChar,1,pointer->rawLen,file);
    }
    void write(FILE* stream, char* name)const;
    bool write(String fileName)const;
    //bool read( String fileName);
//private:

    Length length;
private:
    Data* pointer;
};


class Matrix3{
    float m[3][3];
public:
    Matrix3(){
        m[0][0]=m[1][1]=m[2][2]= 1.0f;
        m[0][1]=m[0][2]= 0.0f;
        m[1][0]=m[1][2]= 0.0f;
        m[2][0]=m[2][1]= 0.0f;
    }
    Matrix3(float m00, float m01, float m02, float m10, float m11, float m12, float m20, float m21, float m22){
        m[0][0]=m00;
        m[0][1]=m01;
        m[0][2]=m02;

        m[1][0]=m10;
        m[1][1]=m11;
        m[1][2]=m12;

        m[2][0]=m20;
        m[2][1]=m21;
        m[2][2]=m22;
    }
    Vec2f operator * (Vec2f vec)const{
        Vec2f tmp;
        tmp.x = vec.x*m[0][0]+vec.y*m[1][0]+m[2][0];
        tmp.y = vec.x*m[0][1]+vec.y*m[1][1]+m[2][1];
        return tmp;
    }

    Matrix3 operator *(Matrix3 mat)const{
        Matrix3 tmp;
        for (int i=0; i<3; i++){
            for (int j=0; j<3; j++){
                tmp.m[i][j]=0;
                for (int k=0; k<3; k++){
                    tmp.m[i][j]+=this->m[i][k] * mat.m[k][j];
                }
            }
        }
        return tmp;
    }
    Matrix3 operator *=(Matrix3 mat){
        *this=*this*mat;
        return *this;
    }
};
inline Matrix3 scaleMatrix(Vec2f par){
    return Matrix3(
        par.x, 0, 0,
        0, par.y, 0,
        0, 0, 1
        );
}
inline Matrix3 translationMatrix(Vec2f par){
    return Matrix3(
        1, 0, 0,
        0, 1, 0,
        par.x, par.y, 1
        );
}
inline Matrix3 rotationMatrix(float angle){

    float cosine = cos(angle);
    float sine = sin(angle);

    return Matrix3(
        cosine, sine, 0,
        -sine, cosine, 0,
        0, 0, 1
        );
}

class Matrix{
public:
    union {
        struct {
            float        f11, f12, f13, f14;
            float        f21, f22, f23, f24;
            float        f31, f32, f33, f34;
            float        f41, f42, f43, f44;
        };
        float m[4][4];
    };
    Matrix(){
        f11 = f22 = f33 = f44 = 1.0f;
        f12 = f13 = f14 = f41 = 0.0f;
        f21 = f23 = f24 = f42 = 0.0f;
        f31 = f32 = f34 = f43 = 0.0f;
    }
    Matrix(float _f11, float _f12, float _f13, float _f14, float _f21, float _f22, float _f23, float _f24, float _f31, float _f32, float _f33, float _f34, float _f41, float _f42, float _f43, float _f44){
        f11=_f11;	f12=_f12;	f13=_f13;	f14=_f14;
        f21=_f21;	f22=_f22;	f23=_f23;	f24=_f24;
        f31=_f31;	f32=_f32;	f33=_f33;	f34=_f34;
        f41=_f41;	f42=_f42;	f43=_f43;	f44=_f44;
    }

    Vec3f multVecMatrix(Vec3f vec)const;
    Vec3f operator * (Vec3f vec)const;

    void scale(Vec3f& sc);
    Matrix operator *(Matrix mat)const;
    Matrix operator *=(Matrix mat);
    //Matrix multLeft(Matrix mat);
    Matrix multRight(Matrix mat);
    void setTransform(const Vec3f translation, const Rotation rotation=Rotation(1,0,0,0), const Vec3f scaleFactor=Vec3f(1, 1, 1));
    //inline void setTransform(const SF<Vec3f,	crap>& translation, const SF<Rotation,	crap>& rotation, const SF<Vec3f,	crap>& scaleFactor);
    Rotation getRotation();
    void toString(String& str);
};

Matrix rotationMatrix(Rotation rot);
Matrix translationMatrix(Vec3f tran);
Matrix scaleMatrix(Vec3f sc);

class Node {
	public:

    friend NodePtr;

    String name;
    union{
        struct {
            bool collapsed;
            bool renderProcessed;
        };
        unsigned short nodeFlags;
    };

    Node():
        nodeFlags(0),
        refCount(0)
    {}

    unsigned refCount;

    virtual ~Node(){};

    virtual Field* getField(const int i)const=0;
    const Properties& getProp(const int index)const;

    //virtual Field* getField2(const int i){
//		return getField(i);
//	}
    virtual int getFLen()const=0; //get field rawLen


    void incRefCount(){
        if(this!=NULL){ //is this line really needed?
           ++refCount;
        }
    }
    unsigned decRefCount(){
        --refCount;
        return refCount;
    }
	
	virtual int getType()const=0;
	void getField(const int& fieldIdx,const String &str,int &i);

	void getNode(const String& str,int& i);
    void getFields(const String &str,int &i);
};

template <class T>class Ptr {
public:
    Ptr(){
        p=nullPtr;
    }
Ptr( T* par){
    p=par;
    if(par){
        par->incRefCount();
    }
}
Ptr(const Ptr& par){
    p=par.p;
    if(p!=NULL){
        p->incRefCount();
    }
}
Ptr& operator = (const Ptr& par){
    par.p->incRefCount();
    if(p!=NULL){
        if(p->decRefCount()==0){
            delete p;
        }
    }
    p=par.p;
    return *this;
}
Ptr& operator = (int par){
    if(par!=0){
        throw this;
    }
    if(p!=NULL){
        if(p->decRefCount()==0){
            delete p;
        }
    }
    p=0;
    return *this;
}
Ptr& operator = ( T* par){
    par->incRefCount();
    if(p!=NULL){
        if(p->decRefCount()==0){
            delete p;
        }
    }
    p=par;
    return *this;
}
~Ptr(){
    if(p!=NULL&&p->decRefCount()==0){
        delete p;
        p=NULL;
    }
}
    bool	operator ==(const Node*& par)const{
        return p==par;
    }
    bool	operator !=(const Node*& par)const{
        return p!=par;
    }

    operator T*()const{
        return p;
    }

    T& operator *(){
        return *p;
    }
    T* operator ->()const{
        return p;
    }
    T* getRawPtr()const{
        return p;
    }
//protected:
    T* p;
};


class Field {
public:
    virtual FieldType getType()const=0;
	virtual void read(const String& str, int& i)=0;
	virtual int countItems(const String& str, int i)=0;
};

typedef Ptr<Node> NodePtr;

class Render;
typedef FD<bool,Single>SFBool;
typedef FD<Color3f,Single>SFColor;
typedef FD<float,Single>SFFloat;
typedef FD<int,Single>SFInt32;
typedef FD<NodePtr,Single>SFNode;
typedef FD<Rotation,Single>SFRotation;
typedef FD<String,Single>SFString;
typedef FD<double,Single>SFTime;
typedef FD<Vec2f,Single>SFVec2f;
typedef FD<Vec3f,Single>SFVec3f;

typedef SFTime SFDouble;


template<class T> class FD<T,Single> : public Field {
public:
    T raw;
    bool is;
    void addValFromStr(const String& str);
    void addValSameAsLast();

    int countItems(const String& str, int i);

    FD<T,Single>(const T& par):
        raw(par),
        is(false)
    {};
    FieldType getType()const;
	void read(const String& str, int& i);
	void getField(const String& str, int& i);

    T& refValue(){
        return raw;
    }
    const T& refValue()const{
        return raw;
    }
    const T& getValue()const{
        return raw;
    }
    virtual void set(const Field* par){
        *this=dynamic_cast<const FD<T,Single>*>(par)->refValue();
    }
    virtual void operator=(const FD<T,Single>& par){
        raw=par.raw;
    }
    virtual void operator=(const T& par){
        raw=par;
    }
    operator const T& () const{
        return raw;
    }

    void connect(Node* node, Node* protoInterface, Field *protoField);

    void toStrCpp(String& str)const;
    void toString(String& str);
    int size();
    virtual void resize(int newLen);
    void toString(String& str, Field *protoField, bool newList);
    void toCPPString(String& str, const String& nodeName, Field *protoField, bool newList);
    void*& refRawPtr(){
        return (void*&)raw;
    }

};

template<> class FD<String,Single> : public Field {
public:
    String raw;
    bool is;
    void addValFromStr(const String& str);
    void addValSameAsLast();
    int countItems(const String& str, int i);

    FD(const String& par):
        raw(par),
        is(false)
    {};
    FieldType getType()const;
	void read(const String& str, int& i);
	void getField(const String& str, int& i);

    String& refValue(){
        return raw;
    }
    const String& refValue()const{
        return raw;
    }
    const String& getValue()const{
        return raw;
    }
    virtual void set(const Field* par){
        *this=dynamic_cast<const SFString*>(par)->refValue();
    }
    virtual void operator=(const SFString& par){
        raw=par.raw;
    }
    virtual void operator=(const String& par){
        raw=par;
    }
    operator const String& () const{
        return raw;
    }

    void connect(Node* node, Node* protoInterface, Field *protoField);

    void toString(String& str);
    int size();
    virtual void resize(int newLen);
    void toString(String& str, Field *protoField, bool newList);
    void*& refRawPtr(){
        return (void*&)raw;
    }

};

template<> class FD<Vec2f,Single> : public Field {
public:
    const float x;
    const float y;

    Vec2f	operator /(const float& v)const{
        return (Vec2f(x/v, y/v));
    }
    Vec2f	operator +(const Vec2f& v)const{
        return (Vec2f(x+v.x, y+v.y));
    }
    Vec2f	operator -(const Vec2f& v)const{
        return (Vec2f(x-v.x, y-v.y));
    }
    Vec2f	operator -()const{
        return (Vec2f(-x, -y));
    }

    Vec2f	operator *(const Vec2f& par)const{
        return (Vec2f(x*par.x, y*par.y));
    }
    Vec2f	operator *(float par)const{
        return (Vec2f(x*par, y*par));
    }
    friend Vec2f operator *(const float& par0, const Vec2f& par1){
        return (Vec2f(par0*par1.x, par0*par1.y));
    }

    Vec2f	operator /(const Vec2f& v)const{
        return (Vec2f(x/v.x, y/v.y));
    }

    friend bool	operator !=(const Vec2f& par1, const SFVec2f& par2){
        if(
            par1.x==par2.x&&
            par1.y==par2.y
            )
            return false;
        else return true;
    }
    friend bool	operator ==(const Vec2f& par1, const SFVec2f& par2){
        if(
            par1.x==par2.x&&
            par1.y==par2.y
            )
            return true;
        else return false;
    }
    Vec2f normalize();
    float length()const;
    void getField(const String& str, int& i);
//////////////////////////////////////////////////
    void addValFromStr(const String& str);
    void addValSameAsLast();
    int countItems(const String& str, int i);

    FD(const Vec2f& par):
        x(par.x),
        y(par.y)
    {};
    FieldType getType()const;
	void read(const String& str, int& i);

    Vec2f& refValue(){
        return *(Vec2f*)&x;
    }
    const Vec2f& refValue()const{
        return *(Vec2f*)&x;
    }
    const Vec2f& getValue()const{
        return refValue();
    }
    virtual void set(const Field* par){
        *this=dynamic_cast<const FD*>(par)->refValue();
    }
    virtual void operator=(const FD& par){
        refValue()=par;
    }
    virtual void operator=(const Vec2f& par){
        refValue()=par;
    }
    operator const Vec2f& () const{
        return refValue();
    }

    void toString(String& str);
    int size();
    virtual void resize(int newLen);
    void toString(String& str, Field* protoField, bool newList);
    void toCPPString(String& str, const String& nodeName, Field *protoField, bool newList);

    void*& refRawPtr(){
        return (void*&)refValue();
    }

};

template<> class FD<Color3f,Single> : public Field {

public:
    float r;
    float g;
    float b;

    Color3f	operator /(const float& v)const{
        return (Color3f(r/v, g/v, b/v));
    }
    Color3f	operator +(const Color3f& v)const{
        return (Color3f(r+v.r, g+v.g, b+v.b));
    }
    Color3f	operator -(const Color3f& v)const{
        return (Color3f(r-v.r, g-v.g, b-v.b));
    }
    Color3f	operator -()const{
        return (Color3f(-r, -g, -b));
    }

    Color3f	operator *(const Color3f& par)const{
        return (Color3f(r*par.r, g*par.g, b*par.b));
    }
    Color3f	operator *(float par)const{
        return (Color3f(r*par, g*par, b*par));
    }
    friend Color3f operator *(const float& par0, const Color3f& par1){
        return (Color3f(par0*par1.r, par0*par1.g, par0*par1.b ));
    }

    Color3f	operator /(const Color3f& v)const{
        return (Color3f(r/v.r, g/v.g, b/v.b));
    }

    friend bool	operator !=(const Color3f& par1, const SFColor& par2){
        if(
            par1.r==par2.r&&
            par1.g==par2.g&&
            par1.b==par2.b
            )
            return false;
        else return true;
    }
    friend bool	operator ==(const Color3f& par1, const SFColor& par2){
        if(
            par1.r==par2.r&&
            par1.g==par2.g&&
            par1.b==par2.b
            )
            return true;
        else return false;
    }
    float dot(const Color3f& v) const{
        return (r * v.r +	g * v.g +  b * v.b);
    }
    Color3f cross ( const Color3f& v)const{
       return Color3f(g * v.b - b * v.g,
                    b * v.r - r * v.b,
                    r * v.g - g * v.r);
    }

    Color3f operator *=(const Matrix matrix);
    Color3f normalize();
    float length()const;
    void getField(const String& str, int& i);
//////////////////////////////////////////////////
    void addValFromStr(const String& str);
    void addValSameAsLast();
    int countItems(const String& str, int i);

    FD(const Color3f& par):
        r(par.r),
        g(par.g),
        b(par.b)
    {};
    FieldType getType()const;
	void read(const String& str, int& i);

    Color3f& refValue(){
        return *(Color3f*)&r;
    }
    const Color3f& refValue()const{
        return *(Color3f*)&r;
    }
    const Color3f& getValue()const{
        return refValue();
    }
    virtual void set(const Field* par){
        *this=dynamic_cast<const SFColor*>(par)->refValue();
    }
    virtual void operator=(const SFColor& par){
        refValue()=par;
    }
    virtual void operator=(const Color3f& par){
        refValue()=par;
    }
    operator Color3f& (){
        return refValue();
    }
    operator const Color3f& () const{
        return refValue();
    }

    void toString(String& str);
    int size();
    virtual void resize(int newLen);
    void toString(String& str, Field *protoField, bool newList);
    void*& refRawPtr(){
        return (void*&)refValue();
    }
};

template<> class FD<Vec3f,Single> : public Field {

public:
    float x;
    float y;
    float z;

    Vec3f	operator /(const float& v)const{
        return (Vec3f(x/v, y/v, z/v));
    }
    Vec3f	operator +(const Vec3f& v)const{
        return (Vec3f(x+v.x, y+v.y, z+v.z));
    }
    Vec3f	operator -(const Vec3f& v)const{
        return (Vec3f(x-v.x, y-v.y, z-v.z));
    }
    Vec3f	operator -()const{
        return (Vec3f(-x, -y, -z));
    }

    Vec3f	operator *(const Vec3f& par)const{
        return (Vec3f(x*par.x, y*par.y, z*par.z));
    }
    Vec3f	operator *(float par)const{
        return (Vec3f(x*par, y*par, z*par));
    }
    friend Vec3f operator *(const float& par0, const Vec3f& par1){
        return (Vec3f(par0*par1.x, par0*par1.y, par0*par1.z ));
    }

    Vec3f	operator /(const Vec3f& v)const{
        return (Vec3f(x/v.x, y/v.y, z/v.z));
    }

    friend bool	operator !=(const Vec3f& par1, const SFVec3f& par2){
        if(
            par1.x==par2.x&&
            par1.y==par2.y&&
            par1.z==par2.z
            )
            return false;
        else return true;
    }
    friend bool	operator ==(const Vec3f& par1, const SFVec3f& par2){
        if(
            par1.x==par2.x&&
            par1.y==par2.y&&
            par1.z==par2.z
            )
            return true;
        else return false;
    }
    float dot(const Vec3f& v) const{
        return (x * v.x +	y * v.y +  z * v.z);
    }
    Vec3f cross ( const Vec3f& v)const{
       return Vec3f(y * v.z - z * v.y,
                    z * v.x - x * v.z,
                    x * v.y - y * v.x);
    }

    Vec3f operator *=(const Matrix matrix);
    Vec3f normalize();
    float length()const;
    void getField(const String& str, int& i);
//////////////////////////////////////////////////
    void addValFromStr(const String& str);
    void addValSameAsLast();
    int countItems(const String& str, int i);

    FD(const Vec3f& par):
        x(par.x),
        y(par.y),
        z(par.z)
    {};
    FieldType getType()const;
	void read(const String& str, int& i);

    Vec3f& refValue(){
        return *(Vec3f*)&x;
    }
    const Vec3f& refValue()const{
        return *(Vec3f*)&x;
    }
    const Vec3f& getValue()const{
        return refValue();
    }
    virtual void set(const Field* par){
        *this=dynamic_cast<const SFVec3f*>(par)->refValue();
    }
    virtual void operator=(const SFVec3f& par){
        refValue()=par;
    }
    virtual void operator=(const Vec3f& par){
        refValue()=par;
    }
    operator Vec3f& (){
        return refValue();
    }
    operator const Vec3f& () const{
        return refValue();
    }

    int size();
    virtual void resize(int newLen);
    void toString(String& str, Field *protoField, bool newList);
    void*& refRawPtr(){
        return (void*&)refValue();
    }
};

template<> class FD<Rotation,Single> : public Field {

public:
    float x;
    float y;
    float z;
    float angle;
Rotation operator -()const{
    return -refValue();
}
//////////////////////////////////////////////////
    void addValFromStr(const String& str);
    void addValSameAsLast();
    int countItems(const String& str, int i);

    FD(const Rotation& par):
        x(par.x),
        y(par.y),
        z(par.z),
        angle(par.angle)
    {};
    FieldType getType()const;
	void read(const String& str, int& i);
	void getField(const String& str, int& i);

    Rotation& refValue(){
        return *(Rotation*)&x;
    }
    const Rotation& refValue()const{
        return *(Rotation*)&x;
    }
    const Rotation& getValue()const{
        return refValue();
    }
    virtual void set(const Field* par){
        *this=dynamic_cast<const SFRotation*>(par)->refValue();
    }
    virtual void operator=(const SFRotation& par){
        refValue()=par;
    }
    virtual void operator=(const Rotation& par){
        refValue()=par;
    }
    Rotation	operator *(const Rotation& par1)const{
        return refValue() * par1;
    }
    operator Rotation& () {
        return refValue();
    }
    operator const Rotation& () const{
        return refValue();
    }
    void toString(String& str);
    int size();
    virtual void resize(int newLen);
    void*& refRawPtr(){
        return (void*&)refValue();
    }
};


template<class T> class FD<T,Multiple> : public Field {
public:
class Length{
	friend FD;
protected:
	unsigned bufLen;
	unsigned rawLen;
	T* pointer;
public:
	Length():
		rawLen(0), bufLen(0), pointer(0)
	{
	}
	Length(const M_<T>& par):
		rawLen(par.length),
		bufLen(par.length*2),
		pointer(new T[bufLen])
	{
		unsigned i;
		for(i=0;i<rawLen;i++)pointer[i]=par[i];
	}
	~Length(){
		delete[] pointer;
	}
	operator int ()const{
		return rawLen;
	}
};


public:
	FD operator +(const T& par)const{
		FD m=*this;
		m+=par;
		return m;
	}
	FD operator +(const M_<T>& par)const{
		FD m=*this;
		m+=par;
		return m;
	}
    const char *showReason() const { 
		return "Exception in FD class."; 
	}
	const T& operator [](const int& index)const{
		if((unsigned)index>=length.rawLen){
			throw this;
		}
		return length.pointer[index];
	}
	const T* getArray()const{
		return length.pointer;
	}
	T* getArray(){
		return length.pointer;
	}
	bool operator ==(const M_<T>& par)const{
        if(length != par.length){
			return false;
		}
		for(int i=length; --i>=0;){
			if(length.pointer[i]!=par[i]){
				return false;
			}
		}
		return true;
	}
	bool operator !=(const M_<T>& par)const{
        if(length != par.length){
			return true;
		}
		for(int i=length; --i>=0;){
			if(length.pointer[i]!=par[i]){
				return true;
			}
		}
		return false;
	}
	void toString(String& str)const;
	void getField(const String& str,int& i);

	void toMNodeString(String& str, bool newList=true);
	void removeNode(T par1);
	void removeNode(FD& par1);
	Art*& refPart(int i)const;
	int& refIndex(int i)const;
	int& refTexIndex(int i)const;

	void rotate(Rotation rot,int offset=0);
	void rotate( float x, float y, float z, float angle, int offset=0);
	void translate(T tran,int offset=0);
	void translate( float x, float y, float z, int offset=0);
	void scale(float xsc,float ysc,float zsc);
	void translateToZero(int offset,int numOf, Rotation& center);

	void multMatrixLeft(Matrix& matrix);
	//void paint( int& yPos);
	int getIndex(String str);
	Length length;
//////////////////

	
	
    void addValFromStr(const String& str);
    void addValSameAsLast();
    int countItems(const String& str, int i);

	FD(const M_<T>& par):
		length(par)
	{};
	FieldType getType()const;
	void read(const String& str, int& i);

	M_<T>& refValue(){
		return *(M_<T>*)&length.bufLen;
	}
	const M_<T>& refValue()const{
		return *(M_<T>*)&length.bufLen;
	}
	const M_<T>& getValue()const{
		return *(M_<T>*)&length.bufLen;
	}
	virtual void set(const Field* par){
		refValue()=dynamic_cast<const FD*>(par)->refValue();
	}
	virtual void operator=(const FD& par){
		refValue()=par.refValue();
	}
	virtual void operator=(const M_<T>& par){
		refValue()=par;
	}
	virtual void operator+=(const T& par){
		refValue()+=par;
	}
	operator const M_<T>& () const{
		return refValue();
	}
	operator M_<T>& () {
		return refValue();
	}

    int size();
    virtual void resize(int newLen);
    void toString(String& str);
	void toString(String& str, Field *protoField, bool newList);
	void*& refRawPtr(){
		return (void*&)refValue();
	}
};


typedef FD<bool, Multiple>		MFBool;
typedef FD<Color3f, Multiple>	MFColor;
typedef FD<float, Multiple>		MFFloat;
typedef FD<int, Multiple>		MFInt32;
typedef FD<NodePtr, Multiple>		MFNode;
typedef FD<Rotation, Multiple>	MFRotation;
typedef FD<String, Multiple>	MFString;
typedef FD<double, Multiple>		MFTime;
typedef FD<Vec2f, Multiple>	MFVec2f;
typedef FD<Vec3f, Multiple>	MFVec3f;

template<> class FD<NodePtr,Single> : public Field {
public:
    Node* p;
    void addValFromStr(const String& str);
    void addValSameAsLast();
    int countItems(const String& str, int i);

inline ~FD();
FD(const NodePtr& par);
inline virtual Node* operator = ( const int par1);
inline virtual Node* operator = ( Node* par1);
inline Node* operator = ( FD& par);

    bool	operator ==(const NodePtr& par)const{
        return p==par.p;
    }
    bool	operator !=(const NodePtr& par)const{
        return p!=par.p;
    }
    bool	operator ==(const Node* par)const{
        return p==par;
    }
    bool	operator !=(const Node* par)const{
        return p!=par;
    }


    operator Node*()const{
        return p;
    }

    Node& operator *(){
        return *p;
    }
    Node* operator ->()const{
        return p;
    }
    Node* getRawPtr()const{
        return p;
    }

    FieldType getType()const;
	void read(const String& str, int& i);
	void getField(const String& str, int& i);

    NodePtr& refValue(){
        return *(NodePtr*)&p;
    }
    const NodePtr& refValue()const{
        return *(NodePtr*)&p;
    }
    const NodePtr& getValue()const{
        return refValue();
    }
    operator const NodePtr& () const{
        return refValue();
    }
    operator void* ();

    void toStrCpp(String& str)const;
    void toString(String& str);
    int size();
    virtual void resize(int newLen);
    void*& refRawPtr(){
        return (void*&)refValue();
    }
};


template<>
inline FieldType SFBool::getType()const{
    return SFBOOL;
}
inline FieldType SFColor::getType()const{
    return SFCOLOR;
}
template<>
inline FieldType SFFloat::getType()const{
    return SFFLOAT;
}
template<>
inline FieldType SFInt32::getType()const{
    return SFINT32;
}
inline FieldType SFNode::getType()const{
    return SFNODE;
}
inline FieldType SFRotation::getType()const{
    return SFROTATION;
}
inline FieldType SFString::getType()const{
    return SFSTRING;
}
template<>
inline FieldType SFTime::getType()const{
    return SFTIME;
}
inline FieldType SFVec2f::getType()const{
    return SFVEC2F;
}
inline FieldType SFVec3f::getType()const{
    return SFVEC3F;
}
template<>
inline FieldType MFBool::getType()const{
    return MFBOOL;
}
template<>
inline FieldType MFColor::getType()const{
    return MFCOLOR;
}
template<>
inline FieldType MFFloat::getType()const{
    return MFFLOAT;
}
template<>
inline FieldType MFInt32::getType()const{
    return MFINT32;
}
template<>
inline FieldType MFNode::getType()const{
    return MFNODE;
}
template<>
inline FieldType MFRotation::getType()const{
    return MFROTATION;
}
template<>
inline FieldType MFString::getType()const{
    return MFSTRING;
}
template<>
inline FieldType MFTime::getType()const{
    return MFDOUBLE;
}
template<>
inline FieldType MFVec2f::getType()const{
    return MFVEC2F;
}
template<>
inline FieldType MFVec3f::getType()const{
    return MFVEC3F;
}

template<> inline void SFBool::resize(int newLen){throw this;}
inline void SFColor::resize(int newLen){throw this;}
template<> inline void SFFloat::resize(int newLen){throw this;}
template<> inline void SFInt32::resize(int newLen){throw this;}
inline void SFNode::resize(int newLen){throw this;}
inline void SFRotation::resize(int newLen){throw this;}
inline void SFString::resize(int newLen){throw this;}
template<> inline void SFTime::resize(int newLen){throw this;}
inline void SFVec2f::resize(int newLen){throw this;}
inline void SFVec3f::resize(int newLen){throw this;}

template<> inline void MFBool::resize(int newLen){refValue().length=newLen;}
template<> inline void MFColor::resize(int newLen){refValue().length=newLen;}
template<> inline void MFFloat::resize(int newLen){refValue().length=newLen;}
template<> inline void MFInt32::resize(int newLen){refValue().length=newLen;}
template<> inline void MFNode::resize(int newLen){refValue().length=newLen;}
template<> inline void MFRotation::resize(int newLen){refValue().length=newLen;}
template<> inline void MFString::resize(int newLen){refValue().length=newLen;}
template<> inline void MFTime::resize(int newLen){refValue().length=newLen;}
template<> inline void MFVec2f::resize(int newLen){refValue().length=newLen;}
template<> inline void MFVec3f::resize(int newLen){refValue().length=newLen;}

extern const char* mFieldErrorMsg;

class MField0 {
public:
    int size()const{return 0;}
    Field* operator [](const int)const{throw this;}
    const char* showReason() const{return mFieldErrorMsg;}
};
class MField1 {
public:
    MField1( Field* f0)
    {
        fieldArray[0]=f0;
    }
    Field* fieldArray[1];
    int size()const{return 1;}
    Field* operator [](const int par)const{if((unsigned)par>=(unsigned)1){	throw this;	}return fieldArray[par];}
    const char* showReason() const{return mFieldErrorMsg;}
};

class MField2 {
public:
    MField2( Field* f0, Field* f1)
    {
        fieldArray[0]=f0;
        fieldArray[1]=f1;
    }
    Field* fieldArray[2];
    int size()const{return 2;}
    Field* operator [](const int par)const{if((unsigned)par>=(unsigned)2){	throw this;	}return fieldArray[par];}
    const char* showReason() const{return mFieldErrorMsg;}
};
class MField3 {
public:
    MField3( Field* f0, Field* f1, Field* f2)
    {
        fieldArray[0]=f0;
        fieldArray[1]=f1;
        fieldArray[2]=f2;
    }
    Field* fieldArray[3];
    int size()const{return 3;}
    Field* operator [](const int par)const{if((unsigned)par>=(unsigned)3){	throw this;	}return fieldArray[par];}
    const char* showReason() const{return mFieldErrorMsg;}
};
class MField4 {
public:
    MField4(Field* f0, Field* f1, Field* f2, Field* f3)
    {
        fieldArray[0]=f0;
        fieldArray[1]=f1;
        fieldArray[2]=f2;
        fieldArray[3]=f3;
    }
    Field* fieldArray[4];
    int size()const{return 4;}
    Field* operator [](const int par)const{if((unsigned)par>=(unsigned)4){	throw this;	}return fieldArray[par];}
    const char* showReason() const{return mFieldErrorMsg;}
};
class MField5 {
public:
    MField5(Field* f0, Field* f1, Field* f2, Field* f3, Field* f4)
    {
        fieldArray[0]=f0;
        fieldArray[1]=f1;
        fieldArray[2]=f2;
        fieldArray[3]=f3;
        fieldArray[4]=f4;
    }
    Field* fieldArray[5];
    int size()const{return 5;}
    Field* operator [](const int par)const{if((unsigned)par>=(unsigned)5){	throw this;	}return fieldArray[par];}
    const char* showReason() const{return mFieldErrorMsg;}
};
class MField6 {
public:
    MField6(Field* f0, Field* f1, Field* f2, Field* f3, Field* f4, Field* f5)
    {
        fieldArray[0]=f0;
        fieldArray[1]=f1;
        fieldArray[2]=f2;
        fieldArray[3]=f3;
        fieldArray[4]=f4;
        fieldArray[5]=f5;
    }
    Field* fieldArray[6];
    int size()const{return 6;}
    Field* operator [](const int par)const{if((unsigned)par>=(unsigned)6){	throw this;	}return fieldArray[par];}
    const char* showReason() const{return mFieldErrorMsg;}
};
class MField7 {
public:
    MField7(Field* f0, Field* f1, Field* f2, Field* f3, Field* f4, Field* f5, Field* f6)
    {
        fieldArray[0]=f0;
        fieldArray[1]=f1;
        fieldArray[2]=f2;
        fieldArray[3]=f3;
        fieldArray[4]=f4;
        fieldArray[5]=f5;
        fieldArray[6]=f6;
    }
    Field* fieldArray[7];
    int size()const{return 7;}
    Field* operator [](const int par)const{if((unsigned)par>=(unsigned)7){	throw this;	}return fieldArray[par];}
    const char* showReason() const{return mFieldErrorMsg;}
};
class MField8 {
public:
    MField8(Field* f0, Field* f1, Field* f2, Field* f3, Field* f4, Field* f5, Field* f6, Field* f7)
    {
        fieldArray[0]=f0;
        fieldArray[1]=f1;
        fieldArray[2]=f2;
        fieldArray[3]=f3;
        fieldArray[4]=f4;
        fieldArray[5]=f5;
        fieldArray[6]=f6;
        fieldArray[7]=f7;
    }
    Field* fieldArray[8];
    int size()const{return 8;}
    Field* operator [](const int par)const{if((unsigned)par>=(unsigned)8){	throw this;	}return fieldArray[par];}
    const char* showReason() const{return mFieldErrorMsg;}
};
class MField9 {
public:
    MField9(Field* f0, Field* f1, Field* f2, Field* f3, Field* f4, Field* f5, Field* f6, Field* f7, Field* f8)
    {
        fieldArray[0]=f0;
        fieldArray[1]=f1;
        fieldArray[2]=f2;
        fieldArray[3]=f3;
        fieldArray[4]=f4;
        fieldArray[5]=f5;
        fieldArray[6]=f6;
        fieldArray[7]=f7;
        fieldArray[8]=f8;
    }
    Field* fieldArray[9];
    int size()const{return 9;}
    Field* operator [](const int par)const{if((unsigned)par>=(unsigned)9){	throw this;	}return fieldArray[par];}
    const char* showReason() const{return mFieldErrorMsg;}
};
class MField10 {
public:
    MField10(Field* f0, Field* f1, Field* f2, Field* f3, Field* f4, Field* f5, Field* f6, Field* f7, Field* f8, Field* f9)
    {
        fieldArray[0]=f0;
        fieldArray[1]=f1;
        fieldArray[2]=f2;
        fieldArray[3]=f3;
        fieldArray[4]=f4;
        fieldArray[5]=f5;
        fieldArray[6]=f6;
        fieldArray[7]=f7;
        fieldArray[8]=f8;
        fieldArray[9]=f9;
    }
    ~MField10()
    {
    };
    Field* fieldArray[10];
    int size()const{return 10;}
    Field* operator [](const int par)const{if((unsigned)par>=(unsigned)10){	throw this;	}return fieldArray[par];}
    const char* showReason() const{return mFieldErrorMsg;}
};
class MField11 {
public:
    MField11(Field* f0, Field* f1, Field* f2, Field* f3, Field* f4, Field* f5, Field* f6, Field* f7, Field* f8, Field* f9, Field* f10)
    {
        fieldArray[0]=f0;
        fieldArray[1]=f1;
        fieldArray[2]=f2;
        fieldArray[3]=f3;
        fieldArray[4]=f4;
        fieldArray[5]=f5;
        fieldArray[6]=f6;
        fieldArray[7]=f7;
        fieldArray[8]=f8;
        fieldArray[9]=f9;
        fieldArray[10]=f10;
    }
    Field* fieldArray[11];
    int size()const{return 11;}
    Field* operator [](const int par)const{if((unsigned)par>=(unsigned)11){	throw this;	}return fieldArray[par];}
    const char* showReason() const{return mFieldErrorMsg;}
};
class MField12 {
public:
    MField12(Field* f0, Field* f1, Field* f2, Field* f3, Field* f4, Field* f5, Field* f6, Field* f7, Field* f8, Field* f9, Field* f10, Field* f11)
    {
        fieldArray[0]=f0;
        fieldArray[1]=f1;
        fieldArray[2]=f2;
        fieldArray[3]=f3;
        fieldArray[4]=f4;
        fieldArray[5]=f5;
        fieldArray[6]=f6;
        fieldArray[7]=f7;
        fieldArray[8]=f8;
        fieldArray[9]=f9;
        fieldArray[10]=f10;
        fieldArray[11]=f11;
    }
    Field* fieldArray[12];
    int size()const{return 12;}
    Field* operator [](const int par)const{if((unsigned)par>=(unsigned)12){	throw this;	}return fieldArray[par];}
    const char* showReason() const{return mFieldErrorMsg;}
};
class MField13 {
public:
    MField13(Field* f0, Field* f1, Field* f2, Field* f3, Field* f4, Field* f5, Field* f6, Field* f7, Field* f8, Field* f9, Field* f10, Field* f11, Field* f12)
    {
        fieldArray[0]=f0;
        fieldArray[1]=f1;
        fieldArray[2]=f2;
        fieldArray[3]=f3;
        fieldArray[4]=f4;
        fieldArray[5]=f5;
        fieldArray[6]=f6;
        fieldArray[7]=f7;
        fieldArray[8]=f8;
        fieldArray[9]=f9;
        fieldArray[10]=f10;
        fieldArray[11]=f11;
        fieldArray[12]=f12;
    }
    Field* fieldArray[13];
    int size()const{return 13;}
    Field* operator [](const int par)const{if((unsigned)par>=(unsigned)13){	throw this;	}return fieldArray[par];}
    const char* showReason() const{return mFieldErrorMsg;}
};
class MField14 {
public:
    MField14(Field* f0, Field* f1, Field* f2, Field* f3, Field* f4, Field* f5, Field* f6, Field* f7, Field* f8, Field* f9, Field* f10, Field* f11, Field* f12, Field* f13)
    {
        fieldArray[0]=f0;
        fieldArray[1]=f1;
        fieldArray[2]=f2;
        fieldArray[3]=f3;
        fieldArray[4]=f4;
        fieldArray[5]=f5;
        fieldArray[6]=f6;
        fieldArray[7]=f7;
        fieldArray[8]=f8;
        fieldArray[9]=f9;
        fieldArray[10]=f10;
        fieldArray[11]=f11;
        fieldArray[12]=f12;
        fieldArray[13]=f13;
    }
    Field* fieldArray[14];
    int size()const{return 14;}
    Field* operator [](const int par)const{if((unsigned)par>=(unsigned)14){	throw this;	}return fieldArray[par];}
    const char* showReason() const{return mFieldErrorMsg;}
};
class MField15 {
public:
    MField15(Field* f0, Field* f1, Field* f2, Field* f3, Field* f4, Field* f5, Field* f6, Field* f7, Field* f8, Field* f9, Field* f10, Field* f11, Field* f12, Field* f13, Field* f14)
    {
        fieldArray[0]=f0;
        fieldArray[1]=f1;
        fieldArray[2]=f2;
        fieldArray[3]=f3;
        fieldArray[4]=f4;
        fieldArray[5]=f5;
        fieldArray[6]=f6;
        fieldArray[7]=f7;
        fieldArray[8]=f8;
        fieldArray[9]=f9;
        fieldArray[10]=f10;
        fieldArray[11]=f11;
        fieldArray[12]=f12;
        fieldArray[13]=f13;
        fieldArray[14]=f14;
    }
    Field* fieldArray[15];
    int size()const{return 15;}
    Field* operator [](const int par)const{if((unsigned)par>=(unsigned)15){	throw this;	}return fieldArray[par];}
    const char* showReason() const{return mFieldErrorMsg;}
};
class MField16 {
public:
    MField16(Field* f0, Field* f1, Field* f2, Field* f3, Field* f4, Field* f5, Field* f6, Field* f7, Field* f8, Field* f9, Field* f10, Field* f11, Field* f12, Field* f13, Field* f14, Field* f15)
    {
        fieldArray[0]=f0;
        fieldArray[1]=f1;
        fieldArray[2]=f2;
        fieldArray[3]=f3;
        fieldArray[4]=f4;
        fieldArray[5]=f5;
        fieldArray[6]=f6;
        fieldArray[7]=f7;
        fieldArray[8]=f8;
        fieldArray[9]=f9;
        fieldArray[10]=f10;
        fieldArray[11]=f11;
        fieldArray[12]=f12;
        fieldArray[13]=f13;
        fieldArray[14]=f14;
        fieldArray[15]=f15;
    }
    Field* fieldArray[16];
    int size()const{return 16;}
    Field* operator [](const int par)const{if((unsigned)par>=(unsigned)16){	throw this;	}return fieldArray[par];}
    const char* showReason() const{return mFieldErrorMsg;}
};
class MField17 {
public:
    MField17(Field* f0, Field* f1, Field* f2, Field* f3, Field* f4, Field* f5, Field* f6, Field* f7, Field* f8, Field* f9, Field* f10, Field* f11, Field* f12, Field* f13, Field* f14, Field* f15, Field* f16)
    {
        fieldArray[0]=f0;
        fieldArray[1]=f1;
        fieldArray[2]=f2;
        fieldArray[3]=f3;
        fieldArray[4]=f4;
        fieldArray[5]=f5;
        fieldArray[6]=f6;
        fieldArray[7]=f7;
        fieldArray[8]=f8;
        fieldArray[9]=f9;
        fieldArray[10]=f10;
        fieldArray[11]=f11;
        fieldArray[12]=f12;
        fieldArray[13]=f13;
        fieldArray[14]=f14;
        fieldArray[15]=f15;
        fieldArray[16]=f16;
    }
    Field* fieldArray[17];
    int size()const{return 17;}
    Field* operator [](const int par)const{if((unsigned)par>=(unsigned)17){	throw this;	}return fieldArray[par];}
    const char* showReason() const{return mFieldErrorMsg;}
};
class MField18 {
public:
    MField18(Field* f0, Field* f1, Field* f2, Field* f3, Field* f4, Field* f5, Field* f6, Field* f7, Field* f8, Field* f9, Field* f10, Field* f11, Field* f12, Field* f13, Field* f14, Field* f15, Field* f16, Field* f17)
    {
        fieldArray[0]=f0;
        fieldArray[1]=f1;
        fieldArray[2]=f2;
        fieldArray[3]=f3;
        fieldArray[4]=f4;
        fieldArray[5]=f5;
        fieldArray[6]=f6;
        fieldArray[7]=f7;
        fieldArray[8]=f8;
        fieldArray[9]=f9;
        fieldArray[10]=f10;
        fieldArray[11]=f11;
        fieldArray[12]=f12;
        fieldArray[13]=f13;
        fieldArray[14]=f14;
        fieldArray[15]=f15;
        fieldArray[16]=f16;
        fieldArray[17]=f17;
    }
    Field* fieldArray[18];
    int size()const{return 18;}
    Field* operator [](const int par)const{if((unsigned)par>=(unsigned)18){	throw this;	}return fieldArray[par];}
    const char* showReason() const{return mFieldErrorMsg;}
};
class MField19 {
public:
    MField19(Field* f0, Field* f1, Field* f2, Field* f3, Field* f4, Field* f5, Field* f6, Field* f7, Field* f8, Field* f9, Field* f10, Field* f11, Field* f12, Field* f13, Field* f14, Field* f15, Field* f16, Field* f17, Field* f18)
    {
        fieldArray[0]=f0;
        fieldArray[1]=f1;
        fieldArray[2]=f2;
        fieldArray[3]=f3;
        fieldArray[4]=f4;
        fieldArray[5]=f5;
        fieldArray[6]=f6;
        fieldArray[7]=f7;
        fieldArray[8]=f8;
        fieldArray[9]=f9;
        fieldArray[10]=f10;
        fieldArray[11]=f11;
        fieldArray[12]=f12;
        fieldArray[13]=f13;
        fieldArray[14]=f14;
        fieldArray[15]=f15;
        fieldArray[16]=f16;
        fieldArray[17]=f17;
        fieldArray[18]=f18;
    }
    Field* fieldArray[19];
    int size()const{return 19;}
    Field* operator [](const int par)const{if((unsigned)par>=(unsigned)19){	throw this;	}return fieldArray[par];}
    const char* showReason() const{return mFieldErrorMsg;}
};
class MField20{
public:
    MField20(Field* f0, Field* f1, Field* f2, Field* f3, Field* f4, Field* f5, Field* f6, Field* f7, Field* f8, Field* f9, Field* f10, Field* f11, Field* f12, Field* f13, Field* f14, Field* f15, Field* f16, Field* f17, Field* f18, Field* f19)
    {
        fieldArray[0]=f0;
        fieldArray[1]=f1;
        fieldArray[2]=f2;
        fieldArray[3]=f3;
        fieldArray[4]=f4;
        fieldArray[5]=f5;
        fieldArray[6]=f6;
        fieldArray[7]=f7;
        fieldArray[8]=f8;
        fieldArray[9]=f9;
        fieldArray[10]=f10;
        fieldArray[11]=f11;
        fieldArray[12]=f12;
        fieldArray[13]=f13;
        fieldArray[14]=f14;
        fieldArray[15]=f15;
        fieldArray[16]=f16;
        fieldArray[17]=f17;
        fieldArray[18]=f18;
        fieldArray[19]=f19;
    }
    Field* fieldArray[20];
    int size()const{return 20;}
    Field* operator [](const int par)const{if((unsigned)par>=(unsigned)20){	throw this;	}return fieldArray[par];}
    const char* showReason() const{return mFieldErrorMsg;}
};
class MField21{
public:
    MField21(Field* f0, Field* f1, Field* f2, Field* f3, Field* f4, Field* f5, Field* f6, Field* f7, Field* f8, Field* f9, Field* f10, Field* f11, Field* f12, Field* f13, Field* f14, Field* f15, Field* f16, Field* f17, Field* f18, Field* f19, Field* f20)
    {
        fieldArray[0]=f0;
        fieldArray[1]=f1;
        fieldArray[2]=f2;
        fieldArray[3]=f3;
        fieldArray[4]=f4;
        fieldArray[5]=f5;
        fieldArray[6]=f6;
        fieldArray[7]=f7;
        fieldArray[8]=f8;
        fieldArray[9]=f9;
        fieldArray[10]=f10;
        fieldArray[11]=f11;
        fieldArray[12]=f12;
        fieldArray[13]=f13;
        fieldArray[14]=f14;
        fieldArray[15]=f15;
        fieldArray[16]=f16;
        fieldArray[17]=f17;
        fieldArray[18]=f18;
        fieldArray[19]=f19;
        fieldArray[20]=f20;
    }
    Field* fieldArray[21];
    int size()const{return 21;}
    Field* operator [](const int par)const{if((unsigned)par>=(unsigned)21){	throw this;	}return fieldArray[par];}
    const char* showReason() const{return mFieldErrorMsg;}
};
class MField22 {
public:
    MField22(Field* f0, Field* f1, Field* f2, Field* f3, Field* f4, Field* f5, Field* f6, Field* f7, Field* f8, Field* f9, Field* f10, Field* f11, Field* f12, Field* f13, Field* f14, Field* f15, Field* f16, Field* f17, Field* f18, Field* f19, Field* f20, Field* f21)
    {
        fieldArray[0]=f0;
        fieldArray[1]=f1;
        fieldArray[2]=f2;
        fieldArray[3]=f3;
        fieldArray[4]=f4;
        fieldArray[5]=f5;
        fieldArray[6]=f6;
        fieldArray[7]=f7;
        fieldArray[8]=f8;
        fieldArray[9]=f9;
        fieldArray[10]=f10;
        fieldArray[11]=f11;
        fieldArray[12]=f12;
        fieldArray[13]=f13;
        fieldArray[14]=f14;
        fieldArray[15]=f15;
        fieldArray[16]=f16;
        fieldArray[17]=f17;
        fieldArray[18]=f18;
        fieldArray[19]=f19;
        fieldArray[20]=f20;
        fieldArray[21]=f21;
    }
    Field* fieldArray[22];
    int size()const{return 22;}
    Field* operator [](const int par)const{if((unsigned)par>=(unsigned)22){	throw this;	}return fieldArray[par];}
    const char* showReason() const{return mFieldErrorMsg;}
};
class MField23 {
public:
    MField23(Field* f0, Field* f1, Field* f2, Field* f3, Field* f4, Field* f5, Field* f6, Field* f7, Field* f8, Field* f9, Field* f10, Field* f11, Field* f12, Field* f13, Field* f14, Field* f15, Field* f16, Field* f17, Field* f18, Field* f19, Field* f20, Field* f21, Field* f22)
    {
        fieldArray[0]=f0;
        fieldArray[1]=f1;
        fieldArray[2]=f2;
        fieldArray[3]=f3;
        fieldArray[4]=f4;
        fieldArray[5]=f5;
        fieldArray[6]=f6;
        fieldArray[7]=f7;
        fieldArray[8]=f8;
        fieldArray[9]=f9;
        fieldArray[10]=f10;
        fieldArray[11]=f11;
        fieldArray[12]=f12;
        fieldArray[13]=f13;
        fieldArray[14]=f14;
        fieldArray[15]=f15;
        fieldArray[16]=f16;
        fieldArray[17]=f17;
        fieldArray[18]=f18;
        fieldArray[19]=f19;
        fieldArray[20]=f20;
        fieldArray[21]=f21;
        fieldArray[22]=f22;
    }
    Field* fieldArray[23];
    int size()const{return 23;}
    Field* operator [](const int par)const{if((unsigned)par>=(unsigned)23){	throw this;	}return fieldArray[par];}
    const char* showReason() const{return mFieldErrorMsg;}
};
class MField24 {
public:
    MField24(Field* f0, Field* f1, Field* f2, Field* f3, Field* f4, Field* f5, Field* f6, Field* f7, Field* f8, Field* f9, Field* f10, Field* f11, Field* f12, Field* f13, Field* f14, Field* f15, Field* f16, Field* f17, Field* f18, Field* f19, Field* f20, Field* f21, Field* f22, Field* f23)
    {
        fieldArray[0]=f0;
        fieldArray[1]=f1;
        fieldArray[2]=f2;
        fieldArray[3]=f3;
        fieldArray[4]=f4;
        fieldArray[5]=f5;
        fieldArray[6]=f6;
        fieldArray[7]=f7;
        fieldArray[8]=f8;
        fieldArray[9]=f9;
        fieldArray[10]=f10;
        fieldArray[11]=f11;
        fieldArray[12]=f12;
        fieldArray[13]=f13;
        fieldArray[14]=f14;
        fieldArray[15]=f15;
        fieldArray[16]=f16;
        fieldArray[17]=f17;
        fieldArray[18]=f18;
        fieldArray[19]=f19;
        fieldArray[20]=f20;
        fieldArray[21]=f21;
        fieldArray[22]=f22;
        fieldArray[23]=f23;
    }
    Field* fieldArray[24];
    int size()const{return 24;}
    Field* operator [](const int par)const{if((unsigned)par>=(unsigned)24){	throw this;	}return fieldArray[par];}
    const char* showReason() const{return mFieldErrorMsg;}
};
class MField25 {
public:
    MField25(Field* f0, Field* f1, Field* f2, Field* f3, Field* f4, Field* f5, Field* f6, Field* f7, Field* f8, Field* f9, Field* f10, Field* f11, Field* f12, Field* f13, Field* f14, Field* f15, Field* f16, Field* f17, Field* f18, Field* f19, Field* f20, Field* f21, Field* f22, Field* f23, Field* f24)
    {
        fieldArray[0]=f0;
        fieldArray[1]=f1;
        fieldArray[2]=f2;
        fieldArray[3]=f3;
        fieldArray[4]=f4;
        fieldArray[5]=f5;
        fieldArray[6]=f6;
        fieldArray[7]=f7;
        fieldArray[8]=f8;
        fieldArray[9]=f9;
        fieldArray[10]=f10;
        fieldArray[11]=f11;
        fieldArray[12]=f12;
        fieldArray[13]=f13;
        fieldArray[14]=f14;
        fieldArray[15]=f15;
        fieldArray[16]=f16;
        fieldArray[17]=f17;
        fieldArray[18]=f18;
        fieldArray[19]=f19;
        fieldArray[20]=f20;
        fieldArray[21]=f21;
        fieldArray[22]=f22;
        fieldArray[23]=f23;
        fieldArray[24]=f24;
    }
    Field* fieldArray[25];
    int size()const{return 25;}
    Field* operator [](const int par)const{if((unsigned)par>=(unsigned)25){	throw this;	}return fieldArray[par];}
    const char* showReason() const{return mFieldErrorMsg;}
};
class MField26 {
public:
    MField26(Field* f0, Field* f1, Field* f2, Field* f3, Field* f4, Field* f5, Field* f6, Field* f7, Field* f8, Field* f9, Field* f10, Field* f11, Field* f12, Field* f13, Field* f14, Field* f15, Field* f16, Field* f17, Field* f18, Field* f19, Field* f20, Field* f21, Field* f22, Field* f23, Field* f24, Field* f25)
    {
        fieldArray[0]=f0;
        fieldArray[1]=f1;
        fieldArray[2]=f2;
        fieldArray[3]=f3;
        fieldArray[4]=f4;
        fieldArray[5]=f5;
        fieldArray[6]=f6;
        fieldArray[7]=f7;
        fieldArray[8]=f8;
        fieldArray[9]=f9;
        fieldArray[10]=f10;
        fieldArray[11]=f11;
        fieldArray[12]=f12;
        fieldArray[13]=f13;
        fieldArray[14]=f14;
        fieldArray[15]=f15;
        fieldArray[16]=f16;
        fieldArray[17]=f17;
        fieldArray[18]=f18;
        fieldArray[19]=f19;
        fieldArray[20]=f20;
        fieldArray[21]=f21;
        fieldArray[22]=f22;
        fieldArray[23]=f23;
        fieldArray[24]=f24;
        fieldArray[25]=f25;
    }
    Field* fieldArray[26];
    int size()const{return 26;}
    Field* operator [](const int par)const{if((unsigned)par>=(unsigned)26){	throw this;	}return fieldArray[par];}
    const char* showReason() const{return mFieldErrorMsg;}
};
class MField27{
public:
    MField27(Field* f0, Field* f1, Field* f2, Field* f3, Field* f4, Field* f5, Field* f6, Field* f7, Field* f8, Field* f9, Field* f10, Field* f11, Field* f12, Field* f13, Field* f14, Field* f15, Field* f16, Field* f17, Field* f18, Field* f19, Field* f20, Field* f21, Field* f22, Field* f23, Field* f24, Field* f25, Field* f26)
    {
        fieldArray[0]=f0;
        fieldArray[1]=f1;
        fieldArray[2]=f2;
        fieldArray[3]=f3;
        fieldArray[4]=f4;
        fieldArray[5]=f5;
        fieldArray[6]=f6;
        fieldArray[7]=f7;
        fieldArray[8]=f8;
        fieldArray[9]=f9;
        fieldArray[10]=f10;
        fieldArray[11]=f11;
        fieldArray[12]=f12;
        fieldArray[13]=f13;
        fieldArray[14]=f14;
        fieldArray[15]=f15;
        fieldArray[16]=f16;
        fieldArray[17]=f17;
        fieldArray[18]=f18;
        fieldArray[19]=f19;
        fieldArray[20]=f20;
        fieldArray[21]=f21;
        fieldArray[22]=f22;
        fieldArray[23]=f23;
        fieldArray[24]=f24;
        fieldArray[25]=f25;
        fieldArray[26]=f26;
    }
    Field* fieldArray[27];
    int size()const{return 27;}
    Field* operator [](const int par)const{if((unsigned)par>=(unsigned)29){	throw this;	}return fieldArray[par];}
    const char* showReason() const{return mFieldErrorMsg;}
};
class MField28{
public:
    MField28(Field* f0, Field* f1, Field* f2, Field* f3, Field* f4, Field* f5, Field* f6, Field* f7, Field* f8, Field* f9, Field* f10, Field* f11, Field* f12, Field* f13, Field* f14, Field* f15, Field* f16, Field* f17, Field* f18, Field* f19, Field* f20, Field* f21, Field* f22, Field* f23, Field* f24, Field* f25, Field* f26, Field* f27)
    {
        fieldArray[0]=f0;
        fieldArray[1]=f1;
        fieldArray[2]=f2;
        fieldArray[3]=f3;
        fieldArray[4]=f4;
        fieldArray[5]=f5;
        fieldArray[6]=f6;
        fieldArray[7]=f7;
        fieldArray[8]=f8;
        fieldArray[9]=f9;
        fieldArray[10]=f10;
        fieldArray[11]=f11;
        fieldArray[12]=f12;
        fieldArray[13]=f13;
        fieldArray[14]=f14;
        fieldArray[15]=f15;
        fieldArray[16]=f16;
        fieldArray[17]=f17;
        fieldArray[18]=f18;
        fieldArray[19]=f19;
        fieldArray[20]=f20;
        fieldArray[21]=f21;
        fieldArray[22]=f22;
        fieldArray[23]=f23;
        fieldArray[24]=f24;
        fieldArray[25]=f25;
        fieldArray[26]=f26;
        fieldArray[27]=f27;
    }
    Field* fieldArray[28];
    int size()const{return 28;}
    Field* operator [](const int par)const{if((unsigned)par>=(unsigned)28){	throw this;	}return fieldArray[par];}
    const char* showReason() const{return mFieldErrorMsg;}
};
class MField29{
public:
    MField29(Field* f0, Field* f1, Field* f2, Field* f3, Field* f4, Field* f5, Field* f6, Field* f7, Field* f8, Field* f9, Field* f10, Field* f11, Field* f12, Field* f13, Field* f14, Field* f15, Field* f16, Field* f17, Field* f18, Field* f19, Field* f20, Field* f21, Field* f22, Field* f23, Field* f24, Field* f25, Field* f26, Field* f27, Field* f28)
    {
        fieldArray[0]=f0;
        fieldArray[1]=f1;
        fieldArray[2]=f2;
        fieldArray[3]=f3;
        fieldArray[4]=f4;
        fieldArray[5]=f5;
        fieldArray[6]=f6;
        fieldArray[7]=f7;
        fieldArray[8]=f8;
        fieldArray[9]=f9;
        fieldArray[10]=f10;
        fieldArray[11]=f11;
        fieldArray[12]=f12;
        fieldArray[13]=f13;
        fieldArray[14]=f14;
        fieldArray[15]=f15;
        fieldArray[16]=f16;
        fieldArray[17]=f17;
        fieldArray[18]=f18;
        fieldArray[19]=f19;
        fieldArray[20]=f20;
        fieldArray[21]=f21;
        fieldArray[22]=f22;
        fieldArray[23]=f23;
        fieldArray[24]=f24;
        fieldArray[25]=f25;
        fieldArray[26]=f26;
        fieldArray[27]=f27;
        fieldArray[28]=f28;
    }
    Field* fieldArray[29];
    int size()const{return 29;}
    Field* operator [](const int par)const{if((unsigned)par>=(unsigned)29){	throw this;	}return fieldArray[par];}
    const char* showReason() const{return mFieldErrorMsg;}
};
class MField30 {
public:
    MField30(Field* f0, Field* f1, Field* f2, Field* f3, Field* f4, Field* f5, Field* f6, Field* f7, Field* f8, Field* f9, Field* f10, Field* f11, Field* f12, Field* f13, Field* f14, Field* f15, Field* f16, Field* f17, Field* f18, Field* f19, Field* f20, Field* f21, Field* f22, Field* f23, Field* f24, Field* f25, Field* f26, Field* f27, Field* f28, Field* f29)
    {
        fieldArray[0]=f0;
        fieldArray[1]=f1;
        fieldArray[2]=f2;
        fieldArray[3]=f3;
        fieldArray[4]=f4;
        fieldArray[5]=f5;
        fieldArray[6]=f6;
        fieldArray[7]=f7;
        fieldArray[8]=f8;
        fieldArray[9]=f9;
        fieldArray[10]=f10;
        fieldArray[11]=f11;
        fieldArray[12]=f12;
        fieldArray[13]=f13;
        fieldArray[14]=f14;
        fieldArray[15]=f15;
        fieldArray[16]=f16;
        fieldArray[17]=f17;
        fieldArray[18]=f18;
        fieldArray[19]=f19;
        fieldArray[20]=f20;
        fieldArray[21]=f21;
        fieldArray[22]=f22;
        fieldArray[23]=f23;
        fieldArray[24]=f24;
        fieldArray[25]=f25;
        fieldArray[26]=f26;
        fieldArray[27]=f27;
        fieldArray[28]=f28;
        fieldArray[29]=f29;
    }
    Field* fieldArray[30];
    int size()const{return 30;}
    Field* operator [](const int par)const{if((unsigned)par>=(unsigned)30){	throw this;	}return fieldArray[par];}
    const char* showReason() const{return mFieldErrorMsg;}
};
class MField31 {
public:
    MField31(Field* f0, Field* f1, Field* f2, Field* f3, Field* f4, Field* f5, Field* f6, Field* f7, Field* f8, Field* f9, Field* f10, Field* f11, Field* f12, Field* f13, Field* f14, Field* f15, Field* f16, Field* f17, Field* f18, Field* f19, Field* f20, Field* f21, Field* f22, Field* f23, Field* f24, Field* f25, Field* f26, Field* f27, Field* f28, Field* f29, Field* f30)
    {
        fieldArray[0]=f0;
        fieldArray[1]=f1;
        fieldArray[2]=f2;
        fieldArray[3]=f3;
        fieldArray[4]=f4;
        fieldArray[5]=f5;
        fieldArray[6]=f6;
        fieldArray[7]=f7;
        fieldArray[8]=f8;
        fieldArray[9]=f9;
        fieldArray[10]=f10;
        fieldArray[11]=f11;
        fieldArray[12]=f12;
        fieldArray[13]=f13;
        fieldArray[14]=f14;
        fieldArray[15]=f15;
        fieldArray[16]=f16;
        fieldArray[17]=f17;
        fieldArray[18]=f18;
        fieldArray[19]=f19;
        fieldArray[20]=f20;
        fieldArray[21]=f21;
        fieldArray[22]=f22;
        fieldArray[23]=f23;
        fieldArray[24]=f24;
        fieldArray[25]=f25;
        fieldArray[26]=f26;
        fieldArray[27]=f27;
        fieldArray[28]=f28;
        fieldArray[29]=f29;
        fieldArray[30]=f30;
    }
    Field* fieldArray[31];
    int size()const{return 31;}
    Field* operator [](const int par)const{if((unsigned)par>=(unsigned)31){	throw this;	}return fieldArray[par];}
    const char* showReason() const{return mFieldErrorMsg;}
};
class MField32 {
public:
    MField32(Field* f0, Field* f1, Field* f2, Field* f3, Field* f4, Field* f5, Field* f6, Field* f7, Field* f8, Field* f9, Field* f10, Field* f11, Field* f12, Field* f13, Field* f14, Field* f15, Field* f16, Field* f17, Field* f18, Field* f19, Field* f20, Field* f21, Field* f22, Field* f23, Field* f24, Field* f25, Field* f26, Field* f27, Field* f28, Field* f29, Field* f30, Field* f31)
    {
        fieldArray[0]=f0;
        fieldArray[1]=f1;
        fieldArray[2]=f2;
        fieldArray[3]=f3;
        fieldArray[4]=f4;
        fieldArray[5]=f5;
        fieldArray[6]=f6;
        fieldArray[7]=f7;
        fieldArray[8]=f8;
        fieldArray[9]=f9;
        fieldArray[10]=f10;
        fieldArray[11]=f11;
        fieldArray[12]=f12;
        fieldArray[13]=f13;
        fieldArray[14]=f14;
        fieldArray[15]=f15;
        fieldArray[16]=f16;
        fieldArray[17]=f17;
        fieldArray[18]=f18;
        fieldArray[19]=f19;
        fieldArray[20]=f20;
        fieldArray[21]=f21;
        fieldArray[22]=f22;
        fieldArray[23]=f23;
        fieldArray[24]=f24;
        fieldArray[25]=f25;
        fieldArray[26]=f26;
        fieldArray[27]=f27;
        fieldArray[28]=f28;
        fieldArray[29]=f29;
        fieldArray[30]=f30;
        fieldArray[31]=f31;
    }
    Field* fieldArray[32];
    int size()const{return 32;}
    Field* operator [](const int par)const{if((unsigned)par>=(unsigned)32){	throw this;	}return fieldArray[par];}
    const char* showReason() const{return mFieldErrorMsg;}
};
class MField33 {
public:
    MField33(Field* f0, Field* f1, Field* f2, Field* f3, Field* f4, Field* f5, Field* f6, Field* f7, Field* f8, Field* f9, Field* f10, Field* f11, Field* f12, Field* f13, Field* f14, Field* f15, Field* f16, Field* f17, Field* f18, Field* f19, Field* f20, Field* f21, Field* f22, Field* f23, Field* f24, Field* f25, Field* f26, Field* f27, Field* f28, Field* f29, Field* f30, Field* f31, Field* f32)
    {
        fieldArray[0]=f0;
        fieldArray[1]=f1;
        fieldArray[2]=f2;
        fieldArray[3]=f3;
        fieldArray[4]=f4;
        fieldArray[5]=f5;
        fieldArray[6]=f6;
        fieldArray[7]=f7;
        fieldArray[8]=f8;
        fieldArray[9]=f9;
        fieldArray[10]=f10;
        fieldArray[11]=f11;
        fieldArray[12]=f12;
        fieldArray[13]=f13;
        fieldArray[14]=f14;
        fieldArray[15]=f15;
        fieldArray[16]=f16;
        fieldArray[17]=f17;
        fieldArray[18]=f18;
        fieldArray[19]=f19;
        fieldArray[20]=f20;
        fieldArray[21]=f21;
        fieldArray[22]=f22;
        fieldArray[23]=f23;
        fieldArray[24]=f24;
        fieldArray[25]=f25;
        fieldArray[26]=f26;
        fieldArray[27]=f27;
        fieldArray[28]=f28;
        fieldArray[29]=f29;
        fieldArray[30]=f30;
        fieldArray[31]=f31;
        fieldArray[32]=f32;
    }
    Field* fieldArray[33];
    int size()const{return 33;}
    Field* operator [](const int par)const{if((unsigned)par>=(unsigned)33){	throw this;	}return fieldArray[par];}
    const char* showReason() const{return mFieldErrorMsg;}
};

class MField34 {
public:
    MField34(Field* f0, Field* f1, Field* f2, Field* f3, Field* f4, Field* f5, Field* f6, Field* f7, Field* f8, Field* f9, Field* f10, Field* f11, Field* f12, Field* f13, Field* f14, Field* f15, Field* f16, Field* f17, Field* f18, Field* f19, Field* f20, Field* f21, Field* f22, Field* f23, Field* f24, Field* f25, Field* f26, Field* f27, Field* f28, Field* f29, Field* f30, Field* f31, Field* f32, Field* f33)
    {
        fieldArray[0]=f0;
        fieldArray[1]=f1;
        fieldArray[2]=f2;
        fieldArray[3]=f3;
        fieldArray[4]=f4;
        fieldArray[5]=f5;
        fieldArray[6]=f6;
        fieldArray[7]=f7;
        fieldArray[8]=f8;
        fieldArray[9]=f9;
        fieldArray[10]=f10;
        fieldArray[11]=f11;
        fieldArray[12]=f12;
        fieldArray[13]=f13;
        fieldArray[14]=f14;
        fieldArray[15]=f15;
        fieldArray[16]=f16;
        fieldArray[17]=f17;
        fieldArray[18]=f18;
        fieldArray[19]=f19;
        fieldArray[20]=f20;
        fieldArray[21]=f21;
        fieldArray[22]=f22;
        fieldArray[23]=f23;
        fieldArray[24]=f24;
        fieldArray[25]=f25;
        fieldArray[26]=f26;
        fieldArray[27]=f27;
        fieldArray[28]=f28;
        fieldArray[29]=f29;
        fieldArray[30]=f30;
        fieldArray[31]=f31;
        fieldArray[32]=f32;
        fieldArray[33]=f33;
    }
    Field* fieldArray[34];
    int size()const{return 34;}
    Field* operator [](const int par)const{if((unsigned)par>=(unsigned)34){	throw this;	}return fieldArray[par];}
    const char* showReason() const{return mFieldErrorMsg;}
};

class MField35 {
public:
    MField35(Field* f0, Field* f1, Field* f2, Field* f3, Field* f4, Field* f5, Field* f6, Field* f7, Field* f8, Field* f9, Field* f10, Field* f11, Field* f12, Field* f13, Field* f14, Field* f15, Field* f16, Field* f17, Field* f18, Field* f19, Field* f20, Field* f21, Field* f22, Field* f23, Field* f24, Field* f25, Field* f26, Field* f27, Field* f28, Field* f29, Field* f30, Field* f31, Field* f32, Field* f33, Field* f34)
    {
        fieldArray[0]=f0;
        fieldArray[1]=f1;
        fieldArray[2]=f2;
        fieldArray[3]=f3;
        fieldArray[4]=f4;
        fieldArray[5]=f5;
        fieldArray[6]=f6;
        fieldArray[7]=f7;
        fieldArray[8]=f8;
        fieldArray[9]=f9;
        fieldArray[10]=f10;
        fieldArray[11]=f11;
        fieldArray[12]=f12;
        fieldArray[13]=f13;
        fieldArray[14]=f14;
        fieldArray[15]=f15;
        fieldArray[16]=f16;
        fieldArray[17]=f17;
        fieldArray[18]=f18;
        fieldArray[19]=f19;
        fieldArray[20]=f20;
        fieldArray[21]=f21;
        fieldArray[22]=f22;
        fieldArray[23]=f23;
        fieldArray[24]=f24;
        fieldArray[25]=f25;
        fieldArray[26]=f26;
        fieldArray[27]=f27;
        fieldArray[28]=f28;
        fieldArray[29]=f29;
        fieldArray[30]=f30;
        fieldArray[31]=f31;
        fieldArray[32]=f32;
        fieldArray[33]=f33;
        fieldArray[34]=f34;
    }
    Field* fieldArray[35];
    int size()const{return 35;}
    Field* operator [](const int par)const{if((unsigned)par>=(unsigned)35){	throw this;	}return fieldArray[par];}
    const char* showReason() const{return mFieldErrorMsg;}
};

class MField36 {
public:
    MField36(Field* f0, Field* f1, Field* f2, Field* f3, Field* f4, Field* f5, Field* f6, Field* f7, Field* f8, Field* f9, Field* f10, Field* f11, Field* f12, Field* f13, Field* f14, Field* f15, Field* f16, Field* f17, Field* f18, Field* f19, Field* f20, Field* f21, Field* f22, Field* f23, Field* f24, Field* f25, Field* f26, Field* f27, Field* f28, Field* f29, Field* f30, Field* f31, Field* f32, Field* f33, Field* f34, Field* f35)
    {
        fieldArray[0]=f0;
        fieldArray[1]=f1;
        fieldArray[2]=f2;
        fieldArray[3]=f3;
        fieldArray[4]=f4;
        fieldArray[5]=f5;
        fieldArray[6]=f6;
        fieldArray[7]=f7;
        fieldArray[8]=f8;
        fieldArray[9]=f9;
        fieldArray[10]=f10;
        fieldArray[11]=f11;
        fieldArray[12]=f12;
        fieldArray[13]=f13;
        fieldArray[14]=f14;
        fieldArray[15]=f15;
        fieldArray[16]=f16;
        fieldArray[17]=f17;
        fieldArray[18]=f18;
        fieldArray[19]=f19;
        fieldArray[20]=f20;
        fieldArray[21]=f21;
        fieldArray[22]=f22;
        fieldArray[23]=f23;
        fieldArray[24]=f24;
        fieldArray[25]=f25;
        fieldArray[26]=f26;
        fieldArray[27]=f27;
        fieldArray[28]=f28;
        fieldArray[29]=f29;
        fieldArray[30]=f30;
        fieldArray[31]=f31;
        fieldArray[32]=f32;
        fieldArray[33]=f33;
        fieldArray[34]=f34;
        fieldArray[35]=f35;
    }
    Field* fieldArray[36];
    int size()const{return 36;}
    Field* operator [](const int par)const{if((unsigned)par>=(unsigned)36){	throw this;	}return fieldArray[par];}
    const char* showReason() const{return mFieldErrorMsg;}
};

enum NodeType{
    MESH,
	PROTODECLARE,
	PROTOINTERFACE,
	PROTOBODY,
};

class Mesh : public Node, public MField4 {
    public:
///////////////////////////
    int getType()const{return MESH;}
    int getFLen()const{return size();};
    Field* getField(const int i)const{return (*this)[i];}

    int getFieldGroupIndex(const int &i)const{return index[i];}
    void setFieldGroupIndex(const int &i, const int &val){index[i]=val;}
    M_Int* getIndexPtr(){return &index;}

    MFVec3f coord;
    MFVec3f normal;
    MFColor color;
    MFInt32 coordIndex;

    Mesh():
        MField4(
            &coord,
            &normal,
            &color,
            &coordIndex
        ),
        coord(M_Vec3f()),
        normal(M_Vec3f()),
        color(M_Color3f()),
        coordIndex(M_Int()),

/////
        render(0)
    {
       index.length=1;
    }
    M_Int index;
    Render* render;

    void convertToRenderFormat();
    void renderNode(Matrix parentMatrix, Vec3f parentColScale);
};

class DynamicMField {
public:
    DynamicMField():
        rawLen(0),
        pointer(0)
    {}
    DynamicMField(const DynamicMField& par):
        rawLen(par.rawLen),
        pointer(par.pointer)
    {
        for(int i=0;i<rawLen;i++)pointer[i]=par.pointer[i];
    }
    DynamicMField& resize(int newLength){
        if(newLength>rawLen){
            int i;
            Field** newPointer=new Field*[newLength];
            for(i=0;i<rawLen;i++){
                newPointer[i]=pointer[i];
            }
            delete[] pointer;
            pointer=newPointer;

        }
        rawLen=newLength;
        return *this;
    }
    DynamicMField& operator +=(DynamicMField& par){
        int i;
        int newLength=par.rawLen+rawLen;
        Field** newPointer=new Field*[newLength];
        for(i=0;i<rawLen;i++){
            newPointer[i]=pointer[i];
        }
        for(i=0;i<par.size();i++){
            newPointer[i+rawLen]=par[i];
        }
        delete pointer;

        rawLen=newLength;
        pointer=newPointer;
        return *this;
    }
    void operator +=(Field* par){
        int i;
        int newLength=rawLen+1;
        Field** newPointer=new Field*[newLength];
        for(i=0;i<rawLen;i++){
            newPointer[i]=pointer[i];
        }
        newPointer[rawLen]=par;
        delete[] pointer;

        rawLen=newLength;
        pointer=newPointer;
    }
    void subLength(int i){
        rawLen-=i;
    }
    DynamicMField& addLength(int par){
        int newLength=par+rawLen;
        if(newLength>rawLen){
            int i;
            Field** newPointer=new Field*[newLength];
            for(i=0;i<rawLen;i++){
                newPointer[i]=pointer[i];
            }
            delete[] pointer;
            pointer=newPointer;

        }
        rawLen=newLength;
        return *this;
    }
    void read(M_Properties &properties, const String& str,int& i);
    void read2(Field *&field, Properties& sprop, bool& wasMF, const String& str,int& i);
    void read(M_Properties &properties, const BIProto& binode, String& str);
	void read2(Field *&field, Properties& sprop, bool& wasMF, const BIField& bi);

    int size()const{
        return rawLen;
    }
    Field* operator [](const int par)const{
        if((unsigned)par>=(unsigned)rawLen){
            throw this;
        }
        return pointer[par];
    }
    int rawLen;
    Field** pointer;
    const char* showReason() const{return mFieldErrorMsg;}
};

enum{
    DO_NOT_OUTPUT	= 0x0001,
    DO_NOT_SAVE		= 0x0002,
    BLOCK_USE		= 0x0004,
    BLOCK_NEW		= 0x0008,
    SAME_AS_LAST	= 0x0010,
    DISABLE			= 0x0020,
    DISABLE_LEN		= 0x0040,
    UNREADABLE		= 0x0080,
    UNWRITEABLE		= 0x0100,
    DO_NOT_DISPLY_NAME	= 0x0200,
    DO_NOT_DISPLY_VALUE	= 0x0400,
    DO_NOT_RESIZE	= 0x0800,
    BLUE_DOT		= 0x1000,
    RED_DOT			= 0x2000,
    AXIS			= 0x4000,
    PUSH			= 0x8000,
    LINK			= 0x10000,
};

class Properties {
public:
    unsigned int flags;

    void setBit(unsigned char bit, bool state);
template <int T> void set_(const bool& state){
    if(state){
        flags |= T;
    }
    else{
        flags &= ~T;
    }
}
    unsigned int doNotOutput()const{
        return flags & DO_NOT_OUTPUT;
    }
    unsigned int doNotSave()const{
        return flags & DO_NOT_SAVE;
    }
    unsigned int blockUse()const{
        return flags & BLOCK_USE;
    }
    unsigned int blockNew()const{
        return flags & BLOCK_NEW;
    }
    unsigned int sameAsLast()const{
        return flags & SAME_AS_LAST;
    }
    unsigned int doNotDisplayName()const{
        return flags & DO_NOT_DISPLY_NAME;
    }
    unsigned int doNotDisplayValue()const{
        return flags & DO_NOT_DISPLY_VALUE;
    }
    unsigned int doNotResize()const{
        return flags & DO_NOT_RESIZE;
    }
    unsigned int disable()const{
        return flags & DISABLE;
    }
    unsigned int disableLen()const{
        return flags & DISABLE_LEN;
    }
    unsigned int unreadable()const{
        return flags & UNREADABLE;
    }
    unsigned int unwritable()const{
        return flags & UNWRITEABLE;
    }
    unsigned int blueDot()const{
        return flags & BLUE_DOT;
    }
    unsigned int retDot()const{
        return flags & RED_DOT;
    }
    unsigned axis()const{
        return flags & AXIS;
    }
    unsigned push()const{
        return flags & PUSH;
    }
    unsigned link()const{
        return flags & LINK;
    }


//void set_inputOutput(const bool& state){ //does nothing
    //set_<0xffffffff ^ (UNREADABLE | UNWRITEABLE)>(state);
//}

void set_doNotOutput(const bool& state){
    set_<DO_NOT_OUTPUT>(state);
}
void set_doNotSave(const bool& state){
    set_<DO_NOT_SAVE>(state);
}
void set_blockUse(const bool& state){
    set_<BLOCK_USE>(state);
}
void set_blockNew(const bool& state){
    set_<BLOCK_NEW>(state);
}
void set_sameAsLast(const bool& state){
    set_<SAME_AS_LAST>(state);
}
void set_disable(const bool& state){
    set_<DISABLE>(state);
}
void set_disableLen(const bool& state){
    set_<DISABLE_LEN>(state);
}
void set_unreadable(const bool& state){
    set_<UNREADABLE>(state);
}
void set_unwritable(const bool& state){
    set_<UNWRITEABLE>(state);
}
void set_blueDot(const bool& state){
    set_<BLUE_DOT>(state);
}
void set_axis(const bool& state){
    set_<AXIS>(state);
}
void set_push(const bool& state){
    set_<PUSH>(state);
}
void set_link(const bool& state){
    set_<LINK>(state);
}

    M_Int promptNode;
    int groupIndex;
    int startAtIndex;
    int	initElLen;		//initial element length
    int	incElLen;	//increment element length
    double rangeFrom;
    double rangeTo;
    const char* tip;
    String name;
    String initElVal;
    String incElVal;
    Properties():
        groupIndex(-1),	//-1 = does not belong to a group
        startAtIndex(0),
        flags(0),
        initElLen(1),
        incElLen(1),
        rangeFrom(-2147483647),
        rangeTo(2147483647),
        tip(0)
    {
    }
    Properties(String i_name):
        name(i_name),
        groupIndex(-1),	//-1 = does not belong to a group
        startAtIndex(0),
        flags(0),
        initElLen(1),
        incElLen(1),
        rangeFrom(-2147483647),
        rangeTo(2147483647),
        tip(0)
    {
    }
    Properties(int)://promptNode ?
        groupIndex(-1),	//-1 = does not belong to a group
        startAtIndex(0),
        flags(0),
        initElLen(1),
        incElLen(1),
        rangeFrom(-2147483647),
        rangeTo(2147483647),
        tip(0)
    {
        //promptNode.resize(5);
    }
    Properties(const String& i_name, const int& groupIndex, const int& startAtIndex, const int& flags, const int& initElLen, const int& incElLen, const double& rangeFrom, const double& rangeTo, const char* tip=0):
        name(i_name),
        groupIndex(groupIndex),	//-1 = does not belong to a group
        startAtIndex(startAtIndex),
        flags(flags),
        initElLen(initElLen),
        incElLen(incElLen),
        rangeFrom(rangeFrom),
        rangeTo(rangeTo),
        tip(tip)
    {
    }
    ~Properties(){
    }
    void operator = (const Properties& par){
        name=par.name;
        groupIndex=par.groupIndex;
        startAtIndex=par.startAtIndex;
        flags=par.flags;
        initElLen=par.initElLen;
        incElLen=par.incElLen;
        initElVal=par.initElVal;
        incElVal=par.incElVal;
        rangeFrom=par.rangeFrom;
        rangeTo=par.rangeTo;
        promptNode=par.promptNode;
        tip=par.tip;
    }
    void read( Field& field, const String& str,int& i, bool& beginGroup, bool& endGroup);

    bool isReadable()const{
        return !(flags & UNREADABLE);
    };
    bool isWritable()const{
        return !(flags & UNWRITEABLE);
    };
    bool inputOutput()const{
        return !((flags & UNREADABLE) || (flags & UNWRITEABLE));
    }
    bool initializeOnly()const{
        return ((flags & UNREADABLE) && (flags & UNWRITEABLE));
    }
    bool inputOnly()const{
        return ((flags & UNREADABLE) && !(flags & UNWRITEABLE));
    }
    bool outputOnly()const{
        return (!(flags & UNREADABLE) && (flags & UNWRITEABLE));
    }
};


class Progenitor{
public:
    MFNode& children;
    Progenitor(MFNode& i_children):
        children(i_children)
    {}
    virtual ~Progenitor()
    {
    }
};

class ProtoInstance : public Node, public DynamicMField {
    public:
    int userNodeType;
    int getType()const{return userNodeType;}
    int getFLen()const{return size();};
    Field* getField(const int i)const{return (*this)[i];}
#ifdef SEAMLESS3DGUI
    int getFieldGroupIndex(const int &i)const{return index[i];}
    void setFieldGroupIndex(const int &i, const int &val){index[i]=val;}
    M_Int* getIndexPtr(){return &index;}
    M_Int index;
#endif

    ProtoInstance(const int userNodeType);
    void toString(String& str, bool newList=true);
    void toXMLString(String& str, bool newList, const char* containerField);
};

class ProtoInterface : public Node, public DynamicMField {
    public:
    int getType()const{return PROTOINTERFACE;}
    int getFLen()const{return size();};
    Field* getField(const int i)const{return (*this)[i];}
#ifdef SEAMLESS3DGUI
    int getFieldGroupIndex(const int &i)const{return index[i];}
    void setFieldGroupIndex(const int &i, const int &val){index[i]=val;}
    M_Int* getIndexPtr(){return &index;}
#endif


    ProtoInterface()
    {
#ifdef SEAMLESS3DGUI
        index.length=20;
#endif
    }
#ifdef SEAMLESS3DGUI
    M_Int index;
#endif
    M_Properties properties;

    void getFields(const String& str, int& i);

    const M_Properties& getProperties()const{
        return properties;
    }
    void toCPPString(String& str, bool newList=true);
};
class ProtoBody : public Node, public Progenitor, public MField1 {
    public:
    int getType()const{return PROTOBODY;}
    int getFLen()const{return size();};
    Field* getField(const int i)const{return (*this)[i];}
    int getFieldGroupIndex(const int &)const{throw this;}
    void setFieldGroupIndex(const int &, const int &){throw this;}
    M_Int* getIndexPtr(){return nullPtr;}


    MFNode children;

    ProtoBody():
        Progenitor(children),
        MField1(
            &children
        ),
        children(M_Node())
    {
    }
};

class ProtoDeclare : public Node, public MField2 {
    public:
    class SFProtoInterface : public SFNode {
    public:

        SFProtoInterface(const NodePtr& value):
            SFNode(value)
        {};
        ProtoInterface*& operator =(ProtoInterface*& par){
            refValue()=par;
            return par;
        }
        ProtoInterface* operator ->()const{
            return dynamic_cast<ProtoInterface*>(refValue().getRawPtr());
        }
        operator ProtoInterface*(){
            return dynamic_cast<ProtoInterface*>(refValue().getRawPtr());
        }
        operator ProtoInterfacePtr(){
            return dynamic_cast<ProtoInterface*>(refValue().p);
        }
    };
    class SFProtoBody : public SFNode {
    public:

        SFProtoBody(const NodePtr& value):
            SFNode(value)
        {};
        ProtoBody*& operator =(ProtoBody*& par){
            refValue()=par;
            return par;
        }
        ProtoBody* operator ->()const{
            return dynamic_cast<ProtoBody*>(refValue().getRawPtr());
        }
    };

//////////////////
    int getType()const{return PROTODECLARE;}
    int getFLen()const{return size();};
    Field* getField(const int i)const{return (*this)[i];}
    int getFieldGroupIndex(const int &)const{throw this;}
    void setFieldGroupIndex(const int &, const int &){throw this;}
    M_Int* getIndexPtr(){return nullPtr;}


    SFProtoInterface protoInterface;
    SFProtoBody protoBody;

    ProtoDeclare():
        MField2(
            &protoInterface,
            &protoBody
        ),

        protoInterface(0),
        protoBody(0),

        lookupDeclarationsIndex(0)
    {
    }
    int lookupDeclarationsIndex;
    void renderNode(Matrix parentMatrix, Vec3f parentColScale);
    void toString(String& str, bool newList=true);
    void getFields(const String& str, int& i);
};

inline SFNode::~SFNode(){
    if(p!=0&&p->decRefCount()==0){
        delete p;
    }
}
inline FD<NodePtr,Single>::FD(const NodePtr& par):
    p(par.getRawPtr())
{
    if(p!=0){
        p->incRefCount();
    }
};

inline Node* SFNode::operator = ( const int par1){
 if(par1 != 0)throw this;
    if(p != 0){
        if(p->decRefCount() == 0){
            delete p;
        }
    }
    p=NULL;
    return p;
}
inline Node* SFNode::operator = ( Node* par1){
    par1->incRefCount();
    if(p!=NULL){
        if(p->decRefCount()==0){
            delete p;
        }
    }
    p=par1;
    return p;
}
inline Node* SFNode::operator = ( SFNode& par){
    Node* parVal=&*(par.refValue());
    parVal->incRefCount();
    if(p!=NULL){
        if(p->decRefCount()==0){
            delete p;
        }
    }
    p=parVal;
    return p;
}
